; conv/asm - kjw/bqsd - 08/82
;
	TITLE	'<DosPLUS II.A - convert utility>'
;
SETBRK	EQU	03		;setup break processor
EVAL	EQU	83		;evaluate user input
VIDKEY	EQU	12		;video/keyboard
ERROR	EQU	39		;display error
WILD	EQU	51		;wild parser
VDREAD	EQU	11		;read video
VDGRAF	EQU	10		;write video
VDCHAR	EQU	08		;char to video
OPEN	EQU	40		;open file
CLOSE	EQU	42		;close file
FEXT	EQU	88		;add default extension
DIRWR	EQU	44		;direct sector write
DIRRD	EQU	35		;direct sector read
;
SVC	EQU	08H		;SVC processor
DISKIO	EQU	10H		;disk I/O processor
;
IOBUFF	EQU	1A00H		;I/O buffer
LOCDCT	EQU	032DH		;load IY with DCT
REGSAVE	EQU	0303H		;save registers
UCASE	EQU	031EH		;char to upper case
LOMEM	EQU	0171H		;low memory
HIMEM	EQU	0173H		;high memory
DIRRED	EQU	0324H		;lfn read
DIRWRT	EQU	0327H		;lfn write
;
DREAD	EQU	3		;data read
SREAD	EQU	6		;system read
DWRIT	EQU	5		;data write
;
ERR26	EQU	26		;dir space full
ERR27	EQU	27		;disk space full
ERR30	EQU	30		;directory full
ERR32	EQU	32		;invalid drive
ERR47	EQU	47		;required param
ERR48	EQU	48		;invalid param
ERR82	EQU	82		;insufficient memory
ERR86	EQU	86		;terminated
ERR89	EQU	89		;invalid I/O channel
ERR92	EQU	92		;no extent terminator
ERR97	EQU	97		;can't establish disk
ERR98	EQU	98		;illegal data range
;
CR	EQU	13
ETX	EQU	3
;
	ORG	2200H		;high overlay area
;
ENTRY	LD	(STACK),SP	;save the stack
	PUSH	HL		;save input pointer
;
	LD	HL,0		;turn off break processor
	LD	A,SETBRK	;SVC #
	RST	SVC		;turn it off
	LD	(OLDBRK),HL	;save old vector
;
	LD	HL,HELLO	;sign on message
	CALL	LINE$		;display it
;
	POP	HL		;restore pointer
;
SCAN	LD	IX,EBLOCK	;eval block
	LD	A,EVAL		;SVC #
	RST	SVC		;evaluate user input
	JR	NZ,POSTE	;error, abort!
;
	LD	A,(IX)		;get eval flags
	AND	7		;any input?
	JR	NZ,PERFORM	;go to it!
;
;	prompt for parameters
;
APARMS	LD	HL,PROMPT	;prompt string
	LD	DE,STRING	;input buffer
	LD	BC,1<8+79	;disp len + key len
	LD	A,VIDKEY	;SVC #
	RST	SVC		;display, get keys
	EX	DE,HL		;swap reg's
	JR	NZ,POSTE	;error, go!
	JR	NC,SCAN		;not break, check input
;
TERMIN	LD	A,ERR86		;program terminated
;
POSTE	AND	7FH		;assure a return
	LD	B,A		;pass error code
POSTEB	LD	A,ERROR		;SVC #
	RST	SVC		;display error
;
EXIT	CALL	FIXDCT		;restore DCT
	LD	HL,0		;get old break vector
OLDBRK	EQU	$-2
	LD	A,SETBRK	;SVC #
	RST	SVC		;put it back
	LD	SP,0		;reset stack
STACK	EQU	$-2
	XOR	A		;return ZERO
	RET			;back to caller
;
PERFORM	BIT	2,A		;source field here?
	LD	B,ERR47		;required param
	JR	Z,POSTEB	;go error if not here
;
	BIT	1,A		;dest field here?
	JR	NZ,PERFOR	;yes, go!
;
;	no dest device, operation MUST be DIR/CAT
;
	LD	A,(DIRPAR)	;get param
	INC	A		;is it?
	JP	NZ,POSTEB	;nope, error!
;
PERFOR	LD	A,(EBLOCK)	;get flags again
	BIT	0,A		;wildmask?
	LD	HL,MDCB+1	;mask DCB
	JR	NZ,SETMASK	;yes, setup mask
;
	LD	HL,SDCB		;source DCB
	LD	A,(HL)		;get flags
	INC	HL		;normal DCB
	BIT	7,A		;device here?
	JR	Z,POSTEB	;required!
	BIT	6,A		;filespec here?
	JR	Z,DUMMASK	;dummy mask
FINTRM	LD	A,(HL)		;get a char
	CP	':'		;drive?
	JR	Z,FIXTRM	;yes, go!
	CP	ETX		;terminator?
	JR	Z,FIXTRM	;yes, go!
	INC	HL		;bump pointer
	JR	FINTRM		;continue
FIXTRM	LD	(HL),ETX	;force term
	LD	HL,SDCB+1	;reset it
	JR	SETMASK		;setup the mask
;
;	no filespec or wildmask, dummy wildmask
;
DUMMASK	LD	HL,MDCB+1	;mask DCB
	LD	(HL),'!'	;all files
;
;	HL => wildmask to use
;
SETMASK	LD	B,0		;setup wild mask
	LD	A,WILD		;SVC #
	RST	SVC		;load it
	JR	NZ,POSTE	;error, abort!
;
;	setup memory buffer pointers
;
	LD	DE,(LOMEM)	;get low memory
	LD	HL,ZZZZZ	;end of program
	XOR	A		;clear carry
	SBC	HL,DE		;compare
	JR	C,$+5		;use low memory
	LD	DE,ZZZZZ	;else end of program
	INC	D		;next page
	LD	E,A		;first even page
	LD	(BUFFER),DE	;save buffer start
	LD	HL,(HIMEM)	;get high memory
	LD	L,0		;end of last even page
	DEC	H		;back another page
	LD	(SPBUFF),HL	;special buffer
	DEC	H		;last full page
	LD	(SQBUFF),HL	;special buffer
	DEC	H
	LD	(BUFEND),HL	;save buffer end
	DEC	H		;min 1 sector buffer
	OR	A		;clear carry
	SBC	HL,DE		;compare buffers
	LD	A,ERR82		;insufficient memory
	JP	C,POSTE		;not enough to do it
;
;	check for dest drive
;
	LD	A,(DIRPAR)	;get dir param
	INC	A		;dir only?
	JR	Z,CONTPER	;yes, go!
	LD	A,(DDCB)	;dest DCB
	BIT	7,A		;device here?
	LD	B,ERR47		;invalid param
	JP	Z,POSTEB	;no device!
	LD	B,ERR48		;invalid param
	BIT	6,A		;filespec here?
	JP	NZ,POSTEB	;no rename on file!
;
;	mask all set, setup DCT for transfer
;
CONTPER	CALL	SETDCT		;setup for TRSDOS
;
;	setup loop to run through the files
;
	XOR	A		;load zero
	LD	(DEC),A		;save logical file #
	LD	HL,(SPBUFF)	;disk I/O buffer
	LD	A,(FILES)	;get sector files start
	LD	E,A		;pass it
	JR	NEXSEC		;continue
;
LOOPER	CALL	CONVERT		;copy this file
;
	LD	A,0		;get current DEC
DEC	EQU	$-1
	INC	A		;bump it
	LD	(DEC),A		;update!
	LD	A,0		;offset to next entry
ENTLEN	EQU	$-1
	ADD	A,L		;add it in
	LD	L,A		;put it back
	JR	NZ,LOOPER	;finish this sector
;
;	check for disk end
;
	LD	IY,(SDCT)	;get DCT
	LD	A,(IY+17)	;get last sector to read
	INC	E		;bump sector
	CP	E		;done?
	JP	Z,EXIT		;yes, go!
;
NEXSEC	LD	IY,0		;get source DCT pointer
SDCT	EQU	$-2
	LD	D,(IY+18)	;load dir track
	LD	B,0		;extended sector
	LD	C,0		;get source drive
SOURCE	EQU	$-1
	CALL	DISKRD		;read directory sector
	JR	LOOPER		;go next sector
;
;	setup DCTS for transfer
;
SETDCT	LD	A,(SDCB)	;get source drive
	AND	0FH		;get device number
	SUB	8		;adjust to 0-7
	LD	B,ERR32		;invalid drive
	JP	C,POSTEB	;go error if @device
	LD	(SOURCE),A	;save source drive
;
	LD	A,(DIRPAR)	;dir only?
	INC	A
	JR	Z,SKPDEST	;yes, skip dest setup
;
	LD	A,(DDCB)	;get dest drive
	AND	0FH		;get device number
	SUB	8		;adjust for 0-7
	JP	C,POSTEB	;go error if @ type
	LD	(DEST),A	;save dest drive
;
SKPDEST	LD	A,(BACPAR)	;dest = trsdos?
	INC	A		;yes?
	JR	Z,DESTRS	;yes, go!
;
;	source is TRSDOS, dest is DosPLUS
;
	LD	A,40H		;entry length
	LD	(ENTLEN),A	;save it
	LD	A,(DIRPAR)	;dir only?
	INC	A
	JR	Z,SET1		;yes, continue
	LD	A,(DEST)	;get dest drive
	CALL	SETDOSP		;set it up
	LD	(DDCT),IY	;save dest DCT
SET1	LD	A,(SOURCE)	;get source drive
	CALL	SETTRS		;set it up
	LD	(SDCT),IY	;save source DCT
	RET			;done
;
;	source is DosPLUS, dest is TRSDOS
;
DESTRS	LD	A,20H		;entry length DosPLUS
	LD	(ENTLEN),A	;save it
	LD	A,(SOURCE)	;get source drive
	CALL	SETDOSP		;set it up
	LD	(SDCT),IY	;save DCT pointer
	LD	A,(DIRPAR)	;dir only?
	INC	A
	RET	Z		;yes, skip dest setup
	LD	A,(DEST)	;get dest drive
	CALL	SETTRS		;set it up
	LD	(DDCT),IY	;save DCT
	RET			;done
;
;	save original DCT
;
SETTRS	LD	C,A		;save drive #
	LD	(DCTDRV),A	;drive # to fix
	CALL	LOCDCT		;load IY with DCT
	BIT	5,(IY+5)	;rigid drive?
	LD	A,ERR89		;invalid I/O channel
	JP	NZ,POSTE
	LD	A,C		;get drive # back
	PUSH	IY		;save start
	POP	HL		;HL => DCT
	LD	DE,DCTSAV	;storage for it
MOVIT	LD	BC,23		;length of DCT
	LDIR			;save it!
	LD	C,A		;put back drive #
;
;	adjust DCT to read TRSDOS disk
;
	LD	A,-1		;set DCT as corrupt!
	LD	(CFLAG),A	;save it
	LD	A,(IY+5)	;get flags
	AND	18H		;bits 4/3 only
	LD	(IY+5),A	;put 'em back
	XOR	A		;load zero
	LD	(IY+7),A	;head offset
	LD	(IY+8),A	;cyl offset
	LD	(IY+9),A	;msb cyl offset
	LD	(IY+13),A	;set single density
	LD	(IY+22),A	;msb sectors/cyl
	LD	(IY+10),A	;sector offset
	INC	A		;load one
	LD	(IY+14),A	;surface count
	LD	(IY+15),77	;cyl count
	LD	(IY+16),26	;sectors/track
	LD	(IY+17),25	;dir length
	LD	(IY+19),5	;sects/gran
	LD	(IY+20),5	;grans/cyl
	LD	(IY+21),26	;lsb sects/cyl
;
;	read boot to find directory
;
	LD	D,0		;track zero
	LD	B,D		;extended sector
	LD	E,2		;sector 2
	LD	HL,IOBUFF	;I/O buffer
	CALL	DISKRD		;read it!
;
	LD	A,(HL)		;get directory track
	LD	(IY+18),A	;save dir track
	LD	D,A		;pass track
	INC	L		;version number
	LD	A,(HL)		;get it
	CP	2		;2.0?
	JR	Z,NOTVER	;yes, go!
	CP	1		;1.0?
NOTVER	LD	(VERSION),A	;save version #
	PUSH	AF		;save version
	LD	A,ERR97		;can't do this disk
	JP	NZ,POSTE	;go error!
	POP	AF		;get version
	CP	1		;1.0?
	JR	NZ,VER2		;nope, go!
	LD	A,2		;hit table
	LD	(HIT),A		;save it
	LD	A,3		;start of files
	LD	(FILES),A	;save it
;
VER2	SET	7,(IY+13)	;set double density
	DEC	L		;begin of buffer again
	DEC	E		;sector one
	CALL	DISKRD		;read system sector
	LD	L,0ADH		;point to dir track byte
	LD	A,(VERSION)	;get version
	CP	2		;2.0?
	JR	Z,VER3		;yes, go!
	LD	L,9AH		;1.0
VER3	LD	A,(IY+18)	;get dir track
	CP	(HL)		;on the directory?
	JR	NZ,NOTVER	;can't find directory
	INC	L		;next byte
	LD	A,(VERSION)	;get version
	CP	(HL)		;same?
	JR	NZ,NOTVER	;not this version
	RET			;OK, DCT all set to go!
;
;	setup for DosPLUS
;
SETDOSP	LD	C,A		;pass drive #
	CALL	LOCDCT		;find the DCT
	LD	E,0		;sector zero
	LD	B,E		;extended sector
	LD	HL,IOBUFF	;I/O buffer
	LD	A,SREAD		;system read
	RST	DISKIO		;read GAT table
	JP	NZ,POSTE	;error, abort!
	RET			;else DCT updated!
;
;	restore original DCT
;
FIXDCT	LD	A,0		;get corrupt DCT flag
CFLAG	EQU	$-1
	INC	A		;corrupt?
	RET	NZ		;nope, do nothing!
;
	LD	C,0		;get drive # to fix
DCTDRV	EQU	$-1
	CALL	LOCDCT		;load IY with DCT
	PUSH	IY		;pass to DE
	POP	DE
	LD	HL,DCTSAV	;where it is stored
	LD	BC,23		;length
	LDIR			;put it back
	RET			;done
;
	PAGE
;
CONVERT	PUSH	HL		;pass to IX
	POP	IX
	LD	A,(BACPAR)	;dest = trsdos?
	INC	A		;FF true?
	JR	NZ,CONVD	;yes, source = TRSDOS
;
;	source is DOSPLUS, check for active file
;
	BIT	4,(HL)		;active file?
	RET	Z		;nope, skip it
	BIT	7,(HL)		;extension?
	RET	NZ		;yes, skip it
	BIT	6,(HL)		;system file?
	RET	NZ		;yes, skip it!
	LD	A,(IX+19)	;get MSB EOF
	OR	A		;extended file?
	RET	NZ		;yes, can't copy it!
	JR	COPY		;valid file, go!
;
;	source is TRSDOS, check for active file
;
CONVD	LD	A,(IX+63)	;get file flag
	OR	A		;active file?
	RET	NZ		;nope, next one
	LD	A,(IX+5)	;any filespec?
	CP	' '		;blank?
	RET	Z		;yes, nil!
;
;	check for SYS option
;
	LD	A,(SYSPAR)	;system too?
	INC	A
	JR	Z,COPY		;yes, go!
	BIT	7,(IX)		;system file?
	RET	NZ		;yes, skip it!
;
;	OK, HL => active entry to copy
;
COPY	CALL	REGSAVE		;save all pointers
	LD	A,L		;get LSB file pointer
	ADD	A,5		;offset to name
	LD	L,A		;HL => name field
;
;	compare to mask
;
	LD	B,3		;command
	LD	A,WILD		;SVC #
	RST	SVC		;compare
	RET	NZ		;nope! skip it!
	LD	DE,MDCB		;re-use this area
	LD	B,2		;command
	LD	A,WILD		;SVC #
	RST	SVC		;combine name/extension
	EX	DE,HL		;HL => filespec
;
	LD	A,(DIRPAR)	;directory?
	INC	A		;FF true?
	JP	Z,SHOWFIL	;yes, display filespec
;
	LD	A,(QUEPAR)	;query for copy?
	INC	A		;yes?
	JR	NZ,CKECHO	;nope, check for echo
;
QUERY	LD	HL,MDCB		;stored filespec
	CALL	LINE$		;display filename
	LD	HL,QUEMSG	;query prompt
	LD	BC,QUEMSGL<8+1	;length + key length
	LD	DE,STRING	;input buffer
	LD	A,VIDKEY	;SVC #
	RST	SVC		;fetch the key
	JP	C,TERMIN	;abort on break!
	LD	A,(DE)		;fetch a key
	CALL	UCASE		;convert to upper case
	CP	'N'		;don't copy?
	RET	Z		;nope, skip this file
	CP	'Y'		;yes?
	JR	NZ,QUERY	;invalid, ask again
	JR	GOCOPY		;copy it!
;
CKECHO	LD	A,(ECHPAR)	;get echo param
	INC	A		;display it?
	JR	NZ,GOCOPY	;just copy it!
	CALL	LINE$		;display filespec
	LD	A,CR		;send CR
	CALL	DSP$		;to the display
;
GOCOPY	CALL	CREATE		;create/overwrite file
	RET	C		;don't overwrite!
	CALL	COPYIT		;copy data
	CALL	CLOSEIT		;close the file
	RET			;done, return!
;
;	IX => dosplus file
;	IY => trsdos file
;
COPYIT	LD	BC,0		;starting rel sector
	LD	A,(BACPAR)	;dest = trsdos?
	INC	A		;FF = true?
	JP	Z,COPYTRS	;yes, copy to TRSDOS
	LD	A,(IY+20)	;eof sector
	AND	(IY+21)		;msb end
	INC	A		;FFFF?
	RET	Z		;yes, nil!
;
COPYDSP	LD	(RELST),BC	;save starting rel sector
	LD	HL,(BUFFER)	;get start buffer
RDTRS	PUSH	BC		;save rel sector
	CALL	POSIT		;position the file
	LD	A,(SOURCE)	;get source drive
	LD	C,A		;pass it
	LD	B,0		;extended sector
	CALL	DISKRD		;read the sector
	POP	BC		;bump rel sector
	CALL	ROOM		;end of memory?
	JR	Z,WRDOSP	;yes, write buffer
	CALL	FILEND		;end of file?
	JR	Z,WRDOSP	;yes, write buffer
	INC	H		;bump buffer pointer
	INC	BC		;bump rel sector
	JR	RDTRS		;read some more
;
WRDOSP	LD	(RELSEC),BC	;save relative sector
	LD	BC,(RELST)	;get starting rel sector
	LD	HL,(BUFFER)	;get buffer start
	DEC	BC		;adjust for next add
;
WRDPL	INC	BC		;bump rel sector
	LD	DE,SDCB		;source DCB
	LD	(SDCB+3),HL	;pass buffer
	LD	(SDCB+5),HL	;pass buffer to write
	LD	A,DIRWR		;SVC #
	RST	SVC		;write a sector
	JP	NZ,POSTE	;error! go
	INC	H		;bump buffer
	PUSH	HL		;save it
	LD	HL,(RELSEC)	;get ending sector
	OR	A		;clear carry
	SBC	HL,BC		;done?
	POP	HL		;restore it
	JR	NZ,WRDPL	;nope, write more
	CALL	FILEND		;end of file?
	JR	NZ,COPYDSP	;nope, read some more
	RET			;else done!
;
COPYTRS	LD	A,(IX+3)	;check for nil file
	OR	(IX+20)
	OR	(IX+21)
	RET	Z		;nil, return!
;
COPYTR	LD	(RELST),BC	;save starting sector
	LD	HL,(BUFFER)	;buffer start
RDDOSP	LD	DE,SDCB		;source DCB
	LD	(SDCB+3),HL	;pass buffer
	LD	(SDCB+5),HL
	LD	A,DIRRD		;SVC #
	RST	SVC		;read a sector
	JP	NZ,POSTE	;error!
	CALL	ROOM		;room for more?
	JR	Z,WRTRS		;nope, write partial
	CALL	FILENDD		;end of file?
	JR	Z,WRTRS		;yes, write partial
	INC	H		;bump buffer
	INC	BC		;bump rel sector
	JR	RDDOSP		;read some more
;
WRTRS	LD	(RELSEC),BC	;save rel sector
	LD	BC,(RELST)	;starting sector
	LD	HL,(BUFFER)	;starting buffer
	DEC	BC		;adjust for add
;
WRTR	INC	BC		;bump rel sector
	PUSH	BC		;save it
	CALL	POSIT		;position the file
	LD	A,(DEST)	;get dest drive
	LD	C,A		;pass it
	LD	B,0		;extended sector
	LD	A,DWRIT		;disk command
	RST	DISKIO		;write to disk
	JP	NZ,POSTE	;error, go!
	INC	H		;bump buffer
	POP	BC		;get rel sector back
	PUSH	HL		;save buffer
	LD	HL,(RELSEC)	;get ending sector
	OR	A		;clear carry flag
	SBC	HL,BC		;end of file?
	POP	HL		;get buffer pointer
	JR	NZ,WRTR		;write some more
	CALL	FILENDD		;end of file?
	JR	NZ,COPYTR	;nope, continue
	RET			;file copied!
;
ROOM	PUSH	HL		;save buff pointer
	PUSH	DE		;save track/sec
	OR	A		;clear carry
	LD	DE,(BUFEND)	;get end of buffer
	SBC	HL,DE		;compare 'em
	POP	DE
	POP	HL
	RET			;Z = buffer full!
;
FILEND	PUSH	HL		;save buffer
	LD	L,(IY+20)	;lsb eof
	LD	H,(IY+21)	;msb eof
	OR	A		;clear carry
	SBC	HL,BC		;compare to current
	POP	HL		;unstack
	RET			;Z = file end
;
FILENDD	PUSH	HL		;save buffer
	LD	L,(IX+20)	;lsb eof
	LD	H,(IX+21)	;msb eof
	SCF			;set carry
	SBC	HL,BC		;compare to current
	POP	HL		;unstack
	RET			;Z = file end
;
FINDT	LD	HL,(SQBUFF)	;system buffer
	LD	A,(FILES)	;starting sector files
	LD	E,A		;E = sector 2 or 3
FINDTT	LD	B,0		;extended sector
	CALL	DISKRD		;read it
FINDTTT	PUSH	HL		;save pointer
	PUSH	DE		;save too
	CALL	NAMMAT		;matching name?
	POP	DE		;restore
	POP	HL
	RET	Z		;yes, return
;
	LD	A,L		;get lsb
	ADD	A,40H		;offset to next name
	LD	L,A		;put it back
	JR	NZ,FINDTTT
	INC	E		;bump sector
	LD	A,E		;get sector
	CP	26		;2-25 only
	JR	C,FINDTT
	OR	-1		;return NZ
	RET			;not found
;
NAMMAT	LD	A,L		;get LSB
	ADD	A,5		;point to name
	LD	L,A		;put it back
	PUSH	IX		;pass IX to DE
	POP	DE
	LD	A,E		;get LSB
	ADD	A,5		;point to name
	LD	E,A		;put it back
	LD	B,11		;11 chars to check
FTTT	LD	A,(DE)		;get char
	CP	(HL)		;match?
	RET	NZ		;nope, skip
	INC	HL		;bump pointers
	INC	DE
	DJNZ	FTTT		;continue
	LD	A,L		;get LSB
	AND	0C0H		;reset to beginning
	LD	L,A		;HL => entry
	PUSH	HL		;pass to IY
	POP	IY
	XOR	A		;set found
	RET			;return Z for match
;
POSIT	PUSH	IY		;save file pointer
	PUSH	HL		;save too
	LD	L,23		;23 extents max
POSLP	LD	A,(IY+16H)	;get extent
	LD	D,A		;pass as track
	CP	77		;out of range?
	JR	NC,POSBAD	;file error!
	LD	A,(IY+17H)	;get elements
	PUSH	AF		;save unmasked
	RLCA			;align to low bits
	RLCA
	RLCA
	AND	7		;3 bits only
	LD	H,A		;pass offset
	LD	A,1		;starting sector
	JR	Z,POSX		;have it, go!
POSY	ADD	A,5		;next gran
	DEC	H		;less offset
	JR	NZ,POSY		;continue
POSX	LD	E,A		;sector start
	POP	AF		;get extents back
	AND	1FH		;# grans -1
	INC	A		;# contig grans
	LD	H,A		;H = # grans
POSO	LD	A,5		;sectors/gran
	EX	AF,AF'		;swap 'EM
POSZ	LD	A,B		;have the sector?
	OR	C
	JR	Z,POSRET	;yes, return!
	DEC	BC		;less rel sector
	CALL	ADVSEC		;DE = next sector
	EX	AF,AF'		;get # sects / gran
	DEC	A		;done with gran?
	JR	Z,POSZZ		;yes, go!
	EX	AF,AF'		;swap back
	JR	POSZ		;continue with gran
POSZZ	DEC	H		;less # grans this extent
	JR	NZ,POSO		;continue with extent
	INC	IY		;bump pointer
	INC	IY
	DEC	L		;less # extents
	JP	NZ,POSLP	;get next extent
	LD	A,ERR92		;no extent terminator
	JP	POSTE		;go error!
;
POSBAD	LD	A,ERR98		;illegal data range
	JP	POSTE		;go error
;
POSRET	POP	HL		;unstack
	POP	IY
	RET			;DE = track/sector
;
ADVSEC	INC	E		;bump sector
	LD	A,E		;get it
	CP	26		;end of track?
	RET	C		;nope, return
	LD	E,1		;reset to start
	INC	D		;bump track
	RET			;DE = next sector
HASH	PUSH	IX		;save
	PUSH	BC		;save
	LD	BC,0B00H	;11 chars, C = code
HASHLP	LD	A,(IX+5)	;get a byte
	INC	IX		;bump pointer
	XOR	C		;hash it
	RLCA
	LD	C,A		;save temp
	DJNZ	HASHLP		;continue
	POP	BC		;unstack
	POP	IX
	OR	A		;zero?
	RET	NZ		;nope, return
	INC	A		;else make it one
	RET			;done
;
CLOSEIT	LD	A,(BACPAR)	;dest = trsdos?
	INC	A
	RET	Z		;yes, no need to close it
	LD	BC,(SDCB+16)	;get drive/lfn
;
	LD	DE,SDCB		;open DCB
	LD	A,CLOSE		;SVC #
	RST	SVC		;close the file
	JP	NZ,POSTE	;go error if bad!
;
	CALL	DIRRED		;read in the file entry
	PUSH	HL		;pass to IX
	POP	IX		;IX => file entry
	LD	A,(IY+10H)	;lsb password
	LD	(IX+10H),A	;pass it
	LD	A,(IY+12H)	;lsb access pass
	LD	(IX+12H),A	;pass it
	LD	A,(IY+11H)	;msb update password
	RLCA			;align to high nibble
	RLCA
	RLCA
	RLCA
	AND	0F0H
	LD	E,A		;save temp
	LD	A,(IY+13H)	;msb access
	AND	0FH		;low nibble
	OR	E		;merge it
	LD	(IX+11H),A	;save two msb's
	LD	A,(IY+57)	;get access level
	AND	7		;low 3 bits only
	OR	(IX)		;merge it
	LD	(IX),A		;prot level set
	LD	A,(IY+3)	;get eof byte
	LD	(IX+3),A	;pass it
	LD	A,(IY+4)	;get LRL
	LD	(IX+4),A	;pass it
;
	LD	A,(IY+54)	;get year
	SUB	80		;less 1980
	RRCA			;move to high bits
	RRCA
	RRCA
	AND	0E0H		;top 3 bits only
	LD	(IX+2),A	;save it
	LD	A,(IY+55)	;get month
	AND	0FH		;low 4 bits
	OR	40H		;set MOD bit
	LD	(IX+1),A	;save it
	LD	A,(IY+56)	;get day
	AND	1FH		;low 5 bits
	OR	(IX+2)		;merge bits
	LD	(IX+2),A	;update
	LD	E,(IY+20)	;get eof sector
	LD	D,(IY+21)
	INC	DE		;adjust to dosplus
	LD	(IX+20),E
	LD	(IX+21),D
	CALL	DIRWRT		;write sys buffer back
;
	RET			;else done, on to next
;
;	open up dest file for I/O
;
CREATE	LD	A,(BACPAR)	;dest = TRSDOS?
	INC	A		;FF true?
	JP	Z,CRTRS		;yes, create on trsdos
;
;	create file on DOSPLUS disk
;
CRDOSP	PUSH	IX		;pass to IY
	POP	IY		;IY => trsdos file
	LD	DE,MDCB		;DE => filespec
	BIT	5,(IY)		;program file?
	JR	Z,NOEXT		;skip extension
	LD	HL,CMDEXT	;add default extension
	LD	A,FEXT		;SVC #
	RST	SVC		;add it on
;
NOEXT	LD	HL,MDCB		;packed filespec
	LD	DE,SDCB		;move it here
	LD	BC,41		;length
	LDIR			;move it!
	LD	DE,SDCB		;here now
	PUSH	DE		;save pointer
NOEXL	LD	A,(DE)		;get file byte
	CP	ETX		;terminator?
	JR	Z,INSDRV	;yes, insert drive #
	INC	DE		;bump pointer
	JR	NOEXL		;continue
;
INSDRV	CALL	INSPASS		;insert password
	LD	A,':'		;insert drive marker
	LD	(DE),A		;put into record
	INC	DE		;bump pointer
	LD	A,(DEST)	;get dest drive
	ADD	A,8		;adjust to device #
	ADD	A,A		;*2
	ADD	A,A		;*4
	ADD	A,2		;offset to name
	LD	L,A		;lsb table pointer
	LD	H,1		;HL => name
	LD	A,(HL)		;get drive name
	LD	(DE),A		;save into DCB
	INC	HL		;bump pointer
	INC	DE		;dcb pointer
	LD	A,(HL)		;get 2nd byte
	LD	(DE),A		;save too
	INC	DE		;bump pointer
	LD	A,ETX		;terminate it
	LD	(DE),A		;to buffer
	POP	DE		;DE => DCB
	LD	HL,OLIST	;open param list
	LD	A,OPEN		;SVC #
	RST	SVC		;open/create file
	JP	NZ,POSTE	;go error!
;
	JR	C,ALLOCD	;allocate space
OVERWT	LD	A,(OVEPAR)	;overwrite file?
	INC	A		;prompt?
	JR	NZ,ALLOCD	;allocate space
;
;	file exists, prompt for overwrite
;
OVERP	LD	HL,MDCB		;packed filespec
	CALL	LINE$		;display name
	LD	HL,OVERMSG	;prompt
	LD	DE,STRING	;input buffer
	LD	BC,OVERMSGL<8+1	;disp len + key len
	LD	A,VIDKEY	;SVC #
	RST	SVC		;display, get keys
	JP	NZ,POSTE	;error!
	JP	C,TERMIN	;break, abort!
	LD	A,(DE)		;get input char
	CALL	UCASE		;make it upper case
	CP	'Y'		;yes?
	JR	Z,ALLOCD	;yes, allocate space
	CP	'N'		;no?
	JR	NZ,OVERP	;invalid, ask again
	SCF			;carry = skip
	RET
;
;	allocate needed space
;
ALLOCD	LD	A,(BACPAR)	;dest = trsdos?
	INC	A
	JP	Z,ALLOCT	;yes, allocate trsdos
;
	LD	DE,SDCB		;point to open DCB
	LD	C,(IY+20)	;lsb EOF
	LD	B,(IY+21)	;msb EOF
	LD	A,(IY+3)	;get EOF byte
	OR	B		;check for nil file
	OR	C		;all zeroes?
	RET	Z		;yes, nil file!
	INC	BC		;adjust ending record
	LD	A,DIRWR		;SVC #
	RST	SVC		;allocate space!
	JP	NZ,POSTE	;go error!
	OR	A		;clear carry
	RET			;done, space allocated
;
;	create file on TRSDOS disk
;
CRTRS	LD	HL,MDCB		;filename is here
	LD	DE,SDCB		;move it here
	LD	B,41		;length of it
CRTRSO	LD	A,(HL)		;get a byte
	CP	ETX		;terminator?
	JR	Z,CRTRSD	;yes, insert password
	LD	(DE),A		;put into buffer
	INC	HL		;bump pointers
	INC	DE
	DJNZ	CRTRSO		;continue
;
CRTRSD	CALL	INSPASS		;insert password
	LD	A,ETX		;terminate
	LD	(DE),A		;stuff it
	LD	DE,SDCB		;start of filespec
	LD	HL,FLIST	;open list
	LD	A,OPEN		;SVC #
	RST	SVC		;create open DCB
	JP	NZ,POSTE	;error!
;
;	change /CMD extension to 'program file'
;
	RES	5,(IX+1)	;turn off flag bit
	LD	A,(IX+13)	;get extension
	CP	'C'
	JR	NZ,NOTEXT
	LD	A,(IX+14)
	CP	'M'
	JR	NZ,NOTEXT
	LD	A,(IX+15)
	CP	'D'		;is it?
	JR	NZ,NOTEXT	;nope, continue
;
	LD	(IX+13),' '
	LD	(IX+14),' '
	LD	(IX+15),' '
	PUSH	IX		;pass it
	POP	HL		;to HL
	LD	A,L		;get LSB
	ADD	A,5		;offset to name
	LD	L,A		;put it back
	LD	DE,MDCB		;mask DCB
	LD	B,2		;command
	LD	A,WILD		;SVC #
	RST	SVC		;change the file
	SET	5,(IX+1)	;set 'program' flag
;
NOTEXT	LD	A,(DEST)	;get dest drive #
	LD	C,A		;pass to C
	LD	IY,(DDCT)	;get dest DCT pointer
	LD	D,(IY+18)	;get directory track
	CALL	FINDT		;see if file exists
	LD	(SECSAV),DE	;save track/sector
	JP	Z,OVERWT	;yes, see if overwrite
;
;	create file in directory
;
	LD	A,(HIT)		;get hit sector
	LD	E,A		;E = hit sector
	LD	B,0		;extended sector
	LD	HL,(SQBUFF)	;use system buffer
	CALL	DISKRD		;read it
;
	LD	B,60H		;max # files
	XOR	A		;load zero
FILEL	CP	(HL)		;open slot?
	JR	Z,FILEY		;yes, go!
	INC	L		;bump pointer
	DJNZ	FILEL		;keep going
	LD	A,ERR26		;dir space full!
	JP	POSTE		;abort!
;
FILEY	LD	A,(FILES)	;start of files
	LD	E,A		;pass it
	LD	A,L		;get DEC
FILEX	SUB	4		;less # files / sector
	JR	C,FILEZ		;found the sector
	INC	E		;bump sector
	JR	FILEX		;continue
;
FILEZ	ADD	A,4		;add it back
	ADD	A,A		;*2
	ADD	A,A		;*4
	ADD	A,A		;*8
	ADD	A,A		;*16
	ADD	A,A		;*32
	ADD	A,A		;*64
	PUSH	AF		;save offset
;
	CALL	HASH		;get hashcode of file
	LD	(HL),A		;put into HIT
;
	LD	(SECSAV),DE	;save sector
	LD	A,(HIT)		;get hit sector
	LD	E,A		;E = hit sector
	LD	B,0		;extended sector
	LD	L,B		;reset to buff beginning
	LD	A,DWRIT		;direct write
	RST	DISKIO		;write it back
	JP	NZ,POSTE	;go error!
;
	LD	DE,(SECSAV)	;get sector
	LD	B,0		;extended sector
	CALL	DISKRD		;read it
	POP	AF		;get sector offset
	LD	L,A		;HL => directory record
	PUSH	HL		;pass to IY
	POP	IY		;IY => directory record
;
	LD	D,H		;pass to DE
	LD	E,L
	INC	DE		;start +1
	LD	BC,63		;length -1
	LD	(HL),-1		;fill with FF
	LDIR			;fill it
;
;	insert filename
;
	PUSH	IX		;save pointers
	PUSH	IY
	LD	B,11		;name length
PUTNAM	LD	A,(IX+5)	;get a name byte
	LD	(IY+5),A	;put into TRSDOS entry
	INC	IX		;bump pointers
	INC	IY
	DJNZ	PUTNAM		;finish it
	POP	IY
	POP	IX		;unstack 'em
;
;	check for 'program' flag bit
;
	LD	(IY),0		;reset flags
	BIT	5,(IX+1)	;program bit set?
	JR	Z,NOTCMD	;nope, continue
	SET	5,(IY)		;set 'program' file
NOTCMD	LD	(IY+1),0
	LD	(IY+2),0
	LD	A,(IX+3)	;get eof byte
	LD	(IY+3),A
	LD	A,(IX+4)	;get lrl byte
	LD	(IY+4),A	;pass it
;
;	insert nil passwords
;
	LD	(IY+16),96H
	LD	(IY+17),42H
	LD	(IY+18),96H
	LD	(IY+19),42H
;
;	reset end of file sector
;
	LD	(IY+20),0
	LD	(IY+21),0
;
;	pass date to file
;
	LD	A,(IX+2)	;get year
	RLCA			;align to low bits
	RLCA
	RLCA
	AND	7		;8 year range
	ADD	A,80		;add offset
	LD	(IY+54),A	;pass here
	LD	(IY+58),A	;here too
;
	LD	A,(IX+1)	;get month
	AND	0FH		;4 bits
	LD	(IY+55),A	;pass it
	LD	(IY+59),A
;
	LD	A,(IX+2)	;get day
	AND	1FH		;5 bits
	LD	(IY+56),A	;pass it
	LD	(IY+60),A
;
	LD	A,(IX)		;get protection level
	AND	7		;low 3 bits
	LD	(IY+57),A	;pass it
	LD	(IY+61),0	;flag
	LD	(IY+63),0	;set active
;
;	allocate # records
;	IY => directory entry
;
ALLOCT	LD	HL,(BUFFER)	;low buffer
	LD	B,0		;extended sector
	LD	A,(DEST)	;get dest drive
	LD	C,A		;pass it
	PUSH	IY		;save it
	LD	IY,(DDCT)	;get dest DCT
	LD	D,(IY+18)	;get dir track
	POP	IY
	LD	A,(GAT)		;get gat sector
	LD	E,A		;DE = gat sector
	CALL	DISKRD		;read the sector
;
ALLOCTM	LD	C,(IY+20)	;get current allocation
	LD	B,(IY+21)
	LD	L,(IX+20)	;get # needed
	LD	H,(IX+21)
	OR	A		;clear carry
	SBC	HL,BC		;have enough?
	JR	Z,PUTALL	;done! write entry back
	JR	C,PUTALL	;done!
	CALL	CHUNK		;get some grans
	JR	ALLOCTM		;get some more
;
PUTALL	LD	C,(IX+20)	;get eof sector
	LD	B,(IX+21)	;from dosplus
	DEC	BC		;adjust to trsdos
	LD	(IY+20),C	;put into file
	LD	(IY+21),B
	LD	A,(DEST)	;get dest drive
	LD	C,A		;pass it here
	LD	B,0		;extended sector
	LD	HL,(SQBUFF)	;dir record here
	LD	DE,(SECSAV)	;get sector for file
	LD	A,DWRIT		;disk command
	RST	DISKIO		;write the sector
	JP	NZ,POSTE	;go error
;
	LD	A,(GAT)		;get gat sector
	LD	E,A		;DE => gat
	LD	HL,(BUFFER)	;it's here now
	LD	A,DWRIT		;disk command
	RST	DISKIO		;write the sector
	JP	NZ,POSTE	;error!
;
;	file created and space allocated!
;
	XOR	A		;return NC
	RET			;done, copy the file!
;
SHOWFIL	LD	D,0		;fetch cursor posit
	LD	A,VDREAD	;SVC #
	RST	SVC		;get cursor into BC
;
	CALL	LINE$		;display filename
;
	LD	A,VDGRAF	;put cursor back
	RST	SVC		;put it back
	LD	HL,TABS		;tab to next area
;
LINE$	LD	A,(HL)		;fetch a byte
	CP	ETX		;terminator?
	RET	Z		;yes, return
	CALL	DSP$		;display char
	INC	HL		;bump pointer
	JR	LINE$		;next char
;
DSP$	PUSH	BC		;save it
	LD	B,A		;pass char
	LD	A,VDCHAR	;SVC #
	RST	SVC		;display char
	POP	BC		;unstack
	RET	Z		;no error, return
	JP	POSTE		;else abort!
;
DISKRD	LD	A,DREAD		;sector read
	RST	DISKIO		;read it
	RET	Z		;no error
	CP	56		;system sector?
	RET	Z		;skip error
	CP	57		;deleted sector?
	RET	Z		;skip error
	JP	POSTE		;else abort in error
;
;	allocate disk space to TRSDOS file
;
;	IY => directory entry
;	L = # sectors needed
;	gat sector loaded at $IOBUFF
;
CHUNK	PUSH	HL		;save # sectors
	PUSH	IY		;pass pointer to HL
	POP	HL		;HL => dir record
	LD	BC,22		;offset to extents
	ADD	HL,BC		;HL => first extent
	POP	BC		;C = # sectors needed
	LD	B,23		;max # extents available
;
CHUNK1	LD	A,(HL)		;get extent element
	INC	A		;FF terminator?
	JR	Z,CHUNK2	;have a slot
	INC	HL		;pass this extent
	INC	HL
	DJNZ	CHUNK1		;continue
	LD	A,ERR30		;can't extend file!
	JP	POSTE		;out of extent elements
;
;	HL => current extent element
;
CHUNK2	PUSH	DE		;save track/sector
	PUSH	HL		;save extent pointer
	CALL	CHUNK3		;get a chunk
	POP	HL		;get pointer
	LD	A,(VERSION)	;get version #
	CP	2		;2.0?
	LD	A,D		;get track
	JR	NZ,CNKV		;nope, go!
	SUB	60H		;remove start offset
CNKV	LD	(HL),A		;insert it
	INC	HL		;bump pointer
	LD	A,B		;get # grans
	DEC	A		;adjust it
	OR	E		;merge with start gran
	LD	(HL),A		;insert into file
	POP	DE		;unstack
	LD	L,B		;# new grans
	LD	H,0		;HL = # new grans
	LD	C,B
	LD	B,H		;BC = # new grans
	ADD	HL,HL		;*2
	ADD	HL,HL		;*4
	ADD	HL,BC		;*5
	LD	C,(IY+20)	;get current sectors
	LD	B,(IY+21)
	ADD	HL,BC		;add new sectors
	LD	(IY+20),L	;put into record
	LD	(IY+21),H
	RET			;done!
;
CHUNK3	LD	HL,(BUFFER)	;start of allocation
	LD	L,60H		;offset to alloc table
	LD	B,0		;# grans fetched
	LD	A,(VERSION)	;get version
	CP	2		;2.0?
	JR	Z,CHNKK		;yes, go!
	LD	L,B		;else start of table
CHNKK	LD	A,L		;get table start
	ADD	A,77		;point to end of table
	LD	(CHNKTP),A	;save for test
	LD	(CHNKTP2),A	;here also
CHNK	LD	D,L		;pass track
	LD	E,0		;start gran mask
	LD	A,(HL)		;get a byte
	RRA			;available?
	JR	NC,CH0		;free, go!
	LD	E,20H		;mask
	RRA			;available?
	JR	NC,CH1		;free, go!
	LD	E,40H
	RRA
	JR	NC,CH2
	LD	E,60H
	RRA
	JR	NC,CH3
	LD	E,80H
	RRA
	JR	NC,CH4
	INC	L		;bump gat pointer
	LD	A,L		;get it
	CP	0		;end of table?
CHNKTP	EQU	$-1
	JR	C,CHNK		;nope, continue
	LD	A,ERR27		;disk space full!
	JP	POSTE		;display and abort!
;
CH0	BIT	0,(HL)		;bit free?
	RET	NZ		;nope, have it all
	SET	0,(HL)		;allocate it
	CALL	NUFF		;done?
CH1	BIT	1,(HL)
	RET	NZ
	SET	1,(HL)
	CALL	NUFF
CH2	BIT	2,(HL)
	RET	NZ
	SET	2,(HL)
	CALL	NUFF
CH3	BIT	3,(HL)
	RET	NZ
	SET	3,(HL)
	CALL	NUFF
CH4	BIT	4,(HL)
	RET	NZ
	SET	4,(HL)
	CALL	NUFF
;
	INC	L		;bump track
	LD	A,L		;fetch counter
	CP	0		;end of table?
CHNKTP2	EQU	$-1
	JR	C,CH0		;nope, continue
	LD	A,ERR27		;disk space full
	JP	POSTE		;go error!
;
NUFF	INC	B		;bump gran counter
	LD	A,C		;get # sectors needed
	SUB	5		;less this gran
	LD	C,A		;put it back
	JR	C,NUFFBAK	;done!
	LD	A,B		;get # grans
	CP	20H		;at max / extent?
	RET	C		;nope, return
NUFFBAK	POP	HL		;remove caller
	RET			;return next level
;
INSPASS	PUSH	HL		;save pointer
	LD	HL,(PWPAR)	;get password
	LD	A,H		;anything input?
	OR	L
	JR	Z,PASSRET	;nope, continue
	LD	A,'.'		;password marker
	LD	(DE),A		;save it
	INC	DE		;bump pointer
INSPS	LD	A,(HL)		;get a byte
	CP	27H		;term?
	JR	Z,PASSRET	;yes, go!
	CP	'"'		;term?
	JR	Z,PASSRET
	CP	CR		;done?
	JR	Z,PASSRET
	CP	7DH
	JR	Z,PASSRET
	LD	(DE),A		;put into buffer
	INC	HL		;bump pointers
	INC	DE
	JR	INSPS		;continue
PASSRET	POP	HL		;unstack
	RET			;done!
;
	PAGE
;
;	text area
;
HELLO	DEFB	CR
	DEFM	'CONV - '
	DEFM	'TRSDOS File Conversion Utility - '
	DEFM	'Series A.00'
	DEFB	CR
	DEFM	'(c)(p) Copyright 1982 '
	DEFM	'by MicroPower, Inc.'
	DEFB	CR
	DEFB	CR
	DEFB	CR
	DEFB	ETX
;
PROMPT	DEFM	'*'
;
	PAGE
;
;	data areas
;
EBLOCK	DEFB	0		;flag
	DEFW	SDCB		;source DCB
	DEFW	DDCB		;dest DCB
	DEFW	MDCB		;mask DCB
	DEFW	PARAMS		;param block
;
PARAMS	DEFB	20H+4
	DEFW	QUEPAR
	DEFM	'QUERY'
;
	DEFB	20H+3
	DEFW	OVEPAR
	DEFM	'OVER'
;
	DEFB	20H+3
	DEFW	BACPAR
	DEFM	'BACK'
;
	DEFB	20H+2
	DEFW	DIRPAR
	DEFM	'DIR'
;
	DEFB	20H+2
	DEFW	DIRPAR
	DEFM	'CAT'
;
	DEFB	20H+3
	DEFW	ECHPAR
	DEFM	'ECHO'
;
	DEFB	20H+2
	DEFW	SYSPAR
	DEFM	'SYS'
;
	DEFB	80H+1
	DEFW	PWPAR
	DEFM	'PW'
;
	DEFB	0
;
OLIST	DEFW	IOBUFF		;I/O buffer
	DEFW	0
	DEFW	0
	DEFB	'W'
	DEFB	0
	DEFB	'F'
	DEFB	2
	DEFB	0
;
FLIST	DEFW	IOBUFF		;I/O buffer
	DEFW	0
	DEFW	0
	DEFB	'R'
	DEFB	0
	DEFB	'F'
	DEFB	3
	DEFB	0
;
QUEPAR	DEFW	8080H		;query
OVEPAR	DEFW	8080H		;overwrite
BACPAR	DEFW	8080H		;dest is trsdos
DIRPAR	DEFW	8080H		;directory
ECHPAR	DEFW	8080H		;echo
SYSPAR	DEFW	8080H		;system files
PWPAR	DEFW	0000H		;master password
;
SDCB	DEFS	41		;source
DDCB	DEFS	41		;dest
MDCB	DEFS	41		;mask
;
STRING	DEFS	80		;input string
DCTSAV	DEFS	25		;storage for DCT restore
BUFFER	DEFW	0		;buffer start
BUFEND	DEFW	0		;buffer end
SPBUFF	DEFW	0		;special I/O buffer
SQBUFF	DEFW	0		;special I/O buffer
SECSAV	DEFW	0		;track/sector saver
RELSEC	DEFW	0		;relative sector
RELST	DEFW	0		;starting rel sector
VERSION	DEFB	0		;program version #
GAT	DEFB	1		;gat table sector
HIT	DEFB	1		;hit table sector
FILES	DEFB	2		;files start sector
;
DEST	DEFW	0
DDCT	DEFW	0
;
QUEMSG	DEFB	CR
	DEFB	11
	DEFB	9
	DEFB	9
	DEFM	'Convert   ? '
QUEMSGL	EQU	$-QUEMSG
;
OVERMSG	DEFB	CR
	DEFB	11
	DEFB	9
	DEFB	9
	DEFM	'Overwrite ? '
OVERMSGL EQU	$-OVERMSG
;
CMDEXT	DEFM	'CMD'
;
TABS	DEFB	9
	DEFB	9
	DEFB	ETX
;
ZZZZZ	EQU	$
;
	END	ENTRY
