DOS3D  ; dos3d/asm
 	TITLE	'<DOS+II rs232 drivers>'
 ;
 ; size = 333 bytes (14DH)
 ;
 ; equates
 SIODA	EQU	3		;SIO channel A DCB number
 SIODB	EQU	4		;SIO channel B DCB number
 ;
 ; vector for SIO (in/out/ctrl) (SVC's 96-101)
 ;	on entry, IX=DCB
 ;		 C = input a byte
 ;		 Z = output a byte
 ;		else @CTRL call (wait on input)
 ;	on exit, B = char, A = status
 ;		 Z = OK, char in B
 ;		NZ = no character
 ;		 Carry = carrier lost
 ;
 SIOCODE PUSH	BC		;save on top
 	LD	C,(IX+19)	;get base port
 	RES	7,(IX+5)	;turn off @CTL
 	JR	C,ABRCV 	;called via @GET
 	JR	Z,ABTX		;called via @PUT (send)
 	SET	7,(IX+5)	;@CTL call (input wait)
 ABRCV	BIT	1,(IX+5)	;initialized?
 	LD	A,66		;'no function exists'
 	JR	NZ,SIORET	;return in error
 ABRCVWT CALL	BUFTAKE 	;any chars in buffer?
 	JR	NZ,RXIN 	;char in buffer
 	BIT	7,(IX+5)	;see if @CTRL call
 	JR	NZ,ABRCVWT	;wait if yes
 	CALL	RSTATUS 	;else get status
 	SCF			;C = error
 	BIT	3,B		;carrier detect?
 	JR	NZ,SIORET	;return if not
 	OR	-1		;set NZ, NC
 	LD	A,B		;restore status
 	JR	SIORET		;done
 RXIN	LD	A,(HL)		;get the character
 	AND	(IX+22) 	;word length mask
 	LD	B,A		;B = char
 	INC	HL		;bump pointer
 	LD	C,(HL)		;C = status byte
 	BIT	0,(IX+5)	;translate?
 	JR	Z,SIO4		;continue if not
 	LD	A,B		;get char
 	LD	L,(IX+6)	;table address
 	LD	H,(IX+7)
 	LD	B,(IX+8)	;# of entries
 	CALL	XLATE		;translate the char
 	LD	B,A		;put char back
 SIO4	POP	HL		;get old BC
 	LD	A,C		;get status
 	LD	C,L		;original C
 	CP	A		;set Z flag for OK
 	RET			;done
 	PAGE
 ;
 ; SIO transmit services
 ;
 ABTX	BIT	1,(IX+5)	;initialized?
 	LD	A,66		;'does not exist'
 	JR	NZ,SIORET	;return in error
 	LD	A,B		;get character
 	AND	(IX+22) 	;word length mask
 	LD	E,A		;save it here
 TXWT	CALL	RSTATUS 	;get status byte
 	BIT	2,B		;tx buffer empty?
 	JR	Z,TXWT		;wait if not
 	LD	C,(IX+19)	;SIO base port
 	DEC	C		;point to data port
 	DEC	C
 	OUT	(C),E		;send the byte
 	XOR	A		;set Z for OK
 	LD	A,B		;get status byte
 SIORET	POP	BC		;restore chars
 	RET			;done
 ;
 	PAGE
 ;
 ; fetch the status of the SIO port
 ;	set the mask byte so 0 bits = OK
 ;
 RSTATUS LD	C,(IX+19)	;get SIO base port
 	LD	A,10H		;clear the latch
 	OUT	(C),A
 	IN	A,(C)		;read R0
 	AND	0ADH		;Brk,Cts,Dcd,Tx,Rx
 	XOR	28H		;reverse Cts,Dcd
 	LD	B,A		;save here
 	LD	A,1		;select R1
 	OUT	(C),A
 	IN	A,(C)		;read R1
 	AND	50H		;framing,parity
 	OR	B		;complete mask
 	OR	(IX+23) 	;with error mask
 	LD	(IX+23),A	;reset mask
 	LD	B,A		;B = resulting status
 	RET
 ;
 	PAGE
 ;
 ; SIO interrupt services
 ;
 RSINTA	PUSH	AF		;save
 	LD	A,SIODA 	;channel A DCB
 	JR	RSINT		;go common
 ;
 RSINTB	PUSH	AF		;save
 	LD	A,SIODB 	;channel B DCB
 ;
 RSINT	PUSH	BC		;save everything
 	PUSH	DE
 	PUSH	HL
 	PUSH	IX
 ;
 	CALL	LOCDCB		;locate DCB
 	LD	C,(IX+19)	;get base port
 	DEC	C
 	DEC	C		;C = data port
 	IN	B,(C)		;get character
 	CALL	BUFFADD 	;room to add?
 	LD	A,20H		;overrun mask
 	JR	C,RSOVER	;overflow
 	LD	(HL),B		;put char in buffer
 	INC	HL		;inc buffer
 	CALL	RSTATUS 	;get status byte
 	LD	(HL),B		;put status byte in
 	XOR	A		;mask for OK
 ;
 RSOVER	OR	(IX+23) 	;set error mask
 	LD	(IX+23),A	;update to DCB
 ;
 	POP	IX		;restore stack
 	POP	HL
 	POP	DE
 	POP	BC
 	POP	AF
 	EI
 	RETI			;return from interrupt
 	PAGE
