DOSSYS ; dossys12/asm - kjw/bqsd - 06/82 - 07/82
 ;
 	TITLE	'<DOS+II - SYS12/SYS>'
 ;
 ;	system is high residing (2200-27FF)
 ;	and contains the following:
 ;
 ;	$VERIFY	- disk verify ON/OFF
 ;	$TIME	- fetch/load TIME$
 ;	$I	- deselect/fetch DCT from disk
 ;	$ERROR	- display error message
 ;	$DEBUG	- debugger ON/OFF
 ;	$DATE	- fetch/load DATE$
 ;	$AUTO	- set/disable AUTO command
 ;	$CLOCK	- clock display ON/OFF
 ;	$LIB	- display library words
 ;	$PAUSE	- wait for <KEY> to continue
 ;	$SCREEN	- send screen to device
 ;
 ;	command syntax used:
 ;
 ;	UPPER CASE	- mandatory input
 ;	lower case	- user specified input
 ;	channel 	- device 0-7 or
 ;			- filespec with no wild chars
 ;	device		- devices 0-15
 ;	drive		- devices 8-15
 ;	space		- mandatory space(s)
 ;	comma		- can be space(s) or comma(s)
 ;	apostrophe	- mandatory comma(s)
 ;
 	PAGE
 ;
 ;	Outside Support Equates
 ;
 VDLINE	EQU	9		;line to video
 PARAM	EQU	87		;fetch parameters
 NMICTL	EQU	92		;add/kill NMI vector
 SERROR	EQU	39		;display error
 OPEN	EQU	40		;open a channel
 CLOSE	EQU	42		;close a channel
 EVAL	EQU	83		;evaluate command
 SDATE	EQU	45		;fetch/load DATE$,TIME$
 KBDW	EQU	73		;key input with wait
 SCRSVC	EQU	91		;screen print
 FSPEC	EQU	84		;fetch file spec
 PUT	EQU	78		;byte to device
 VIDKEY	EQU	12		;display & key input
 ;
 DREAD	EQU	6		;read system sector
 DWRIT	EQU	7		;write system sector
 ;
 SVC	EQU	08H		;execute SVC
 DISKIO	EQU	10H		;disk I/O
 OVLY	EQU	18H		;load/exec overlay
 ;
 LOCDCB	EQU	032AH		;locate DCB
 LOCDCT	EQU	032DH		;locate DCT
 REGSAV	EQU	0303H		;save registers
 RECMEM	EQU	0333H		;reclaim high memory
 ;
 KDCB	EQU	0		;keyboard DCB#
 SLOTCLK EQU	4		;clock slot number
 CLKTASK EQU	0060H		;clock interrupt vector
 FLAG1S	EQU	0199H		;debug 'trigger'
 FLAG2S	EQU	019AH		;system flag
 FLAGK1	EQU	0173H		;system flag #1
 FLAGK3	EQU	0175H		;system flag #3
 LASTERR EQU	FLAGK3		;last error to system
 ;
 BADF	EQU	1		;'bad function code'
 INVI	EQU	80		;'invalid input channel'
 INVO	EQU	81		;'invalid output channel'
 ILLDRV	EQU	32		;illegal drive number
 ;
 ETX	EQU	3		;end of text
 CR	EQU	13		;carriage return
 ECR	EQU	';'             ;embedded CR
 ETB	EQU	23		;clear to EOL
 ESC	EQU	27		;escape
 DODCB	EQU	1		;display DCB #
 PRDCB	EQU	2		;printer DCB #
 ;
 	PAGE
 ;
 	ORG	2200H		;high overlay
 ;
 ;	Table of vectored routines
 ;
 VECTORS DEFW	RETURN		;0 - return
 	DEFW	VERIFY		;1
 	DEFW	TIME		;2
 	DEFW	I@		;3
 	DEFW	ERROR		;4
 	DEFW	DEBUG		;5
 	DEFW	DATE		;6
 	DEFW	UNDEF		;7 (formerly $BOOT)
 	DEFW	AUTO		;8
 	DEFW	CLOCK		;9
 	DEFW	LIB		;10
 	DEFW	PAUSE		;11
 	DEFW	SCREEN		;12
 	DEFW	UNDEF		;13 - undefined
 	DEFW	UNDEF		;14
 	DEFW	UNDEF		;15
 ;
 ;	undefined system call vector
 ;
 UNDEF	LD	A,BADF		;'bad function call'
 	OR	A		;set NZ
 	RET			;back to caller
 ;
 ;	exit vector for LIB commands
 ;
 EXIT	AND	7FH		;assure a return
 	LD	B,A		;pass error code here
 EXITB	LD	A,SERROR	;SVC #
 	RST	SVC		;call $ERROR
 RETURN	XOR	A		;set Z condition
 	RET			;done
 ;
 	PAGE
 ;
 ;	$LIB - library command
 ;
 ;		LIB
 ;		LIB TO channel
 ;
 ;
 ;	shorthand
 ;
 ;		LIB
 ;		LIB TO channel
 ;
 ;	ENT	HL => command input string
 ;
 ;	EXT	library command list sent to channel
 ;
 LIB	LD	A,DODCB		;default to display
 	CALL	GETCHA		;setup channel
 	JR	NZ,EXIT		;error, display it
 ;
 ;	set-up loop to display names
 ;
 	CALL	GETLIB		;get library word addr.
 ;
 LIB2	LD	E,8		;# names per line
 LIB3	LD	D,10		;chars per name
 ;
 ;	fetch a character, check for terminator
 ;
 	LD	A,(HL)		;get a char
 	AND	7FH		;remove high bit
 	JR	Z,LIBRET	;done, send CR, return
 ;
 LIBLP2	LD	B,(HL)		;fetch a char
 	RES	7,B		;turn off high bit
 	CALL	OUTPUT		;send to device
 	JR	NZ,EXIT 	;exit in error
 	INC	HL		;bump string pointer
 	DEC	D		;less byte counter
 	BIT	7,(HL)		;start of next word?
 	JR	Z,LIBLP2	;get next word if not
 ;
 ;	check if end of line
 ;
 	DEC	E		;dec word counter
 	JR	Z,NXTLIN	;next line
 ;
 ;	else pad rest of word length with spaces
 ;
 	LD	B,' '           ;else send spaces
 LIBLP3	CALL	OUTPUT		;to pad this word
 	JR	NZ,EXIT 	;error, abort!
 	DEC	D		;count this byte
 	JR	NZ,LIBLP3	;finish this word
 	JR	LIB3		;go next word
 ;
 ;	line done, send C/R
 ;
 NXTLIN	CALL	OUTCR		;send a C/R
 	JR	NZ,EXIT 	;error, exit!
 	JR	LIB2		;go next word
 ;
 ;	completed, send CR and close output channel
 ;
 LIBRET	CALL	OUTCR		;send C/R
 	JR	NZ,EXIT 	;exit program
 	LD	DE,DDCB		;device
 	CALL	DCLOSE		;close the device
 	JR	NZ,EXIT		;error exit
 	XOR	A		;force ZERO
 	RET
 ;
 ;	fetch address of LIB commands
 ;
 GETLIB	LD	A,1-1<4+14	;sys1, command 14
 	RST	OVLY		;fetch lib address
 ;
 	PAGE
 ;
 ;	$PAUSE	- wait for key input
 ;
 ;		PAUSE
 ;		PAUSE,message
 ;
 ;	shorthand
 ;
 ;		PAUSE
 ;		PAUSE,message
 ;
 ;	ENT	HL => pause display message
 ;
 ;	EXT	return after first key pressed
 ;
 ;	turn off chaining to get the key
 ;
 PAUSE	LD	A,KDCB		;keyboard DCB
 	CALL	LOCDCB		;find it
 	LD	A,(IX+5)	;get flags
 	PUSH	AF		;save 'em
 	RES	1,(IX+5)	;turn off chaining
 ;
 	LD	HL,PAUMSG	;'<KEY> to continue:'
 	LD	DE,BUFADR	;dummy buffer
 	LD	BC,PAULEN<8+1	;length + key input
 	LD	A,VIDKEY	;display + key input
 	RST	SVC		;display it
 	JR	NC,CONTPAU	;not a break
 ;
 	POP	AF		;restore stack
 	BIT	1,A		;chanining was on?
 	JR	Z,CONTPAU	;nope, forget it
 	LD	L,(IX+19)	;get DCB
 	LD	H,(IX+20)
 	LD	BC,256+40	;length
 	CALL	RECMEM		;reclaim memory
 	XOR	A
 	RET
 ;
 CONTPAU	POP	AF		;restore stack
 	LD	(IX+5),A	;restore flags
 	XOR	A		;set Z
 	RET			;done
 ;
 ;	message displayed during pause
 ;
 PAUMSG	DEFM	'<KEY> to continue:'
 PAULEN	EQU	$-PAUMSG
 ;
 	PAGE
 ;
 ;	$CLOCK	- activate/deactivate clock display
 ;
 ;		CLOCK
 ;		CLOCK,ON	(ON=ON)(OFF=OFF)
 ;		CLOCK,OFF	(OFF=ON)(ON=OFF)
 ;
 ;	shorthand
 ;
 ;		CLOCK		(turns on)
 ;		CLOCK,N 	(turns off)
 ;
 ;	ENT	HL => input command string
 ;
 ;	EXT	clock turned on/off if valid command
 ;
 CLOCK	CALL	ONOFF		;check user parameters
 ;
 	JR	NZ,DOCOFF	;turn clock OFF
 ;
 ;	turn clock display ON
 ;
 DOCON	LD	B,-1		;setup for ADD
 	JR	CLKGO		;go common
 ;
 ;	turn clock display OFF
 ;
 DOCOFF	LD	B,0		;setup for REMOVE
 ;
 CLKGO	LD	A,NMICTL	;SVC #
 	LD	C,SLOTCLK	;slot number for clock
 	LD	DE,(CLKTASK)	;interrupt task
 	RST	SVC		;enable/disable it
 	XOR	A		;set Z
 	RET			;done
 ;
 	PAGE
 ;
 ;	$DEBUG	- activate/deactivate DEBUGGER
 ;
 ;		DEBUG
 ;		DEBUG,OFF	(ON=OFF,OFF=ON)
 ;		DEBUG,ON	(ON=ON,OFF=OFF)
 ;		DEBUG,BREAK	(BREAK=ON)
 ;		DEBUG,BREAK=OFF
 ;		DEBUG,ON,BREAK
 ;
 ;	shorthand
 ;
 ;		DEBUG			(on, break=y)
 ;		DEBUG,N 		(off)
 ;		DEBUG,B=N		(on, brk=n)
 ;
 ;	ENTRY	HL => parameter string
 ;
 ;	EXIT	debug flags set
 ;
 DEBUG	LD	A,80H		;set non evaluated
 ;
 ;	reset parameter flags
 ;
 	LD	(PON),A		;debug ON flag
 	LD	(POFF),A	;debug OFF flag
 	LD	(DBGBRK),A	;debug on BREAK flag
 ;
 ;	check input parameters
 ;
 	LD	DE,DBGPAR	;debug parameter block
 	LD	A,PARAM 	;get the params
 	RST	SVC		;call $PARAM
 	JP	NZ,EXIT		;error, return
 ;
 ;	check input
 ;
 ;	check user input, check OFF params
 ;
 	LD	HL,FLAGK1	;point to system flag
 	LD	A,(POFF)	;get off parameter
 	INC	A		;OFF, OFF=YES
 	JR	Z,DODOFF	;do debug OFF
 	DEC	A		;OFF=NO
 	JR	Z,DODON 	;do debug ON
 ;
 ;	check for ON params
 ;
 	LD	A,(PON)		;get ON parmeter
 	INC	A		;ON, ON=YES
 	JR	Z,DODON 	;do debug ON
 	DEC	A		;ON=NO
 	JR	Z,DODOFF	;do debug off
 ;
 ;	turn debugger ON
 ;
 DODON	SET	2,(HL)		;turn on debug
 	LD	A,-1		;set TRUE
 	JR	DODCONT 	;continue
 ;
 ;	turn debugger OFF
 ;
 DODOFF	RES	2,(HL)		;turn off debug
 	XOR	A		;set FALSE
 ;
 DODCONT LD	(FLAG1S),A	;set debug trigger
 	LD	(FLAG2S),A	;set debug saver
 ;
 	LD	A,(DBGBRK)	;get break flag
 	INC	A		;BREAK=ON
 	JR	Z,DBGBON	;turn it on
 	DEC	A		;BREAK=OFF
 	JR	Z,DBGBOFF	;turn it off
 ;
 ;	turn on debug trigger with break
 ;
 DBGBON	RES	0,(HL)		;allow BREAK debug
 	JR	DBGGO		;continue
 ;
 ;	turn off debug trigger with break
 ;
 DBGBOFF SET	0,(HL)		;no debug on BREAK
 ;
 DBGGO	XOR	A		;set Z
 	RET			;back to caller
 ;
 	PAGE
 ;
 ;	$VERIFY - activate/deactivate disk write verify
 ;
 ;			VERIFY
 ;			VERIFY,ON	(ON=ON,OFF=OFF)
 ;			VERIFY,OFF	(ON=OFF,OFF=ON)
 ;
 ;	shorthand
 ;
 ;			VERIFY		(turns on)
 ;			VERIFY,N	(turns off)
 ;
 VERIFY	CALL	ONOFF		;evaluate user input
 	LD	HL,FLAGK1
 ;
 	SET	3,(HL)		;turn it on
 	JR	Z,VEROK		;done, return
 ;
 DOVOFF	RES	3,(HL)		;turn it off
 ;
 VEROK	XOR	A		;set Z
 	RET			;done
 ;
 	PAGE
 ;
 ;	$ERROR	- send error message to channel
 ;
 ;			ERROR
 ;			ERROR value
 ;
 ;	shorthand
 ;
 ;			ERROR		(send last error)
 ;			ERROR value	(send val. error)
 ;
 ERROR	CALL	POSHL		;any input?
 	LD	A,(LASTERR)	;use last error if not
 	JR	Z,SHOWERR	;display it
 ;
 	CALL	VAL		;else get value
 	JP	NZ,EXIT		;error, show error
 	LD	A,C		;get the error num
 ;
 ;	display error message in A
 ;
 SHOWERR AND	7FH		;mask high bit off
 	LD	B,A		;put it here for error
 	LD	A,SERROR	;SVC #
 	RST	SVC		;call $ERROR/SYS
 ;
 	XOR	A		;else for ZERO
 	RET			;done
 ;
 ;
 	PAGE
 ;
 ;	$ONOFF	- check for ON/OFF user parameters
 ;
 ;	ENT	HL => user parameters
 ;
 ;	EXT	Z = ON
 ;		NZ = OFF
 ;		if invalid parameters found,
 ;		stack is popped, and an exit
 ;		is made to $EXIT with the error code
 ;
 ONOFF	LD	A,80H		;set non-evaluated
 ;
 ;	reset parameter flags
 ;
 	LD	(PON),A 	;ON parameter
 	LD	(POFF),A	;OFF parameter
 ;
 ;	check for input parameters
 ;
 	LD	DE,OFPAR	;ON/OFF param block
 	LD	A,PARAM 	;SVC #
 	RST	SVC		;load param blocks
 	JR	NZ,ONOFFB	;bad parameters
 ;
 ;	check user input, check for OFF params
 ;
 	LD	A,(POFF)	;get OFF param
 	INC	A		;OFF, OFF=YES
 	JR	Z,ONOFFN	;set NO
 	DEC	A		;OFF=NO
 	JR	Z,ONOFFY	;set YES
 ;
 ;	nil input, check for ON params
 ;
 	LD	A,(PON) 	;get ON param
 	INC	A		;ON, ON=YES
 	JR	Z,ONOFFY	;set YES
 	DEC	A		;ON=NO
 	JR	Z,ONOFFN	;set NO
 ;
 ;	no input, default to YES
 ;
 ONOFFY	XOR	A		;set Z for YES
 	RET			;return to caller
 ;
 ;	set NO
 ;
 ONOFFN	OR	-1		;set NZ for NO
 	RET			;return to caller
 ;
 ;	invalid parameters
 ;
 ONOFFB	POP	DE		;remove caller return
 	JP	EXIT		;back to LIB caller
 ;
 ;	extended block for DEBUG use
 ;
 DBGPAR	DEFB	20H+4		;switch + length-1
 	DEFW	DBGBRK		;vector
 	DEFM	'BREAK'		;param
 ;
 ;	ON/OFF parameter block
 ;
 OFPAR	DEFB	20H+1		;switch + length
 	DEFW	PON		;ON data vector
 	DEFM	'ON'            ;parameter
 ;
 	DEFB	20H+2		;switch + length-1
 	DEFW	POFF		;OFF data vector
 	DEFM	'OFF'           ;parameter
 ;
 	DEFB	20H+2		;switch + length
 	DEFW	PON		;ON vector
 	DEFM	'YES'		;param
 ;
 	DEFB	20H+1		;switch + length-1
 	DEFW	POFF		;OFF vector
 	DEFM	'NO'
 ;
 	DEFB	0		;terminator
 ;
 ;	data area for params
 ;
 PON	DEFW	8080H		;vector for ON
 POFF	DEFW	8080H		;vector for OFF
 DBGBRK	DEFW	8080H		;vector debug on BREAK
 ;
 	PAGE
 ;
 ;	$OUTCR	- output CR to device
 ;
 ;	$OUTPUT - output B to logical device
 ;
 ;	ENT	B = character to output
 ;		(ddcb) loaded with channel
 ;
 ;	EXT	Z = OK, B sent to device
 ;		NZ = A = error code
 ;
 ;	$INPUT	- input B from logical device number
 ;
 ;	ENT	(sdcb) loaded with channel
 ;
 ;	EXT	Z = OK, B = character
 ;		NZ = A = error code
 ;
 OUTCR	LD	B,CR		;send C/R
 ;
 ;	output B to device
 ;
 OUTPUT	PUSH	DE		;save this
 	LD	DE,DDCB+1	;point to DCB
 	LD	A,PUT		;SVC #
 	RST	SVC		;send it
 	POP	DE		;restore stack
 	RET			;Z = OK
 ;
 	PAGE
 ;
 ;	$OPENO	- open device for output
 ;	$DCLOSE - close device
 ;
 ;	ENT	DE => DCB returned by $FSPEC
 ;
 ;	EXT	Z = OK
 ;		NZ = A = error code
 ;
 ; (NOTE) devices 0-7 (character devices) are not
 ;	attempted to be opened
 ;
 OPENO	LD	A,(DE)		;get device number
 	AND	0FH		;bits 4-0 = device
 	CP	8		;devices 0-7?
 	JR	C,OPENOK	;open the device
 	INC	DE		;set to normal DCB
 	LD	HL,OLIST	;OPEN param list
 	LD	A,OPEN		;SVC #
 	RST	SVC		;call it
 	RET			;Z = OK
 ;
 OPENOK	CALL	LOCDCB		;find the DCB
 	LD	(DDCB+4),IX	;save it
 	LD	A,10H		;set ROUTE
 	LD	(DDCB+1),A
 	XOR	A		;set Z for OK
 	RET			;done
 ;
 DCLOSE	LD	A,(DE)		;get device
 	AND	0FH		;bits 4-0 = device
 	CP	8		;0-7 ?
 	JR	C,CLOSEOK	;no need to close it
 	INC	DE		;point to DCB
 	LD	A,CLOSE 	;SVC #
 	RST	SVC		;close the file
 	RET			;Z = OK
 CLOSEOK XOR	A		;set Z
 	RET
 ;
 ;
 ;	definition of library data area
 ;
 DDCB	DEFS	41		;source info
 ;
 ;	definition of $EVAL param block
 ;
 EBLOCK	DB	0		;work byte, +0
 	DW	DDCB		;source, +1,2
 	DW	DDCB		;dest, +3,4
 	DW	BUFADR		;mask, dummy
 EPARM	DW	0		;parameter list, +9,10
 PARNIL	DB	0		;nil parameter list
 ;
 ;	$OLIST	- $OPEN parameter list
 ;
 OLIST	DW	BUFADR		;+0,1 - buffer address
 	DW	RECADR		;+2,3 - record address
 	DW	0		;+4,5 - EODAD
 	DB	'R'             ;+6 - read/write
 	DB	1		;+7 - logical record len
 	DB	'F'             ;+8 - fixed length file
 	DB	3		;+9 - open type
 	DB	0		;+10 - param list term.
 RECADR	DB	0		;bytes transferred here
 ;
 ;
 ;	$EVALIT - evaluate LIB expression
 ;
 ;	ENT	HL => command string
 ;		DE = $PARAM block address
 ;
 ;	EXT	(EBLOCK) loaded, and exit to $EVAL
 ;		Z = OK, EBLOCK loaded
 ;		NZ = A = error code
 ;
 EVALIT	LD	IX,EBLOCK	;point to eval block
 	LD	(EPARM),DE	;insert $PARAM block
 	LD	A,EVAL		;SVC #
 	RST	SVC		;call $EVAL
 	RET			;back to caller, Z = OK
 ;
 ;
 VAL	LD	A,1-1<4+13	;sys1, # 13
 	RST	OVLY		;get it
 ;
 ;	$POSHL	- position HL to first significant char
 ;
 	INC	HL
 POSHL	LD	A,(HL)		;get a char
 	CP	' '		;separator?
 	JR	Z,POSHL-1
 	CP	','		;sep?
 	JR	Z,POSHL-1
 ;
 CKTERM	CP	13
 	RET	Z
 	CP	';'
 	RET	Z
 	CP	3
 	RET
 ;
 ;
 	PAGE
 ;
 ;	$GETCHA	- fetch output channel
 ;
 GETCHA	LD	(DDCB),A	;setup the device
 ;
 ;	evaluate user input
 ;
 	LD	DE,PARNIL	;nil parameters
 	CALL	EVALIT		;evaluate expression
 	RET	NZ		;error, return
 ;
 ;	check for valid output channel
 ;
 ;	open the channel for ouput
 ;
 	LD	DE,DDCB 	;dest DCB
 	JP	OPENO		;open for ouput
 ;
 	PAGE
 ;
 ;	$GETDRV	- fetch drive number from string
 ;
 GETDRV	LD	A,FSPEC		;evaluate the drive #
 	LD	DE,DDCB		;use DCB area
 	RST	SVC		;fetch it
 	RET	NZ		;error, return
 ;
 ;	check for valid drive number
 ;
 	LD	A,(DE)		;get flag byte
 	BIT	7,A		;device evaluated?
 	JR	Z,IERR		;bad drive number
 	AND	0FH		;get device number
 	CP	8		;0-7?
 	JR	C,IERR		;not a drive number
 ;
 ;	have valid device number in A
 ;
 	SUB	8		;make it device 0-7
 	LD	C,A		;need it here
 	XOR	A		;set Z
 	RET			;done, return
 ;
 IERR	LD	A,ILLDRV	;illegal drive number
 	OR	A		;set NZ
 	RET			;back to caller
 ;
 	PAGE
 ;
 ;	$TIME	- fetch/load TIME$
 ;
 TIME	CALL	POSHL		;any input?
 	JR	Z,SHOTIME	;display time if not
 ;
 ;	load TIME$
 ;
 	LD	B,2		;command #
 	LD	A,SDATE		;SVC #
 	RST	SVC		;load it
 ;
 	JP	NZ,EXIT		;error, display it
 	XOR	A		;force ZERO
 	RET			;done
 ;
 ;	fetch TIME$
 ;
 SHOTIME	LD	B,0		;command #
 	LD	A,SDATE		;SVC #
 	LD	HL,BUFADR	;place data here
 	RST	SVC		;fetch it
 	JP	NZ,EXIT		;error, display it
 ;
 	LD	DE,15		;offset to time
 	ADD	HL,DE		;point to it
 	LD	BC,08<8+CR	;length + term
 	LD	A,VDLINE	;SVC #
 	RST	SVC		;display it
 ;
 	JP	NZ,EXIT		;error, display it
 	XOR	A		;force ZERO
 	RET			;done
 ;
 	PAGE
 ;
 ;	$DATE	- fetch/load DATE$
 ;
 DATE	CALL	POSHL		;any input?
 	JR	Z,SHODATE	;display date if not
 ;
 ;	load DATE$
 ;
 	LD	B,1		;command #
 	LD	A,SDATE		;SVC #
 	RST	SVC		;load it
 ;
 	JP	NZ,EXIT		;error, exit
 	XOR	A		;force ZERO
 	RET			;done
 ;
 ;	fetch DATE$
 ;
 SHODATE	LD	B,0		;command #
 	LD	A,SDATE		;SVC #
 	LD	HL,BUFADR	;data goes here
 	RST	SVC		;fetch it
 	JP	NZ,EXIT		;error, abort
 ;
 	LD	DE,DATE0	;where it goes
 	PUSH	DE		;save string start
 	LD	BC,3		;3 chars to move
 	LDIR			;move name of day
 ;
 	INC	DE		;bump to next field
 	INC	DE
 	INC	DE
 	LD	BC,3		;3 chars
 	LDIR			;move month name
 ;
 	INC	DE		;next field
 	LD	BC,2		;2 chars
 	LDIR			;move day of month
 ;
 	INC	DE		;next field
 	INC	DE
 	LD	BC,4		;4 chars
 	LDIR			;move year
 ;
 	INC	DE		;next field
 	INC	DE
 	INC	DE
 	LD	BC,3		;3 chars
 	LDIR			;move day number in year
 ;
 ;	display it
 	POP	HL		;get string start back
 	LD	BC,24<8+CR	;length + term
 	LD	A,VDLINE	;SVC #
 	RST	SVC		;display it
 ;
 	JP	NZ,ERROR	;error, display it
 	XOR	A		;force ZERO
 	RET			;done, go back
 ;
 ;	date text string
 ;
 DATE0	DEFM	'day - '
 DATE1	DEFM	'mon '
 DATE2	DEFM	'##, '
 DATE3	DEFM	'1980 - '
 DATE4	DEFM	'###'
 ;
 	PAGE
 ;
 ;	$SCREEN	- send contents of video to device
 ;
 SCREEN	LD	A,PRDCB		;default device
 	CALL	GETCHA		;get output channel
 	JP	NZ,EXIT		;error, abort
 ;
 	LD	DE,DDCB+1	;point to DCB
 	LD	A,SCRSVC	;screen SVC #
 	RST	SVC		;send to device
 	JP	NZ,EXIT		;error, abort
 ;
 	LD	DE,DDCB		;point to DCB
 	CALL	DCLOSE		;close it up
 	JP	NZ,EXIT		;error, abort
 	XOR	A		;force ZERO
 	RET			;back to caller
 ;
 ;	$I	- init disk drives
 ;
 I@	CALL	POSHL		;any input?
 	JR	Z,DESEL		;init all drives
 ;
 	CALL	GETDRV		;fetch the drive number
 	JP	NZ,EXIT		;error, abort
 	CALL	NIL		;nil the drive
 ;
 	XOR	A		;done, return ZERO
 	RET
 ;
 ;	init all drives
 ;
 DESEL	LD	BC,0800H	;B=looper, C=drive
 DESELL	CALL	NIL		;this drive
 	INC	C		;bump drive
 	DJNZ	DESELL		;do 'em all
 ;
 	XOR	A		;done
 	RET
 ;
 NIL	CALL	LOCDCT		;find the DCT
 	SET	0,(IY+5)	;force init next time
 	RET			;done
 	PAGE
 ;
 ;	$AUTO	- setup/disable AUTO command
 ;
 AUTO	CALL	POSHL		;any input?
 	LD	C,0		;turn off drive 0 if yes
 	JR	Z,NOAUTO	;go!, no input
 ;
 ;	check for drive number
 ;
 	CP	':'		;drive specifier?
 	JR	NZ,SETAUTO	;turn it on, use drive 0
 ;
 	CALL	GETDRV		;get the drive number
 	JP	NZ,EXIT		;error, abort
 	CALL	POSHL		;any more input?
 	JR	Z,NOAUTO	;nope, turn auto off
 ;
 ;	set auto command, drive number in C (0-7)
 ;
 SETAUTO	LD	DE,DDCB		;use this area for string
 	LD	B,32		;32 chars max
 ;
 FILAUTO	LD	A,(HL)		;get a character
 	CALL	CKTERM		;terminator?
 	JR	Z,AUTOT		;yes, done
 ;
 	LD	(DE),A		;put in buffer
 	INC	HL		;bump pointers
 	INC	DE
 	DJNZ	FILAUTO		;fill more
 ;
 AUTOT	LD	A,CR		;put in CR
 	LD	(DE),A		;into the end
 ;
 	INC	B		;buffer filled?
 FILATO	DEC	B		;yes?
 	JR	Z,PUTAUTO	;done, put on disk
 	INC	DE		;bump pointer
 	LD	(DE),A		;put in a space
 	JR	FILATO		;go more
 ;
 ;
 ;	write command to disk
 ;
 PUTAUTO	CALL	READGAT		;read the GAT table
 	JP	NZ,EXIT		;disk error, abort
 ;
 ;	move auto command into buffer
 ;
 	PUSH	BC		;save drive number
 	LD	HL,DDCB		;data is here
 	LD	DE,BUFADR+0E0H	;offset for command
 	LD	BC,32		;auto length
 PUTBAK	LDIR			;move it in
 	POP	BC		;restore drive
 ;
 	CALL	WRITGAT		;write back to disk
 	JP	NZ,EXIT		;error, abort
 ;
 	XOR	A		;force ZERO
 	RET			;done
 ;
 ;	disable AUTO command
 ;
 NOAUTO	CALL	READGAT		;read the GAT
 	JP	NZ,EXIT		;error, abort
 	PUSH	BC		;save drive number
 ;
 	LD	HL,BUFADR+0E0H	;command start
 	LD	DE,BUFADR+0E1H
 	LD	(HL),CR		;fill with CR's
 	LD	BC,31		;length -1
 	JR	PUTBAK		;write it back
 ;
 ;	read gat table into (BUFADR)
 ;
 READGAT	LD	HL,BUFADR	;data dest
 	LD	B,0		;set sector to 000
 	LD	E,B		;BE = sector 0
 	LD	A,DREAD		;directory read
 	RST	DISKIO		;read it
 	RET			;back to caller
 ;
 ;	write gat table from (BUFADR)
 ;
 WRITGAT	LD	HL,BUFADR	;data source
 	LD	B,0		;set sector 000
 	LD	E,B
 	LD	A,DWRIT		;directory write
 	RST	DISKIO		;write it back
 	RET			;done, return
 ;
 BUFADR	DEFS	100H		;I/O buffer
 	DB	0		;end of program
 ;
 	END	VECTORS
