DOSSYS ; dossys13/asm - kjw/bqsd - 07/82
 ;
 	TITLE	'<DosPLUS II.A - SYS13/SYS>'
 ;
 ;	system is high residing @ 2200H and has:
 ;
 ;		$SETCOM	- setup comm lines
 ;		$DO	- chain @KI from file
 ;		$FORMS	- setup printer
 ;
 	PAGE
 ;
 ;	support
 ;
 FEXT	EQU	88		;add extension
 PARAM	EQU	87		;fetch params
 PRCHAR	EQU	18		;char to printer
 BINDEC	EQU	21		;binary<>decimal
 VDLINE	EQU	9		;line to video
 ERROR	EQU	39		;display error
 EVAL	EQU	83		;evaluate command line
 RS232C	EQU	55		;init RS232
 OPEN	EQU	40		;open file
 ;
 FETMEM	EQU	0330H		;fetch high memory
 LOMEM	EQU	017EH		;low memory
 HIMEM	EQU	0180H		;high memory
 UCASE	EQU	031EH		;A => upper case
 LOCDCB	EQU	032AH		;locate DCB
 DCB	EQU	01BCH		;use run DCB
 ;
 BADF	EQU	1		;bad function code
 TOF	EQU	0CH		;printer top of form
 PDCB	EQU	2		;printer DCB #
 KDCB	EQU	0		;keyboard DCB #
 ;
 SVC	EQU	08H		;execute SVC
 ;
 	PAGE
 ;
 	ORG	2200H		;high overlay
 ;
 VECTORS	DW	RETURN		;0 - nil
 	DW	SETCOM		;1
 	DW	DO		;2
 	DW	FORMS		;3
 	DW	UNDEF		;4 - undefinded
 	DW	UNDEF		;5
 	DW	UNDEF		;6
 	DW	UNDEF		;7
 	DW	UNDEF		;8
 	DW	UNDEF		;9
 	DW	UNDEF		;10
 	DW	UNDEF		;11
 	DW	UNDEF		;12
 	DW	UNDEF		;13
 	DW	UNDEF		;14
 	DW	UNDEF		;15
 ;
 ;
 ;	undefined vector
 ;
 UNDEF	LD	A,BADF		;bad function call
 	OR	A		;set NZ
 	RET			;back to caller
 ;
 ;
 EXIT	AND	7FH		;force a return
 	LD	B,A		;pass error
 EXITB	LD	A,ERROR		;SVC #
 	RST	SVC		;display error
 RETURN	XOR	A		;force ZERO
 	RET			;back to caller
 ;
 	PAGE
 ;
 ;	$SETCOM	- setup comm line
 ;
 SETCOM	CALL	POSHL		;any input
 	JP	Z,SHOWCOM	;nil, display settings
 ;
 ;	setup default parameters
 ;
 	LD	BC,300		;300 baud
 	LD	(BAUPAR),BC	;save it
 	LD	BC,8		;8 bit words
 	LD	(WORPAR),BC
 	LD	(NILPAR),BC	;set NOT 00 or FF
 	LD	BC,0		;parity string
 	LD	(PARPAR),BC
 	LD	(BSPAR),BC	;buffer size
 	INC	C		;bump counter
 	LD	(STOPAR),BC	;stop bits
 	LD	BC,0		;buffer size (nil)
 	LD	(BSPAR),BC
 ;
 ;	evaluate input params
 ;
 	LD	IX,COMBLK	;setcom EVAL block
 	LD	A,EVAL		;SVC #
 	RST	SVC		;evaluate input
 	JR	NZ,EXIT		;error, abort
 ;
 	LD	A,(DCB)		;fetch device
 	LD	B,81		;invalid channel
 	BIT	7,A		;device here?
 	JR	Z,EXITB		;abort if not
 	BIT	6,A		;filespec here?
 	JR	NZ,EXITB	;abort if yes
 ;
 	AND	0FH		;get device number
 	LD	C,'A'		;channel A
 	SUB	3		;device 3 (ch A?)
 	JR	Z,GOSETC	;continue if yes
 ;
 	INC	C		;channel B
 	DEC	A		;device 4 (ch B?)
 	JR	NZ,EXITB	;nope, error
 ;
 ;	correct device issued, build params
 ;
 GOSETC	LD	HL,DCB		;re-use it
 	LD	(HL),C		;put in channel
 ;
 ;	fetch baud rate
 ;
 	LD	DE,(BAUPAR)	;get baud rate
 	LD	IX,BAUDTBL	;baud lookup table
 	LD	BC,0801H	;B=looper, C=baud #
 ;
 FINBAUD	LD	L,(IX)		;get a value
 	LD	H,(IX+1)	;HL = baud value
 	OR	A		;clear carry
 	SBC	HL,DE		;compare 'em
 	JR	Z,HAVBAUD	;have it, go!
 ;
 	INC	IX		;next value
 	INC	IX
 	INC	C		;bump baud rate
 	DJNZ	FINBAUD		;continue
 ;
 ;	invalid baud rate, error
 ;
 	LD	A,83		;invalid data
 	JR	EXIT		;error, abort
 ;
 ;	have baud rate, put in table
 ;
 HAVBAUD	LD	HL,DCB+1	;next area
 	LD	(HL),C		;put in the baud rate
 	INC	HL		;bump pointer
 ;
 ;	fetch word length
 ;
 	LD	A,(WORPAR)	;get param
 	LD	B,9		;invalid data
 	CP	5		;must be 5-8
 	JP	C,EXITB		;error
 	CP	9
 	JP	NC,EXITB	;error
 	LD	(HL),A		;put in table
 	INC	HL		;bump pointer
 ;
 ;	fetch parity
 ;
 	LD	DE,(PARPAR)	;get string pointer
 	LD	A,D		;anything?
 	OR	E
 	LD	A,'N'		;default none
 	JR	Z,SETPARI	;go
 	LD	A,(DE)		;get the char
 	CALL	UCASE		;make it upper case
 ;
 	CP	'O'		;odd?
 	JR	Z,SETPARI	;yes, go
 	CP	'E'		;even?
 	JR	Z,SETPARI	;yes, go
 	CP	'N'		;none?
 	JP	NZ,EXITB	;nope, error
 ;
 SETPARI	LD	(HL),A		;put in buffer
 	INC	HL		;bump pointer
 ;
 ;	fetch # stop bits
 ;
 	LD	A,(STOPAR)	;get value
 	OR	A		;0?
 	JP	Z,EXITB		;invalid
 	CP	3		;1-2?
 	JP	NC,EXITB	;invalid
 	LD	(HL),A		;put into buffer
 	INC	HL		;bump pointer
 ;
 ;	insert table terminator
 ;
 	LD	(HL),0		;terminator
 ;
 ;	check for buffer size
 ;
 	LD	A,(DCB)		;get channel
 	SUB	'A'		;remove ascii
 	ADD	A,3		;make it read
 	CALL	LOCDCB		;load IX with it
 ;
 	LD	BC,(BSPAR)	;get param
 	LD	A,B		;anything?
 	OR	C
 	JR	Z,CKNIL		;check for nil
 ;
 	CALL	FETMEM		;fetch top memory
 	JP	NZ,EXIT		;nope, abort
 ;
 ;	load new buffer into DCB
 ;
 	CALL	NILBUFF		;set to nil
 ;
 ;	check for NIL
 ;
 CKNIL	LD	A,(NILPAR)	;get param
 	INC	A		;yes?
 	LD	B,0		;for remove
 	JR	Z,INITR		;yes, go!
 ;
 	DEC	B		;B = -1 for add
 ;
 ;	setup parameters
 ;
 INITR	LD	HL,DCB		;HL => param list
 	LD	A,RS232C	;SVC #
 	RST	SVC		;init RS232
 	JP	NZ,EXIT		;error, display why
 ;
 	XOR	A		;force ZERO
 	RET			;done
 ;
 	PAGE
 ;
 ;	display comm parameters
 ;
 SHOWCOM	LD	A,3		;channel A DCB#
 	CALL	SHOWC		;display channel
 	JP	NZ,EXIT		;error abort
 	LD	A,4		;channel B
 	CALL	SHOWC		;display channel
 	JP	NZ,EXIT		;error, abort
 ;
 	XOR	A		;force ZERO
 	RET			;done, back to caller
 ;
 SHOWC	LD	C,A		;save channel
 	CALL	LOCDCB		;find the DCB
 ;
 	LD	A,C		;fetch it back
 	ADD	A,'0'		;make it a device #
 	LD	(COM0),A	;to the string
 ;
 ;	fetch real name
 ;
 	LD	A,C		;get device
 	ADD	A,A		;*2
 	ADD	A,A		;*4
 	ADD	A,2		;+2 for offset to name
 	LD	L,A		;give it to L
 	LD	H,1		;start of table
 	LD	A,(HL)		;get a char
 	LD	(COM00),A	;to the string
 	INC	HL
 	LD	A,(HL)		;next
 	LD	(COM00+1),A	;to the string
 ;
 	LD	HL,COMMSG	;string
 	LD	BC,COMMSGL<8+0	;len + term
 	LD	A,VDLINE	;SVC #
 	RST	SVC		;display it
 	RET	NZ		;error
 ;
 ;	check for nil device
 ;
 	BIT	1,(IX+5)	;active?
 	LD	HL,NILMSG	;message
 	LD	BC,NILMSGL<8+13	;len + term
 	LD	A,VDLINE	;SVC #
 	JP	NZ,SVC		;done, display
 ;
 ;	fetch parameters from DCB
 ;
 ;	fetch baud
 ;
 	LD	A,(IX+25)	;param byte
 	RRCA			;align it
 	RRCA
 	RRCA
 	AND	0EH		;A = BAUD# times 2
 	LD	E,A		;give it to DE
 	LD	D,0
 	LD	HL,BAUDTBL	;baud lookup table
 	ADD	HL,DE		;HL => baud rate
 	LD	E,(HL)		;fetch it
 	INC	HL
 	LD	D,(HL)
 	LD	HL,COM1		;text where it goes
 	LD	B,0		;for SVC
 	LD	A,BINDEC	;bin > dec ascii
 	RST	SVC		;get ascii into string
 ;
 ;	fetch word length
 ;
 	LD	A,(IX+25)	;get word length
 	AND	0CH		;A = binary word len
 	ADD	A,'0'		;make it ascii
 	LD	(COM2),A	;to the string
 ;
 ;	fetch parity
 ;
 	LD	B,(IX+25)	;fetch param
 	LD	A,'N'		;any parity?
 	RR	B		;check bit 0
 	JR	NC,GOTPAR	;have parity
 	LD	A,'O'		;odd?
 	RR	B		;check bit 1
 	JR	NC,GOTPAR	;have it
 	LD	A,'E'		;else even
 ;
 GOTPAR	LD	(COM3),A	;to the string
 ;
 ;	check for stop bits
 ;
 	LD	A,(IX+5)	;get param
 	RRCA			;align it
 	RRCA
 	AND	3		;0-3 only
 	ADD	A,'0'		;make it ascii
 	LD	(COM4),A	;to the string
 ;
 ;	fetch buffer size
 ;
 	LD	E,(IX+11)	;get LSB size
 	LD	D,(IX+12)	;MSB
 	LD	HL,COM5		;text
 	LD	B,0		;convert direction
 	LD	A,BINDEC	;binary > decimal
 	RST	SVC		;load the string
 ;
 ;	fetch buffer count
 ;
 	LD	E,(IX+13)	;get counter
 	LD	D,(IX+14)
 	LD	HL,COM6		;text
 	LD	B,A		;direction (0)
 	LD	A,BINDEC	;bin > dec
 	RST	SVC		;fill it
 ;
 ;	string filled, display it
 ;
 	LD	HL,COMM		;text
 	LD	BC,COMML<8+13	;len + term
 	LD	A,VDLINE	;SVC #
 	RST	SVC		;display it
 	RET			;done, return status
 ;
 	PAGE
 ;
 ;	$FORMS	- setup printer commands
 ;
 FORMS	LD	A,PDCB		;DCB #
 	CALL	LOCDCB		;find it
 	CALL	POSHL		;any input?
 	JP	Z,SHOWFOR	;no, display settings
 ;
 ;	clear out data blocks
 ;
 	PUSH	HL		;save pointer
 	LD	HL,TOPPAR	;start of data
 	LD	B,22		;length
 FILLP	LD	(HL),80H	;nil it
 	INC	HL		;bump pointer
 	DJNZ	FILLP		;do whole block
 	LD	HL,0		;special
 	LD	(BSPAR),HL	;buffer size
 	LD	(CODPAR),HL	;code
 	POP	HL		;restore pointer
 ;
 ;	evaluate input command
 ;
 	LD	DE,FORPAR	;forms params
 	LD	A,PARAM		;SVC #
 	RST	SVC		;fetch 'em
 	JP	NZ,EXIT		;error, abort
 ;
 	LD	A,(TOPPAR)	;TOP?
 	INC	A		;yes, FF?
 	JR	NZ,NOTTOP	;no, continue
 ;
 ;	send TOF out immediately
 ;
 	LD	B,TOF		;ascii
 	LD	A,PRCHAR	;send to printer
 	RST	SVC		;do it
 	JP	NZ,EXIT		;error, abort
 ;
 NOTTOP	LD	HL,(CODPAR)	;any code issued?
 	LD	A,H		;anything?
 	OR	L
 	JR	Z,CKPAG		;nope, continue
 ;
 	LD	B,L		;get LSB
 	LD	A,PRCHAR	;SVC #
 	RST	SVC		;print it
 	JP	NZ,EXIT		;error
 	LD	A,H		;get MSB
 	OR	A		;2 bytes long?
 	JR	Z,CKPAG		;nope, forget it
 	LD	B,A		;put it here
 	LD	A,PRCHAR	;SVC
 	RST	SVC
 	JP	NZ,EXIT		;both done
 ;
 ;	check for page length
 ;
 CKPAG	LD	A,(PAGPAR+1)	;get MSB
 	OR	A		;zero?
 	JR	NZ,CKLIN	;nothing
 ;
 	LD	A,(PAGPAR)	;get data
 	LD	(IX+20),A	;put in DCB
 ;
 ;	check for lines/page
 ;
 CKLIN	LD	A,(LINPAR+1)	;get MSB
 	OR	A		;zero?
 	JR	NZ,CKWID	;nil, check width
 ;
 	LD	A,(LINPAR)	;get data
 	LD	(IX+19),A	;put into DCB
 ;
 ;	check for page width
 ;
 CKWID	LD	A,(WIDPAR+1)	;get MSB
 	OR	A		;anything?
 	JR	Z,CKXLA		;no, check xlate
 ;
 	LD	A,(WIDPAR)	;get data
 	LD	(IX+22),A	;put into DCB
 ;
 ;	check for xlate (transparent)
 ;
 CKXLA	LD	A,(XLAPAR)	;get param
 	INC	A		;yes, FF?
 	JR	Z,XLAON		;yes, go
 	DEC	A		;no, 00?
 	JR	NZ,CKLF		;neither, go!
 ;
 XLAOFF	RES	3,(IX+5)	;turn it off
 	JR	CKLF		;go
 ;
 XLAON	SET	3,(IX+5)	;turn it on
 ;
 ;	check for linefeed on CR
 ;
 CKLF	LD	A,(LFPAR)	;get param
 	INC	A		;yes?
 	JR	Z,LFON		;yes, go!
 	DEC	A		;no?
 	JR	NZ,CKSER	;no, check serial
 ;
 LFOFF	RES	6,(IX+5)	;turn it off
 	JR	CKSER		;continue
 ;
 LFON	SET	6,(IX+5)	;turn it on
 ;
 ;	check for serial/parallel
 ;
 CKSER	LD	A,(SERPAR)	;get param
 	INC	A		;yes?
 	JR	Z,SETSER	;set serial
 	DEC	A		;no
 	JR	NZ,CKBS		;check buffer size
 ;
 SETPAR	RES	7,(IX+5)	;set parallel
 	JR	CKBS		;continue
 ;
 SETSER	SET	7,(IX+5)	;set serial
 ;
 ;	check for buffer size
 ;
 CKBS	LD	BC,(BSPAR)	;get param
 	LD	A,B		;anything?
 	OR	C
 	JR	Z,CKRES		;check for reset
 ;
 	CALL	FETMEM		;enough room?
 	JP	NZ,EXIT		;insufficient memory
 ;
 ;	setup new buffer
 ;
 	CALL	NILBUFF		;set to nil
 	JR	RESON		;force RESET param
 ;
 ;	check for reset
 ;
 CKRES	LD	A,(RESPAR)	;get param
 	INC	A		;yes?
 	JR	NZ,CKUCA	;check case switch
 ;
 RESON	LD	(IX+21),0	;line counter
 	LD	(IX+23),0	;char counter
 ;
 ;	check for upper case switch
 ;
 CKUCA	LD	A,(UCAPAR)	;get param
 	INC	A		;yes?
 	JR	Z,UCAON		;yes, go!
 	DEC	A		;no?
 	JR	NZ,CKNXT	;continue
 ;
 UCAOFF	SET	4,(IX+5)	;turn on lower
 	JR	CKNXT		;continue
 ;
 UCAON	RES	4,(IX+5)	;turn on upper
 ;
 CKNXT	XOR	A		;set ZERO
 	RET			;done
 ;
 	PAGE
 ;
 ;	display forms parameters
 ;
 SHOWFOR	LD	HL,PARMSG	;parallel?
 	LD	DE,PR0		;text area
 	LD	BC,8		;length
 	BIT	7,(IX+5)	;yes?
 	JR	Z,SHOW1		;yes, go
 	LD	DE,SERMSG	;serial
 ;
 SHOW1	LDIR			;move into string
 ;
 ;	page length
 ;
 	LD	HL,PR1		;text
 	LD	A,(IX+20)	;value
 	CALL	ASCII		;move it in
 ;
 ;	lines/page
 ;
 	LD	HL,PR2		;text
 	LD	A,(IX+19)	;line/page
 	CALL	ASCII		;move it in
 ;
 ;	page width
 ;
 	LD	HL,PR3		;text
 	LD	A,(IX+22)	;width
 	CALL	ASCII		;to the string
 ;
 ;	buffer size
 ;
 	LD	E,(IX+11)	;LSB
 	LD	D,(IX+12)
 	LD	HL,PR4		;text
 	LD	B,0		;direction
 	LD	A,BINDEC	;SVC #
 	RST	SVC
 ;
 ;	buffer count
 ;
 	LD	E,(IX+13)	;counter
 	LD	D,(IX+14)
 	LD	HL,PR5		;text
 	LD	B,A		;B = 0
 	LD	A,BINDEC	;SVC #
 	RST	SVC		;convert
 ;
 ;	string filled, display it
 ;
 	LD	HL,PRMSG	;text start
 	LD	BC,PRMSGL<8+13	;len + term
 	LD	A,VDLINE	;SVC #
 	RST	SVC		;display
 ;
 	XOR	A		;set ZERO
 	RET
 ;
 	PAGE
 ;
 ;	$DO	- chain a file to the keyboard
 ;
 DO	LD	DE,8080H	;reset the flag
 	LD	(BRKPAR),DE	;for BREAK
 	LD	IX,DOBLK	;block for EVAL
 	LD	A,EVAL		;SVC #
 	RST	SVC		;fill it
 	JP	NZ,EXIT		;param errors
 ;
 ;	fetch the DCB
 ;
 	LD	A,KDCB		;keyboard DCB
 	CALL	LOCDCB		;load it
 ;
 ;	check for valid channel
 ;
 	LD	A,(DCB)		;get flag byte
 	LD	B,80		;invalid channel
 	BIT	6,A		;filespec here?
 	JP	Z,EXITB		;nope, abort
 ;
 ;	check if chaining right now
 ;
 	LD	L,(IX+19)	;get current DCB
 	LD	H,(IX+20)
 	DEC	H		;point to buffer start
 	BIT	1,(IX+5)	;yes?
 	JR	NZ,NOWCHN	;don't need any memory
 ;
 ;	check for sufficient memory to chain
 ;
 	LD	BC,256+40	;need this much
 	CALL	FETMEM		;enough room?
 	JP	NZ,EXIT		;nope, error
 NOWCHN	LD	(OLIST),HL	;put into list
 	LD	(OLIST+2),HL	;start of buffer
 ;
 ;	add default extension of /TXT
 ;
 	LD	DE,DCB+1	;DCB
 	LD	HL,TXT		;text
 	LD	A,FEXT		;SVC #
 	RST	SVC		;add it on
 ;
 ;	open the file up
 ;
 	LD	HL,OLIST	;open list
 	LD	A,OPEN		;SVC #
 	RST	SVC		;find it
 	JR	NZ,RESMEM	;error, reset topmem
 ;
 ;	move DCB into correct area
 ;
 	LD	DE,(OLIST)	;get buffer start
 	INC	D		;+ length => DCB area
 	LD	HL,DCB+1	;open DCB right now
 	LD	BC,40		;length
 ;
 ;	save DCB address before upsetting DE
 ;
 	LD	(IX+19),E	;put into DCB
 	LD	(IX+20),D
 ;
 ;	move the open DCB up to topmem
 ;
 	LDIR			;move it!
 ;
 ;	check for BREAK parameter
 ;
 	LD	A,(BRKPAR)	;param
 	INC	A		;yes?
 	JR	Z,BRKON		;go
 	DEC	A		;no?
 	JR	NZ,DODONE	;done!
 ;
 BRKOFF	SET	6,(IX+5)	;turn it off
 	JR	DODONE		;continue
 ;
 BRKON	RES	6,(IX+5)	;turn it on
 ;
 ;
 ;	everything all set, turn on chaining flag
 ;
 DODONE	SET	1,(IX+5)	;chaining!
 	XOR	A		;set ZERO
 	RET			;all done!
 ;
 ;	error, reset topmem before exit
 ;
 RESMEM	LD	HL,(HIMEM)	;get what it is
 	LD	DE,256+40	;length we offset
 	ADD	HL,DE		;put it back
 	LD	(HIMEM),HL	;back in place
 	JP	EXIT		;now display error & exit
 ;
 	PAGE
 ;
 ;	$POSHL
 ;	position HL to input character
 ;
 	INC	HL		;bump pointer
 POSHL	LD	A,(HL)		;fetch a char
 	CP	','		;sep?
 	JR	Z,POSHL-1	;go if yes
 	CP	' '		;space?
 	JR	Z,POSHL-1	;skip if yes
 CKTERM	CP	13
 	RET	Z
 	CP	';'
 	RET	Z
 	CP	3
 	RET
 ;
 ;
 ;	convert char in A to (HL) in decimal ascii
 ;
 ASCII	LD	B,'0'		;start digit
 ASC1	SUB	100		;check 100's place
 	JR	C,ASC2		;go if done
 	INC	B		;bump ascii
 	JR	ASC1		;continue
 ;
 ASC2	PUSH	AF		;save result
 	LD	A,B		;check for leading 0
 	CP	'0'		;yes?
 	JR	NZ,ASC3		;no, go
 	LD	B,' '		;change to space
 ASC3	LD	(HL),B		;put char on video
 	INC	HL		;bump pointer
 	POP	AF		;get it back
 ;
 	ADD	A,100		;add it back
 	LD	B,'0'		;start again
 ;
 ASC4	SUB	10		;10's this time
 	JR	C,ASC5		;done, go
 	INC	B		;bump ascii
 	JR	ASC4		;go more
 ;
 ASC5	LD	(HL),B		;put in NSB digit
 	INC	HL		;bump pointer;
 	ADD	A,'0'+10	;last sub + ascii
 	LD	(HL),A		;to the string
 	RET			;done
 ;
 ;
 ;	$NILBUFF - clear out a spooled buffer
 ;
 NILBUFF	DI			;disable for change
 ;
 	XOR	A		;set ZERO
 	LD	(IX+13),A	;char count
 	LD	(IX+14),A
 	LD	(IX+15),A	;add offset
 	LD	(IX+16),A
 	LD	(IX+17),A	;take offset
 	LD	(IX+18),A
 	LD	(IX+12),B	;buffer length
 	LD	(IX+11),C
 	LD	(IX+10),H	;buffer start
 	LD	(IX+9),L
 	EI			;can enable now
 	RET			;done
 ;
 	PAGE
 ;
 ;	baud rate table
 ;
 BAUDTBL	DEFW	110
 	DEFW	150
 	DEFW	300
 	DEFW	600
 	DEFW	1200
 	DEFW	2400
 	DEFW	4800
 	DEFW	9600
 ;
 ;	parameter blocks
 ;
 COMBLK	DEFB	0		;flag byte
 	DEFW	DCB		;source
 	DEFW	DCB		;dest
 	DEFW	DCB		;all the same
 	DEFW	COMPAR		;param block
 ;
 COMPAR	DEFB	40H+3		;value + length
 	DEFW	BAUPAR		;vector for data
 	DEFM	'BAUD'		;param
 ;
 	DEFB	40H+3
 	DEFW	WORPAR
 	DEFM	'WORD'
 ;
 	DEFB	80H+5
 	DEFW	PARPAR
 	DEFM	'PARITY'
 ;
 	DEFB	40H+3
 	DEFW	STOPAR
 	DEFM	'STOP'
 ;
 	DEFB	40H+1
 	DEFW	BSPAR
 	DEFM	'BS'
 ;
 	DEFB	20H+2
 	DEFW	NILPAR
 	DEFM	'NIL'
 ;
 	DEFB	0
 ;
 ;	data
 ;
 BAUPAR	DEFW	0		;baud rate
 WORPAR	DEFW	0		;word length
 PARPAR	DEFW	0		;parity
 STOPAR	DEFW	0		;stop bits
 BSPAR	DEFW	0		;buffer size
 NILPAR	DEFW	0		;nil
 ;
 ;	text for display
 ;
 COMMSG	DEFM	'$'
 COM0	DEFM	'x  @'
 COM00	DEFM	'xx '
 COMMSGL	EQU	$-COMMSG
 ;
 COMM	DEFM	'Baud='
 COM1	DEFM	'xxxxx Word='
 COM2	DEFM	'x Parity='
 COM3	DEFM	'x Stop='
 COM4	DEFM	'x BuffSize='
 COM5	DEFM	'xxxxx Count='
 COM6	DEFM	'xxxxx'
 COMML	EQU	$-COMM
 ;
 NILMSG	DEFM	'Nil'
 NILMSGL	EQU	$-NILMSG
 ;
 	PAGE
 ;
 ;	parameters for FORMS
 ;
 FORPAR	DEFB	20H+2		;switch + length -1
 	DEFW	TOPPAR		;data vector
 	DEFM	'TOP'		;param
 ;
 	DEFB	40H+3
 	DEFW	CODPAR
 	DEFM	'CODE'
 ;
 	DEFB	40H+3
 	DEFW	PAGPAR
 	DEFM	'PAGE'
 ;
 	DEFB	40H+4
 	DEFW	LINPAR
 	DEFM	'LINES'
 ;
 	DEFB	20H+4
 	DEFW	XLAPAR
 	DEFM	'XLATE'
 ;
 	DEFB	20H+1
 	DEFW	LFPAR
 	DEFM	'LF'
 ;
 	DEFB	20H+5
 	DEFW	SERPAR
 	DEFM	'SERIAL'
 ;
 	DEFB	40H+1
 	DEFW	BSPAR
 	DEFM	'BS'
 ;
 	DEFB	20H+4
 	DEFW	RESPAR
 	DEFM	'RESET'
 ;
 	DEFB	20H+4
 	DEFW	UCAPAR
 	DEFM	'UCASE'
 ;
 	DEFB	0
 ;
 ;	data vectors
 ;
 TOPPAR	DEFW	0		;top of form
 CODPAR	DEFW	0		;code to printer
 PAGPAR	DEFW	0		;page length
 LINPAR	DEFW	0		;lines/page
 WIDPAR	DEFW	0		;line width
 XLAPAR	DEFW	0		;transparent
 LFPAR	DEFW	0		;LF on CR
 SERPAR	DEFW	0		;parallel/serial
 RESPAR	DEFW	0		;reset counters
 UCAPAR	DEFW	0		;upper case
 ;
 ;	text
 ;
 PRMSG	EQU	$
 PR0	DEFM	'xxxxxxxx Page='
 PR1	DEFM	'xxx Lines='
 PR2	DEFM	'xxx Width='
 PR3	DEFM	'xxx BuffSize='
 PR4	DEFM	'xxxxx Count='
 PR5	DEFM	'xxxxx'
 PRMSGL	EQU	$-PRMSG
 ;
 PARMSG	DEFM	'Parallel'
 SERMSG	DEFM	'Serial  '
 ;
 ;	$EVAL block for $DO
 ;
 DOBLK	DEFB	0		;work byte
 	DEFW	DCB		;source
 	DEFW	DCB		;dest
 	DEFW	DCB		;mask
 	DEFW	DOPAR		;param block
 ;
 DOPAR	DEFB	20H+4
 	DEFW	BRKPAR
 	DEFM	'BREAK'
 ;
 	DEFB	0
 ;
 BRKPAR	DEFW	0		;break param
 ;
 ;	parameter list for OPEN for $DO
 ;
 OLIST	DEFW	0000H		;bufadr
 	DEFW	0000H		;recadr
 	DEFW	0000H		;EODAD
 	DEFB	'R'		;read
 	DEFB	1		;record length
 	DEFB	'F'		;fixed length
 	DEFB	3		;open new LRL
 	DEFB	0		;terminator
 ;
 TXT	DEFM	'TXT'		;$DO default extension
 ;
 	END	VECTORS
