DOSSYS ; dossys14/asm
 ;
 	TITLE	'<DOS+II - SYS14/SYS - Series A.00>'
 ;
 ;	THIS SYSTEM IS HIGH RESIDING (2200-27FF)
 ;	IT CONTAINS THE FOLLOWING ROUTINES:
 ;
 ;	1 -	$PROT	- FILE PROTECTION
 ;	2 -	$ATTRIB - FILE ATTRIBUTES
 ;	3 -	$RENAME - RENAME DEVICE/FILE
 ;	4 -	$FREE	- FREE SPACE
 ;
 ;	Created: 17-JUN-82	SJP/MSS
 ;	Revised: 17-JUN-82	SJP/MSS
 ;
 	PAGE
 ;	LOCAL SUPPORT EQUIVALENCES
 ;
 BADF	EQU	1		;BAD FUNCTION CALL
 ERR3	EQU	3		;PARAMETER ERROR
 ERR8	EQU	8		;DRIVE NOT READY
 NCOL	EQU	4		;CAT COLS
 NROW	EQU	21		;CAT ROWS
 CR	EQU	13		;ASCII C/R
 ETX	EQU	3		;ASCII ETX
 BREAK	EQU	3		;BREAK KEY
 DDCB?	EQU	1		;DISPLAY DEVICE#
 LD	EQU	01H		;LD BC,NNNN
 ;
 BUFER1	EQU	2700H		;ALT I/O BUFFER
 BUFER2	EQU	2800H		;ALT I/O BUFFER
 ;
 ;	EXTERNAL SVC EQUIVALENCES
 ;
 KBD	EQU	4		;KEYBOARD CHARACTER
 BINDEC	EQU	21		;BINARY/DECIMAL CONV
 HLDKEY	EQU	29		;PROCESS HOLD KEY
 ERROR	EQU	39		;POST ERROR MESSAGE
 OPEN	EQU	40		;OPEN DEVICE/FILE
 CLOSE	EQU	42		;CLOSE DEVICE/FILE
 KBDW	EQU	73		;KBD INPUT W/WAIT
 PUT	EQU	78		;OUTPUT BYTE DEVICE
 EVAL	EQU	83		;EVALUATE CMD LINE
 PARAM	EQU	87		;FETCH OPTIONAL PARAMS
 ;
 ;	EXTERNAL I/O EQUIVALENCES
 ;
 DCHECK	EQU	0		;CHECK DRIVE READY
 SREAD	EQU	6		;READ SYSTEM SECTOR
 ;
 ;	EXTERNAL SUPPORT EQUIVALENCES
 ;
 SVC	EQU	08H		;EXECUTE SVC
 DIO	EQU	10H		;DISK I/O MANAGER
 ;
 DEVTBL	EQU	0100H		;DEVICE TABLE
 SDIVD	EQU	030FH		;8 X 8 DIVIDE
 DMULT	EQU	0312H		;8 X 16 MULTIPLY
 TMULT	EQU	0318H		;8 X 24 MULTIPLY
 TDIVD	EQU	031BH		;8 X 24 DIVIDE
 DIRRED	EQU	0324H		;DIRECTORY READ
 LOCDCB	EQU	032AH		;LOCATE DCB
 LOCDCT	EQU	032DH		;LOCATE DCT
 ;
 BUFFER	EQU	1B00H		;MAIN I/O BUFFER
 ;
 	PAGE
 ;	TABLE OF VECTORED ROUTINES
 ;
 	ORG	2200H		;HIGH OVERLAY
 ;
 START	DEFW	RETURN		;0  - NOTHING DONE
 	DEFW	UNDEF		;1  - MISC PARAMS
 	DEFW	UNDEF		;2  - FILE ATTRIBUTES
 	DEFW	UNDEF		;3  - RENAME DEVICE/FILE
 	DEFW	FREE		;4  - FREE SPACE
 	DEFW	UNDEF		;5  - UNDEFINED
 	DEFW	UNDEF		;6  - UNDEFINED
 	DEFW	UNDEF		;7  - UNDEFINED
 	DEFW	UNDEF		;8  - UNDEFINED
 	DEFW	UNDEF		;9  - UNDEFINED
 	DEFW	UNDEF		;10 - UNDEFINED
 	DEFW	UNDEF		;11 - UNDEFINED
 	DEFW	UNDEF		;12 - UNDEFINED
 	DEFW	UNDEF		;13 - UNDEFINED
 	DEFW	UNDEF		;14 - UNDEFINED
 	DEFW	UNDEF		;15 - UNDEFINED
 ;
 ;	UNDEFINED ROUTINE HANDLER
 ;
 UNDEF	LD	A,BADF		;BAD FUNCTION CALL
 	OR	A		;NZ STATUS
 RETURN	RET			;EXIT
 ;
 	PAGE
 ;	FREE SPACE/MAP ROUTINES
 ;
 ;	ENT	HL =>	COMMAND LINE
 ;
 ;	EVALUATE OPTIONAL FIELDS
 ;
 FREE	LD	IX,PBLOCK	;IX => PARAM BLOCK
 	LD	A,EVAL		;EVAL CMD LINE
 	RST	SVC		;DO IT
 	JP	NZ,POSTE	;ERROR!
 ;
 ;	CHECK FOR PROPER FIELD TYPES
 ;
 	BIT	2,(IX+0)	;ANY SOURCE?
 	LD	C,0		;DEFAULT
 	JR	Z,FREE1 	;IF GLOBAL
 	LD	A,(DCB?1)	;GET DEVICE
 	BIT	6,A		;FILESPEC?
 	JP	NZ,ERR1 	;ERROR!
 	AND	0FH		;IGNORE FLAGS
 	SUB	8		;CORRECT
 	JP	C,ERR1		;IF ERROR
 	LD	C,A		;C = DRIVE
 ;
 FREE1	BIT	1,(IX+0)	;ANY DEST?
 	LD	A,DDCB? 	;DISPLAY DEVICE
 	JR	NZ,FREE2	;IF YES - SKIP
 	LD	(DCB?2),A	;DEFALULT DEVICE
 ;
 	PAGE
 ;	INITIALIZE FIELDS
 ;
 ;	ENT	DCB?1 => OPTIONAL SOURCE
 ;		DCB?2 => OPTIONAL DEST
 ;
 FREE2	CALL	DOPEN		;OPEN DEST DEVICE
 	JP	NZ,POSTE	;ERROR!
 ;
 ;	READ GAT FOR NAME/DATE
 ;
 FREE7	LD	A,DCHECK	;CHECK DRIVE
 	RST	DIO		;DO IT
 	JR	Z,FREE5 	;DRIVE READY
 	BIT	2,(IX+0)	;GLOBAL?
 	JP	Z,FREE6 	;IF YES
 	JP	ERR2		;ERROR!
 ;
 FREE5	LD	HL,BUFER1	;I/O BUFFER
 	LD	E,0		;GAT SECTOR
 	LD	A,SREAD 	;SYSTEM READ
 	RST	DIO		;DO IT
 	JP	NZ,POSTE	;I/O ERROR!
 	CALL	LOCDCT		;LOCATE DCT
 ;
 ;	FETCH DEVICE NAME/DATE
 ;
 	PUSH	BC		;SAVE REG
 	LD	H,DEVTBL<-8	;START DEVICE TABLE
 	LD	A,C		;DRIVE
 	ADD	A,8		;CORRECT
 	ADD	A,A		;* 2
 	ADD	A,A		;* 4
 	ADD	A,2		;TO NAME
 	LD	L,A		;HL => DEVICE NAME
 	LD	DE,FTITLE+7	;TITLE OFFSET
 	LD	BC,2		;COUNT
 	LDIR
 	LD	HL,BUFER1+0D0H	;HL => DISK TITLE
 	LD	DE,FTITLE+11	;TITLE OFFSET
 	LD	C,8		;BC = COUNT
 	LDIR			;TO TITLE
 	POP	BC		;RESTORE
 ;
 	PAGE
 ;	CALCULATE FREE SPACE
 ;
 	LD	HL,BUFER1	;HL => GAT
 	LD	DE,0		;INIT GRAN COUNT
 	LD	A,(BUFER1+0CCH) ;NUM TRKS OFFSET
 	ADD	A,35		;SMALLEST NUM TRK
 	LD	B,A		;B = COUNT
 ;
 CFREE1	LD	A,(HL)		;GET TRK DATA
 CFREE2	SCF			;TO PAD LEFT
 	RRA			;GRAN TO CRY
 	JR	C,CFREE3	;IF GRAN ALLOC
 	INC	DE		;UPD GRAN CNT
 CFREE3	CP	-1		;DONE WITH TRK?
 	JR	NZ,CFREE2	;IF NOT
 	INC	L		;POS TO NEXT TRK
 	DJNZ	CFREE1		;TIL DONE
 ;
 ;	GET FREE SPACE IN FILES
 ;
 	PUSH	DE		;SAVE GRANS
 	LD	HL,BUFFER	;I/O BUFFER
 	LD	E,2		;START FILES
 	LD	D,0		;FREE COUNT
 	LD	B,0		;FILE COUNT
 ;
 FREE3	LD	A,SREAD 	;SYSTEM READ
 	RST	DIO		;DO IT
 	JR	Z,FREE4 	;ALL OK
 	POP	DE		;GET GRANS
 	JP	POSTE		;I/O ERROR!
 ;
 FREE4	BIT	4,(HL)		;ACTIVE?
 	JR	NZ,$+3		;IF YES
 	INC	D		;INC FREE
 	INC	B		;INC COUNT
 	LD	A,20H		;FILE OFFSET
 	ADD	A,L
 	LD	L,A		;HL => NEXT
 	JR	NC,FREE4	;NO READ
 	LD	A,(IY+18)	;DIR LENGTH
 	DEC	A		;ILAST SECTOR
 	INC	E		;NEXT SEC
 	CP	E		;DONE?
 	JR	NC,FREE3	;TIL DONE
 ;
 ;	OUTPUT FREE SPACE INFO
 ;
 	LD	HL,FTITLE	;FREE TITLE
 	CALL	LINE$		;TO DEVICE
 	LD	E,D
 	LD	D,0		;DE = FREE FILES
 	CALL	DEC		;OUTPUT
 	LD	A,'/'           ;DELIM
 	CALL	PUT$		;TO DEVICE
 	LD	E,B		;DE = TOTAL FILES
 	CALL	DEC		;OUTPUT
 	LD	HL,SUBTL1	;'Size: '
 	CALL	LINE$		;OUTPUT
 	POP	HL		;GET GRANS
 	LD	A,(IY+20)	;SEC/GRAN
 	CALL	DMULT		;GET PAGES
 	LD	B,H
 	LD	H,L
 	LD	L,A		;BHL = PAGES
 	CALL	SIZEK		;OUTPUT SIZE
 	CALL	OUTCR		;OUTPUT C/R
 ;
 FREE6	BIT	2,(IX+0)	;GLOBAL?
 	JR	NZ,OMAP 	;IF NOT
 	LD	A,7		;LAST DRIVE
 	INC	C		;NEXT
 	CP	C		;DONE?
 	JP	NC,FREE7	;IF NOT
 	CALL	DCLOSE		;CLOSE DEVICE
 	RET
 ;
 	PAGE
 ;	OUTPUT FREE SPACE MAP
 ;
 OMAP	LD	HL,BUFER1	;HL => GAT
 	CALL	OUTCR		;OUTPUT C/R
 ;
 MAP1	LD	A,(BUFER1+0CCH) ;NUM TRKS OFFSET
 	ADD	A,35-1		;A = LAST TRK
 	CP	L		;DONE?
 	JR	NC,MAP2 	;IF NOT
 	CALL	DCLOSE		;CLOSE DEVICE
 	RET
 ;
 MAP2	LD	A,L		;GET TRK
 	CALL	BDEC		;DSP TRK
 	LD	A,'-'
 	CALL	PUT$		;OUTPUT
 	LD	A,L		;GET TRK
 	ADD	A,6
 	LD	B,A		;ASSUME 6 TRKS
 	LD	A,(BUFER1+0CCH) ;NUM TRKS OFFSET
 	ADD	A,35-1		;A = LAST TRK
 	CP	B		;7 VALID?
 	JR	C,$+3		;IF YES
 	LD	A,B		;GET LAST TRK
 	LD	(CNT),A 	;SAVE COUNT
 	CALL	BDEC		;DSP TRK
 	LD	A,':'           ;':'
 	CALL	PUT$		;OUTPUT
 	LD	A,' '           ;':  '
 	CALL	PUT$		;OUTPUT
 	CALL	PUT$
 ;
 MAP3	LD	B,0		;INIT GRAN/TRK
 MAP4	BIT	5,(IY+5)	;HARD?
 	JR	NZ,MAP7 	;SKIP LOCKED
 	PUSH	HL
 	LD	A,60H		;GET CYL
 	ADD	A,L
 	LD	L,A		;HL => LOCKED STAT
 	CALL	CHKBIT		;GRAN LOCKED?
 	POP	HL
 	LD	A,'L'           ;ASSUME LOCKED
 	JR	NZ,MAP5 	;IF LOCKED
 ;
 MAP7	LD	A,(IY+19)	;DIR LOC
 	CP	L		;GET CYL
 	JR	NZ,MAP6 	;NOT AT DIR
 	LD	E,(IY+18)	;SEC/DIR
 	LD	A,(IY+20)	;SEC/GRN
 	CALL	SDIVD		;A = DIR GRNS
 	INC	E
 	DEC	E		;PARTIAL?
 	JR	NZ,$+3		;IF YES
 	DEC	A		;LAST GRAN
 	CP	B		;DIR GRAN?
 	LD	A,'D'           ;ASSUME YES
 	JR	NC,MAP5 	;IF YES
 ;
 MAP6	CALL	CHKBIT		;GRAN ALLOC?
 	LD	A,'X'           ;ASSUME YES
 	JR	NZ,MAP5 	;IF YES
 	LD	A,'.'           ;NOT ALLOC
 ;
 MAP5	CALL	PUT$		;OUTPUT
 	LD	A,(IY+21)	;GRAN/CYL
 	INC	B		;NEXT GRAN
 	CP	B		;END OF TRK?
 	JR	NZ,MAP4 	;IF NOT
 ;
 CNT	EQU	$+1
 	LD	A,0
 	INC	L		;NEXT TRACK
 	CP	L		;DONE?
 	JR	C,MAP8		;IF YES
 	LD	A,' '           ;BLANK
 	CALL	PUT$		;OUTPUT
 	JR	MAP3		;TIL EOL
 ;
 MAP8	CALL	OUTCR		;C/R
 	JP	MAP1		;TIL DONE
 ;
 	PAGE
 ;	MESSAGES AND TEXT STRINGS
 ;
 FTITLE	DEFM	'Drive: NS  diskname - Files: '
 	DEFB	3
 SUBTL1	DEFM	'  Space: '
 	DEFB	3
 ;
 ;	$EVAL PARAMETER BLOCK
 ;
 PBLOCK	DEFB	0		;MISC FLAGS
 	DEFW	DCB?1		;SOURCE DCB
 	DEFW	DCB?2		;DEST DCB
 	DEFW	BUFER1		;DUMMY WILD
 	DEFW	PLIST		;PARAMETER LIST
 ;
 ;	$PARAM PARAMETER BLOCK
 ;
 PLIST	DEFB	0		;NO PARAMETERS
 ;
 ;	RESERVED DATA AREAS
 ;
 DCB?1	DEFS	41		;SOURCE DEVICE DCB
 DCB?2	DEFS	41		;DEST DEVICE DCB
 ;
 	PAGE
 ;	CHECK BIT ROUTINE
 ;
 CHKBIT	LD	A,B		;A = BIT
 	AND	7		;MASK
 	RLCA
 	RLCA
 	RLCA
 	OR	46H		;A = INST
 	LD	(BIT),A 	;SET INST
 BIT	EQU	$+1
 	BIT	0,(HL)		;CHECK IT
 	RET
 ;
 ;	OUTPUT BYTE TO DEST DEVICE
 ;
 OUTCR	LD	A,CR		;C/R
 ;
 PUT$	PUSH	DE		;SAVE
 	PUSH	BC
 	LD	DE,DCB?2+1	;DEVICE DCB
 	LD	B,A		;B = CHAR
 	LD	A,PUT		;PUT CHAR
 	RST	SVC		;DO IT
 	POP	BC
 	POP	DE		;RESTORE
 	RET
 ;
 ;	OUTPUT LINE TO DEST DEVICE
 ;
 ;	ENT	HL =>	ASCII TEXT
 ;
 LINE$	LD	A,(HL)		;GET CHAR
 	INC	HL
 	CP	ETX		;END TEXT?
 	RET	Z		;IF YES
 	CALL	PUT$		;OUTPUT
 	JR	LINE$		;IF NOT
 ;
 ;	OPEN DEST DEVICE FOR I/O
 ;
 ;	EXT	AF =	ERROR STATUS
 ;
 DOPEN	LD	A,(DCB?2)	;GET DEVICE
 	AND	0FH		;IGNORE FLAGS
 	CP	8		;FILESPEC?
 	JR	C,OPEN1 	;IF NOT
 	LD	DE,DCB?2+1	;DEST DCB
 	LD	HL,OLIST	;OPEN PARAM LIST
 	LD	A,OPEN		;OPEN
 	RST	SVC		;DO IT!
 	RET
 ;
 OPEN1	PUSH	IX		;SAVE
 	CALL	LOCDCB		;LOCATE DCB
 	LD	(DCB?2+4),IX	;SET ROUTE
 	LD	A,10H		;ROUTED DEVICE
 	LD	(DCB?2+1),A	;SET ROUTE
 	POP	IX		;RESTORE
 	XOR	A
 	RET
 ;
 ;	CLOSE DEST DEVICE I/O
 ;
 DCLOSE	LD	DE,DCB?2+1	;DEST DCB
 	LD	A,(DE)		;GET TYPE
 	AND	80H		;DEVICE?
 	RET	Z		;ALL OK
 	LD	A,CLOSE 	;CLOSE DEVICE
 	RST	SVC		;DO IT
 	RET
 ;
 ;	OPEN PARAMETER LIST
 ;
 OLIST	DEFW	BUFER2		;BUFADR
 	DEFW	0		;RECADR
 	DEFW	0		;EOFADR
 	DEFM	'W'             ;READ/WRITE
 	DEFB	0		;LRL = 256
 	DEFB	'F'             ;FIXED LENGTH
 	DEFB	2		;CREATE
 ;
 ;	S/R TO OUTPUT SIZE IN K
 ;
 ;	ENT	BHL =	PAGE COUNT
 ;
 SIZEK	LD	A,4		;PAGES/K
 	CALL	TDIVD		;GET K
 	PUSH	AF		;SAVE REMAINDER
 	EX	DE,HL		;DE = RESULT
 	CALL	DEC		;OUTPUT
 	LD	A,'.'           ;DECIMAL
 	CALL	PUT$		;TO DEVICE
 	POP	AF		;GET REMAIN
 	LD	E,A		;E = REMAIN
 	ADD	A,A		;* 2
 	ADD	A,A		;* 4
 	ADD	A,E		;* 5
 	SRL	A		;/ 2
 	ADD	A,'0'           ;ASCII
 	CALL	PUT$		;TO DEVICE
 	LD	A,'k'           ;IN K
 	JP	PUT$		;TO DEVICE
 ;
 ;	CONVERT BINARY TO ASCII & OUTPUT
 ;
 ;	ENT	DE =	BINARY
 ;
 BDEC	PUSH	BC		;SAVE
 	LD	E,A
 	LD	D,0		;DE = BINARY
 	LD	B,2		;DIGIT REQUEST
 	JR	DEC0		;SKIP
 ;
 DEC	PUSH	BC		;SAVE
 	LD	B,4		;DIGIT REQUEST
 ;
 DEC0	PUSH	HL		;SAVE REG
 	PUSH	BC
 	LD	HL,TBUFF	;TEMP DATA
 	LD	B,0		;BIN TO ASCII
 	LD	A,BINDEC	;BINARY/DEC
 	RST	SVC		;DO IT!
 	POP	BC		;GET DIGITS
 	LD	C,5		;PRECISION
 ;
 DEC1	LD	A,(HL)		;GET DIGIT
 	CP	'0'             ;LEAD 0?
 	JR	NZ,DEC2 	;IF NOT
 	INC	HL		;NEXT
 	DEC	C		;NEXT
 	DJNZ	DEC1		;TIL DONE
 ;
 DEC2	LD	A,(HL)		;GET DIGIT
 	CALL	PUT$		;OUTPUT
 	INC	HL		;NEXT
 	DEC	C		;NEXT
 	JR	NZ,DEC2 	;TIL DONE
 	POP	HL		;RESTORE REG
 	POP	BC
 	RET
 ;
 TBUFF	DEFS	5		;TEMP BUFFER
 ;
 ;	POST ERROR MESSAGE
 ;
 ERR1	LD	A,ERR3		;'ILLEGAL DRIVEPSEC'
 	DEFB	LD
 ERR2	LD	A,ERR8		;'DRIVE NOT READY'
 ;
 POSTE	AND	7FH		;NO ABORT
 	LD	B,A		;B = CODE
 	LD	A,ERROR 	;POST ERROR
 	RST	SVC		;DO IT!
 	XOR	A		;NO ERROR
 	RET
 ;
 	END	START
