; dossys7/asm - kjw/bqsd - sjp/mss - 08/82
;
	TITLE	'<Dos+ II.A - SYS07/SYS>'
;
;
;	system is low residing (1C00-21FF)
;	and contains the following:
;
;	$INITIO - SVC 0 - initialize I/O
;	$KBINIT - SVC 1 - initialize keyboard
;	$VDINIT - SVC 7 - initialize video
;	$PRINIT - SVC 17 - initialize printer
;	$RS232C - SVC 55 - initialize RS232
;	$DATE	- SVC 45 - fetch/load DATE$
;	$PRCTL	- SVC 95 - printer param control
;	$ACTL	- SVC 100 - SIO ch A control
;	$BCTL	- SVC 101 - SIO ch B control
;
	PAGE
;
;	Outside Support Equates
;
PAGE0	EQU	0000H		;PAGE 0 ADDRESS
OFFINT	EQU	70H		;$SIO INT vector (page 0)
;
TIME$	EQU	0150H		;time
DATE$	EQU	0154H		;date
MONTHS	EQU	015BH		;month table
FLAGK1	EQU	0167H		;system flag #1
VMASK	EQU	0199H		;video mask
;
REGSAV	EQU	0303H		;save registers
LOCDCB	EQU	032AH		;locate DCB from A
UCASE	EQU	031EH		;convert A to upper case
FETMEM	EQU	0330H		;fetch top memory
;
DELAY	EQU	6		;decrement BC till 0
BADF	EQU	1		;bad function code
PERR	EQU	3		;parameter error
BDAT	EQU	9		;invalid data provided
LOOKUP	EQU	28		;lookup entry in table
VDCHAR	EQU	8		;char to video
;
SADCB	EQU	3		;SIOA dcb number
SBDCB	EQU	4		;SIOB dcb number
KIDCB	EQU	0		;KI dcb number
PRDCB	EQU	2		;PR dcb number
;
SVC	EQU	08H		;execute SVC
;
ERR01	EQU	01		;bad function code
ERR89	EQU	89		;illegal I/O channel
ERR98	EQU	98		;data out of range
;
	PAGE
;
	ORG	1C00H		;low overlay
;
;	Table of vectored routines
;
VECTORS DW	RETURN		;0 - return
	DW	KBINIT		;1
	DW	VDINIT		;2
	DW	PRINIT		;3
	DW	RS232C		;4
	DW	DATE		;5
	DW	PRCTL		;6
	DW	ACTL		;7
	DW	BCTL		;8
	DW	INITIO		;9
	DW	JPINIT		;10
	DW	UNDEF		;11 - undefined
	DW	UNDEF		;12
	DW	UNDEF		;13
	DW	UNDEF		;14
	DW	UNDEF		;15
;
;	undefined system call vector
;
UNDEF	LD	A,BADF		;'bad function call'
	OR	A		;set NZ
	RET			;back to caller
;
	PAGE
;
;
;	definition of SIO initialization table
;
;	Channel A data
;
SIOA	DEFB	3		;# of tables
;
;	setup for SIO ports
;
	DEFB	11		;# entries this table
	DEFB	0F6H		;SIO base port address
	DEFB	18H		;channel reset
	DEFB	4		;R4
	DEFB	44H		;x16, stop=1
	DEFB	3		;R3
	DEFB	0E1H		;word=8, RX
	DEFB	5		;R5
	DEFB	0EAH		;DTR,W=8,TX,RTS
	DEFB	2		;R2
	DEFB	OFFINT		;int vector
	DEFB	1+10H		;reset int, select R1
	DEFB	1CH		;int all RX chars
;
;	CTC #1 setup
;
	DEFB	3		;# entries this table
	DEFB	0F0H		;CTC-0 base port address
	DEFB	07H		;count, reset
	DEFB	34H		;default 300 baud
	DEFB	OFFINT+16	;int vector
;
;	CTC #2 setup
;
	DEFB	2		;# entries
	DEFB	0F1H		;CTC-1 base port
	DEFB	07H		;count, reset
	DEFB	34H		;default, 300 baud
;
;	baud table
;
SIOBAUD DEFW	8E07H		;110
	DEFW	6807H		;150
	DEFW	3407H		;300
	DEFW	1A07H		;600
	DEFW	0D07H		;1200
	DEFW	3447H		;2400
	DEFW	1A47H		;4800
	DEFW	0D47H		;9600
;
	PAGE
;
;	$RS232C - SVC 55
;	initialize RS232 channels
;
;	ENT	HL => parameter list (6 bytes)
;		B = 0 = turn off channel
;		B >< 0 = turn on channel and define
;
;	EXT	Z = OK
;		NZ = A = error code
;
;	PARAMETER LIST
;
;	+0	- channel ("A", "a", "B", "b")
;	+1	- baud rate (1-8)
;		1 = 110 baud
;		2 = 150 baud
;		3 = 300 baud
;		4 = 600 baud
;		5 = 1200 baud
;		6 = 2400 baud
;		7 = 4800 baud
;		8 = 9600 baud
;	+2	- word length (5-8)
;	+3	- parity ("O","o","E","e","N","n")
;	+4	- stop bits (1,2)
;	+5	- end list marker (00H)
;
; extended buffer size field
;
;	+5	- 1 (indicates extended field)
;	+6,+7	- start of buffer
;	+8,+9	- end of buffer
;	+10	- end list marker (00H)
;
RS232C	CALL	REGSAV		;save registers
	PUSH	HL		;HL => user parameters
	POP	IY		;load into IY
	INC	B		;see if add/delete
	DEC	B		;B = 0 = delete
	EX	AF,AF'          ;AF' = Z = REMOVE
;
;	check channel number
;
	LD	A,(IY)		;get channel #
	CALL	UCASE		;convert to upper case
;
;	channel A?
;
	LD	DE,0F0F1H	;CTC port address
	LD	BC,SADCB<8+0F6H ;B=DCB#, C=port
	SUB	'A'             ;remove ascii
	JR	Z,RSCGO 	;have the channel
;
;	channel B?
;
	LD	DE,0F200H	;CTC ports (00=dummy)
	INC	B		;bump DCB #
	INC	C		;bump base port
	DEC	A		;channel B?
	LD	A,ERR89 	;invalid channel
	RET	NZ		;illegal channel, return
;
;	channel specifics loaded, find DCB
;
RSCGO	LD	IX,SIOA 	;parameter list CH-A/B
	LD	(IX+02),C	;SIO base port
	LD	(IX+15),D	;CTC #1 port
	LD	(IX+20),E	;CTC #2 port
	LD	A,B		;get DCB #
	CALL	LOCDCB		;locate the DCB
	LD	(IX+18),D	;load CTC ports
	LD	(IX+19),E
	EX	AF,AF'          ;Z = remove channel
	JR	Z,RSCREM	;remove it
;
;	activate channel
;
	CALL	NILBUFF 	;set buffer @ HL, clear
	LD	(IX+17),C	;SIO base port (status)
	CALL	SIOPARM 	;get user parameters
	RET	NZ		;return in error
	LD	HL,SIOA 	;get parameter list
	CALL	SNDDAT		;setup SIO & CTC ports
	RES	1,(IX+5)	;set channel as init'ed
	RES	3,(IX)		;turn off nil
	XOR	A		;set Z flag
	RET
;
;	remove channel
;
RSCREM	SET	1,(IX+5)	;turn off channel
	SET	3,(IX)		;set as nil device
	LD	C,(IX+17)	;get base port
	LD	A,18H		;channel reset
	OUT	(C),A		;send SIO
	LD	C,(IX+18)	;CTC #1
	LD	A,3		;disable int's, reset
	OUT	(C),A		;send CTC
	LD	C,(IX+19)	;CTC #2
	INC	C		;check if dummy (00)
	DEC	C		;see if nil
	JR	Z,SIO6		;go if nil
	OUT	(C),A		;else reset CTC #2
SIO6	XOR	A		;Z = OK
	RET
;
	PAGE
;	following sends data to I/O ports
;
SNDDAT	LD	E,(HL)		;get table count
	INC	HL		;point to data
;
SETSLP	LD	B,(HL)		;get data count
	INC	HL		;point to data
	LD	C,(HL)		;get port #
	INC	C
	DEC	C
	RET	Z		;if dummy port
	INC	HL		;bump pointer
	OTIR			;send block to port
	DEC	E		;more?
	JR	NZ,SETSLP	;go if yes
	RET			;else done
;
	PAGE
;	interpret data from user parameter list
;
;	ENT	IY =>	user list
;
SIOPARM PUSH	IY		;user list
	POP	HL		;HL => user list
	LD	IY,SIOA 	;IY => config table
;
;	interpret baud rate
;
	INC	HL		;point to baud rate
	LD	A,(HL)		;get baud
	DEC	A		;must be 1-8
	JP	M,RSCERR	;error if 0
	CP	8		;test for max
	JP	NC,RSCERR	;error if >8
;
;	save into DCB mask
;
	PUSH	AF		;save it
	RLCA			;move it over
	RLCA			;3 places
	RLCA
	RLCA			;4 places
	AND	70H		;3 bits only
	LD	(IX+23),A	;save into mask
	POP	AF		;fetch it
;
;	lookup baud rate in table
;
	PUSH	HL		;save param list
	LD	HL,SIOBAUD	;baud rate table
	ADD	A,A		;double for table
	ADD	A,L		;add to LSB
	LD	L,A		;update it
	JR	NC,SIO7 	;page boundary?
	INC	H		;bump page
SIO7	LD	A,(HL)		;get counter select
	INC	HL		;bump pointer
	LD	H,(HL)		;get count
	LD	(IY+16),A	;RX baud
	LD	(IY+21),A	;TX baud
	LD	(IY+17),H	;RX count
	LD	(IY+22),H	;TX count
	POP	HL		;restore pointer
;
;	interpret word length
;
	INC	HL		;point to word length
	LD	A,(HL)		;word length
	CP	5		;must be 5-8
	JP	C,RSCERR	;go if <5
	CP	9		;test for max
	JP	NC,RSCERR	;go if >8
;
;	compute word length mask
;
	LD	B,A		;for loop
	XOR	A		;zero
SIO8	SCF			;set a bit
	RLA			;move into mask
	DJNZ	SIO8		;for word length
	LD	(IX+20),A	;save word mask
;
;	save into DCB mask
;
	LD	A,(HL)		;get again
	SUB	5		;adjust for strange bits
	PUSH	AF		;save status
	RLCA
	RLCA
	OR	(IX+23) 	;with the rest
	LD	(IX+23),A	;put it in
	POP	AF		;get status
;
;	set-up word length
;
	JR	Z,SIO9		;no correct
	CP	3		;ok?
	JR	Z,SIO9		;no correct
	XOR	3		;reverse bits
SIO9	RRCA			;align bits
	RRCA
	RRCA
	OR	8AH		;DTR, TX enable, RTS, len
	LD	(IY+09),A	;save into TX table
	LD	(IX+22),A	;save for CTL calls (R5)
	RLCA			;align
	OR	21H		;auto trigger, RX enable
	LD	(IY+07),A	;put into RX table
;
;	interpret parity
;
	INC	HL		;point to it
	LD	A,(HL)		;get parity
	CALL	UCASE		;make it upper case
	LD	B,40H		;parity mask
	CP	'N'             ;none?
	JR	Z,RSPAR 	;have it
	SET	0,B		;set odd
	CP	'O'             ;odd?
	JR	Z,RSPAR 	;go if yes
	SET	1,B		;set even
	CP	'E'             ;even?
	JR	NZ,RSCERR	;parameter error
RSPAR	LD	A,B		;get result
	AND	3		;low 2 bits only
	OR	(IX+23) 	;set with mask
	LD	(IX+23),A	;put it in
;
;	interpret stop bits
;
	INC	HL		;point to stop bits
	LD	A,(HL)		;stop bits
	CP	4		;must be 0-3
	JR	NC,RSCERR	;error if not
	LD	(IX+21),A	;update it
	RLCA
	RLCA
	OR	B		;merge with mask
	LD	(IY+05),A	;put into param table
;
;	interpret extended field
;
	INC	HL		;bump pointer
	LD	A,(HL)		;fetch param byte
	DEC	A		;01 extended field
	JP	NZ,RETURN	;nope, done!
;
	INC	HL		;bump pointer
	LD	A,(HL)		;get LSB start
	LD	E,A		;save it
	INC	HL		;bump pointer
	LD	D,(HL)		;MSB start
	INC	HL		;bump pointer
	LD	A,(HL)		;LSB end
	INC	HL		;bump pointer
	LD	H,(HL)		;MSB end
	LD	L,A		;HL=end, DE=start
	SBC	HL,DE		;HL = buff length
	LD	B,H		;pass to BC
	LD	C,L		;BC = buff length
	CALL	FETMEM		;enough memory?
	RET	NZ		;nope, return
;
;	insert new buffer into DCB
;
	LD	(IX+9),L	;buffer start
	LD	(IX+10),H
	LD	(IX+11),C	;buffer size
	LD	(IX+12),B
	CALL	NILBUFF 	;clear it out
;
RETURN	XOR	A		;set Z for OK
	RET			;parameters OK
;
;	parameter error
;
RSCERR	LD	A,ERR98 	;illegal data range
	OR	A		;set NZ
	RET			;back to caller
;
	PAGE
;
;	init data for PIO ports
;
PIO	DEFB	2		;2 tables
;
	DEFB	5		;5 entries
	DEFB	0E2H		;port
	DEFB	0		;int vector
	DEFB	0CFH		;bit mode
	DEFB	0F7H		;I/O mask
	DEFB	37H		;int off
	DEFB	0FEH		;int mask
;
	DEFB	3		;data #
	DEFB	0E3H		;port
	DEFB	0		;int vector
	DEFB	0FH		;output mode
	DEFB	7		;no int
;
	PAGE
;
;	$PRINIT - SVC 17
;	initialize printer
;
;	ENT	B = page length (66 standard)
;		C = printed lines/page (60 standard)
;			if C = 0, no automatic form
;			feed is done.
;		D = max chars/line (132 standard)
;
;	EXT	Z = OK
;		NZ = A = error code
;
PRINIT	CALL	REGSAV		;save registers
;
;	locate the DCB
;
	LD	A,PRDCB 	;DCB number for printer
	CALL	LOCDCB		;locate the DCB
;
	XOR	A		;test for 0 page length
	CP	B		;page length
	JR	Z,PRIN1 	;check both B&C = 0
	CP	C		;printed lines
	JR	NZ,PRIN1	;OK, go
;
	LD	A,B		;length
	CP	C		;printed
	JR	C,PRERR 	;invalid
	LD	E,0		;mask
	JR	PRIN2		;go!
;
PRIN1	OR	B		;both must be 0
	OR	C
	JR	NZ,PRERR	;parameter error
	LD	E,4		;real FF
;
;	setup control byte
;
PRIN2	LD	A,(IX+5)	;get control byte
	AND	0F1H		;bits 1,2,3 off
	OR	E		;set form feed control
	INC	D		;see if 0 max chars
	DEC	D
	JR	NZ,PRIN3	;go if <> 0
	OR	2		;set real tabs
PRIN3	LD	(IX+5),A	;update control byte
	RES	3,(IX)		;turn off nil device
;
;	setup and nil printer buffer
;
	CALL	NILBUFF 	;setup and clear it
	LD	HL,PIO		;PIO data table
	CALL	SNDDAT		;send to the ports
;
;	load parameters into DCB
;
	XOR	A		;set Z flag for OK
	LD	(IX+17),C	;printed lines/page
	LD	(IX+18),B	;page length
	LD	(IX+19),A	;line counter
	LD	(IX+20),D	;max chars/line
	LD	(IX+21),A	;char counter
	RET			;done
;
;	parameter error
;
PRERR	LD	A,PERR		;set parameter error
	OR	A		;set NZ
	RET
;
	PAGE
;
;	init data for keyboard CTC ports
;
KBIO	DEFB	2		;2 tables
;
	DEFB	1		;data #
	DEFB	0F0H		;port
	DEFB	OFFINT+16	;vector
;
	DEFB	2		;data #
	DEFB	0F3H		;port
	DEFB	0C7H		;reset, TC follows
	DEFB	1		;time constant
;
	PAGE
;
;	$KBINIT - SVC 1
;	initialize keyboard
;
;	ENT	NONE
;
;	EXT	Z = OK
;		NZ = A = error code
;
KBINIT	CALL	REGSAV		;save registers
	LD	A,KIDCB 	;DCB for keyboard
	CALL	LOCDCB		;load IX with DCB
;
;	setup and nill keyboard buffer
;
	LD	HL,KBIO 	;I/O table
	CALL	SNDDAT		;send to ports
	CALL	NILBUFF		;setup and clear
;
;	make sure keyboard hardware is clear
;
	IN	A,(0FCH)	;clear keyboard port
	LD	BC,80H		;make sure its not hard
	LD	A,DELAY 	;SVC #
	RST	SVC		;call delay
	IN	A,(0FCH)	;make sure it's clear
;
;	reset flags
;
	LD	A,(IX+5)	;get flags
	AND	1		;save XLATE only
	LD	(IX+5),A	;put control byte back
	XOR	A		;set Z flag
	RET
;
	PAGE
;
;	$NILBUFF
;	clear 'spooled' buffer
;
;	ENT	IX => DCB
;
;	EXT	buffer set and cleared to nil
;		A = 0
;
NILBUFF XOR	A		;load with 0
	DI
	LD	(IX+13),A	;clear adder offset
	LD	(IX+14),A
	LD	(IX+15),A	;clear taker offset
	LD	(IX+16),A
	EI
	RET			;done
;
	PAGE
;
;	$VDINIT - VIDEO INITIALIZATION
;
;	ENT	 B =	CHAR/LINE SWITCH
;		 C =	NORMAL/REVERSE SWITCH
;
;	EXT	AF =	ERROR CODE & STATUS
;
DDCB@	EQU	1		;DISPLAY DCB NUMBER
ESC	EQU	27H		;CLEAR SCREEN CODE
CRTA	EQU	0FCH		;CRTC ADD REGISTER
CRTD	EQU	0FDH		;CRTC DATA REGISTER
MDSL	EQU	0FFH		;MODE SEL PORT
VIDEO	EQU	0F800H		;START VIDEO MEMORY
;
;	SAVE REGISTERS & LOCATE DCB
;
VDINIT	CALL	REGSAV		;SAVE REG
	PUSH	BC		;save commands
	LD	B,1BH		;clear screen code
	LD	A,VDCHAR	;vid char
	RST	SVC		;clear screen
	POP	BC		;restore commands
;
	LD	A,DDCB@ 	;DISPLAY DCB
	CALL	LOCDCB		;IX => DCB ADDRESS
;
;	SET CHAR MODE (REVERSE/NORMAL)
;
VINIT0	RES	7,(IX+9)	;ASSUME NORMAL
	INC	C		;ZERO
	DEC	C		;NORMAL?
	JR	NZ,VINIT1	;IF YES
	SET	7,(IX+9)	;REVERSE MODE
;
;	SET LINE MODE (40/80 CHAR WIDE)
;
VINIT1	LD	A,(VMASK)	;get video mask
	RES	4,A		;mod sel 80
	LD	HL,DVRD80	;TOTAL COLS
	LD	DE,CRTD80	;CRTC 40 CHAR DATA
	INC	B		;MODE SEL 80
	DEC	B		;40 CHAR MODE?
	JR	NZ,VINIT2	;IF YES
	SET	4,A		;MODE SEL 40
	LD	HL,DVRD40	;TOTAL COLS
	LD	DE,CRTD40	;CRTC 40 CHAR DATA
;
;	INIT DRIVER VARIABLES
;
VINIT2	LD	(VMASK),A	;save mask
	PUSH	DE		;SAVE CRTC DATA
	PUSH	HL		;SAVE DRIVER DATA
	PUSH	IX
	POP	HL		;HL => DCB
	LD	BC,10		;OFFSET
	ADD	HL,BC		;HL => INFO
	EX	DE,HL		;DE => INFO
	POP	HL		;HL => NEW INFO
	LD	BC,9		;COUNT
	LDIR			;MOVE!
;
;	INITIALIZE CRTC HERE
;
	POP	HL		;HL => INIT DATA
	LD	DE,CRTDI	;DE => CRTC DATA
	LD	BC,4		;COUNT
	LDIR			;MOVE
;
	LD	BC,16<8+CRTD	;COUNT+REG
	LD	HL,CRTDI	;DE => CRTC DATA
	XOR	A		;START
VINIT3	OUT	(CRTA),A	;SET REG
	INC	A		;NEXT REG
	OUTI			;MOVE DATA
	JR	NZ,VINIT3	;TIL DONE
	LD	A,(VMASK)	;GET MODE
	OUT	(MDSL),A	;SET MODE
	XOR	A		;NO ERRORS
	RET
;
;	DRIVER INITIALIZATION
;
DVRD80	DEFB	80		;CHARS/LINE
	DEFW	VIDEO		;CURSOR POSN
	DEFW	VIDEO		;START SCROLL
	DEFW	VIDEO+1920	;END SCROLL
	DEFW	1840		;SCROLL COUNT
;
DVRD40	DEFB	40		;CHARS/LINE
	DEFW	VIDEO		;CURSOR POSN
	DEFW	VIDEO		;START SCROLL
	DEFW	VIDEO+960	;END SCROLL
	DEFW	920		;SCROLL COUNT
;
;	CRTC INITIALIZATION
;
CRTD80	DEFB	80+19		;H. TOTAL
	DEFB	80		;H. DISPLAYED
	DEFB	80+4		;H. SYNC POSITION
	DEFB	8		;H. SYNC WIDTH
;
CRTD40	DEFB	40+9		;H. TOTAL
	DEFB	40		;H. DISPLAYED
	DEFB	40+3		;H. SYNC POSITION
	DEFB	4		;H. SYNC WIDTH
;
CRTDI	DEFB	80+19		;H. TOTAL
	DEFB	80		;H. DISPLAYED
	DEFB	80+4		;H. SYNC POSITION
	DEFB	8		;H. SYNC WIDTH
;
	DEFB	24+1		;V. TOTAL
	DEFB	0		;V. TOTAL ADJUST
	DEFB	24		;V. DISPLAYED
	DEFB	24		;V. SYNC POSITION
	DEFB	0		;INTERLACE MODE
	DEFB	9		;MAX SCAN LINE ADD
	DEFB	5+60H		;CURSOR START & CTL
	DEFB	9		;CURSOR END
	DEFW	0		;START ADDRESS
	DEFW	0		;CURSOR POSITION
;
	PAGE
;
;	$PRCTL	- SVC 95 - setup printer data
;
;	ENT	B = command # 0-10
;
;	EXT	Z = OK
;		NZ = A = error code
;
;	commands:
;	0	- get status
;	1	- select serial
;	2	- select parallel
;	3	- reset line counter to C
;	4	- reset char counter to C
;	5	- begin transparent
;	6	- end transparent
;	7	- begin dummy
;	8	- end dummy
;	9	- begin auto LF
;	10	- end auto LF
;
PRCTL	PUSH	IX		;save it
	LD	A,PRDCB 	;dcb #
	CALL	LOCDCB		;find it
;
;	check for valid command
;
	PUSH	HL		;save
	LD	HL,PRCTBL	;command table
	LD	A,LOOKUP	;SVC #
	RST	SVC		;see if here
	JR	NZ,PRCERR	;invalid command
;
;	command found, leave return vector
;
	PUSH	HL		;save command address
	LD	HL,PRCRET	;return vector
	EX	(SP),HL 	;leave on stack
	JP	(HL)		;go vector!
;
PRCERR	POP	HL		;restore and return
	POP	IX
	LD	A,ERR01 	;bad function call
	RET			;NZ already set
;
PRCRET	POP	HL		;restore stack
	LD	B,(IX+18)	;page length
	LD	C,(IX+17)	;lines/page
	LD	D,(IX+20)	;max chars
	LD	H,(IX+21)	;char count
	LD	L,(IX+19)	;line count
	LD	E,'P'           ;parallel?
	BIT	7,(IX+5)	;yes?
	POP	IX		;restore it
	LD	A,0		;return with a 0
	RET	Z		;done, go back
	LD	E,'S'           ;serial
	OR	A		;set Z
	RET			;done, return
;
;	command vectors
;
PRC1	SET	7,(IX+5)	;set serial
PRC0	RET
PRC2	RES	7,(IX+5)	;set parallel
	RET
;
PRC3	LD	(IX+19),C	;reset line counter
	RET
PRC4	LD	(IX+21),C	;reset char counter
	RET
;
PRC5	SET	3,(IX+5)	;begin transparent
	RET
PRC6	RES	3,(IX+5)	;end transparent
	RET
;
PRC7	SET	3,(IX)		;begin dummy
	RET
PRC8	RES	3,(IX)		;end dummy
	RET
;
PRC9	SET	6,(IX+5)	;set LF
	RET
PRC10	RES	6,(IX+5)	;end LF
	RET
;
;	lookup table for printer commands
;
PRCTBL	DB	0		;get status
	DW	PRC0
	DB	1		;select serial
	DW	PRC1
	DB	2		;select parallel
	DW	PRC2
	DB	3		;reset line counter to C
	DW	PRC3
	DB	4		;reset char counter to C
	DW	PRC4
	DB	5		;begin transparent
	DW	PRC5
	DB	6		;end transparent
	DW	PRC6
	DB	7		;begin dummy
	DW	PRC7
	DB	8		;end dummy
	DW	PRC8
	DB	9		;begin auto LF
	DW	PRC9
	DB	10		;end auto LF
	DW	PRC10
;
	DB	-1		;terminator
;
;	table of starting days for years 1980-1987
;
STDAY	DB	0
	DB	2
	DB	3
	DB	4
	DB	5
	DB	0
	DB	1
	DB	2
;
	PAGE
;
;	$DATE - SVC 45
;	fetch/load data from TIME$ and DATE$
;
;	ENT	B = 0 = fetch time/date
;		HL => 26 byte buffer to be loaded:
;	+0-+2	- name of day
;	+3-+5	- month
;	+6,+7	- day of month
;	+8-+11	- year
;	+12-+14 - day of year
;	+15-+22 - time (HH:MM:SS)
;	+23,+24 - month number
;	+25	- day of week (monday = day 0)
;
;	ENT	B = 1 = set date
;		HL => 10 byte buffer (MM/DD/YYYY)
;
;	ENT	B = 2 = set time
;		HL => 8 byte buffer (HH:MM:SS)
;
;	EXT	Z = OK
;		NZ = A = error code
;
DATE	CALL	REGSAV		;save registers
;
	LD	A,B		;get function code
	OR	A		;B = 0?
	JR	Z,GETTD 	;get time/date
	DEC	A		;B = 1?
	JR	Z,SETDATE	;set date
	DEC	A		;B = 2?
	JR	Z,SETTIME	;set time
;
;	parameter error exit
;
DATERR	LD	A,ERR01 	;bad function call
	JR	DATERET 	;unstack and return
;
;	invalid data provided
;
DATEBAD LD	A,BDAT		;'bad data provided'
;
;	restore stack, set flags, and return
;
DATERET OR	A		;set flags
	RET			;done
;
;	load DATE$
;
SETDATE LD	DE,DATE$+1	;point to month
;
;	fetch month
;
	CALL	VALUE		;get the value
	JR	C,DATEBAD	;invalid characters
	OR	A		;0?
	JR	NZ,CNTD1	;nope, continue
	INC	A		;else make it JAN
CNTD1	CP	13		;1-12?
	JR	NC,DATEBAD	;invalid
	LD	(DE),A		;set the month
	DEC	DE		;point to day of week
;
;	fetch day of month
;
	CALL	VALUE		;get the value
	JR	C,DATEBAD	;invalid chars
	OR	A		;zero?
	JR	NZ,CNTD2	;nope, continue
	INC	A		;else make it first day
CNTD2	CP	32		;1-31?
	JR	NC,DATEBAD	;invalid
	LD	(DE),A		;set the day
	INC	DE		;point to year
	INC	DE
;
;	fetch year
;
	CALL	VALUE		;get the value
	JR	C,DATERET	;invalid chars
	JR	Z,DATEZ 	;<256, go!
	LD	H,B		;get the year
	LD	L,C
	LD	BC,1980 	;base year
	OR	A		;clear carry
	SBC	HL,BC		;reduce it
	LD	A,L		;get LSB result
;
DATEZ	AND	7		;force to MOD year
;
DATEY	LD	(DE),A		;put in the year
	XOR	A		;Z = OK
	RET			;done, return
;
;	load TIME$
;
SETTIME LD	DE,TIME$+3	;point to TIME
;
;	fetch hour
;
	CALL	VALUE		;get the value
	JR	C,DATEBAD	;invalid chars
	CP	25		;0-24?
	JR	NC,DATEBAD	;invalid
	LD	(DE),A		;set the hour
	DEC	DE		;point to minutes
;
;	fetch minutes
;
	CALL	VALUE		;get the value
	JR	C,DATEBAD	;invalid chars
	CP	61		;0-60?
	JR	NC,DATEBAD	;invalid
	LD	(DE),A		;set the minute
	DEC	DE		;point to seconds
;
;	fetch seconds
;
	CALL	VALUE		;get the value
	JR	C,DATEBAD	;invalid chars
	CP	61		;0-60?
	JR	NC,DATEBAD	;invalid
	JR	DATEY		;insert, return with Z
;
;	load HL with time and date
;
GETTD	EX	DE,HL		;DE => user string
	CALL	COMPDAYS	;day in year/week
;
;	insert name of day
;
	LD	A,0		;get day # of week
DOW	EQU	$-1		;day of week
	LD	HL,DAYTBL	;day of week table
	CALL	POSTBL		;position the table
	LDIR			;move into user buffer
;
;	insert name of month
;
	LD	A,(DATE$+1)	;get month #
	DEC	A		;adjust 1-12 => 0-11
	LD	HL,MONTBL	;table of months
	CALL	POSTBL		;position the table
	LDIR			;move into user string
	EX	DE,HL		;HL => user string
;
;	insert day of month
;
	LD	A,(DATE$)	;get day of month
	CALL	ASCII		;make it ascii
	CALL	PUTASC		;move into user buffer
;
;	insert ascii year
;
	LD	(HL),'1'        ;insert year prefix
	INC	HL		;bump pointer
	LD	(HL),'9'        ;1900 base year
	INC	HL		;bump pointer
	LD	(HL),'8'        ;put base year
	INC	HL		;bump pointer
	LD	A,(DATE$+2)	;get the year -1900
	ADD	A,'0'           ;make it ascii
	LD	(HL),A		;put in buffer
	INC	HL		;bump pointer
;
;	insert day # of year
;
	PUSH	HL		;save string pointer
	LD	HL,0		;get day of year
DOY	EQU	$-2		;day of year
	LD	BC,100		;compute 100's place
	LD	A,'0'           ;start with this
DOYL	OR	A		;clear carry
	SBC	HL,BC		;remove a hundred
	JR	C,DOYG		;got the 100's
	INC	A		;else bump ascii
	JR	DOYL		;go again
DOYG	ADD	HL,BC		;adjust it back
	LD	B,A		;save the digit
	LD	A,L		;get remainder
	POP	HL		;restore string
	LD	(HL),B		;insert MSB digit
	INC	HL		;bump pointer
	CALL	ASCII		;make it ascii
	CALL	PUTASC		;put NSB and LSB in
;
;	insert TIME$
;
	LD	DE,TIME$+3	;point to hour
	LD	A,3		;3 entries for time
TIMLP	EX	AF,AF'          ;save counter here
	LD	A,(DE)		;get data
	DEC	DE		;move pointer down
	CALL	ASCII		;make it ascii decimal
	CALL	PUTASC		;move B and C into HL
	EX	AF,AF'          ;get counter back
	DEC	A		;done?
	JR	Z,TIMDON	;go if yes
	LD	(HL),':'        ;else insert separator
	INC	HL		;bump pointer
	JR	TIMLP		;go more!
;
;	insert month number
;
TIMDON	LD	A,(DATE$+1)	;get month
	CALL	ASCII		;make it ascii
	CALL	PUTASC		;move into string
;
;	insert day of week number
;
	LD	A,(DOW) 	;get day of week
	ADD	A,'0'           ;adjust to real ascii
	LD	(HL),A		;put into string
	XOR	A		;set Z flag for OK
	JP	DATERET 	;return
;
;	$VALUE - compute value from HL string
;
;	ENT	HL => string
;
;	EXT	HL => terminating char
;		BC = value
;		Carry = invalid digits
;		No Carry = OK
;		Z = number <256
;		NZ = number >255
;
;	terminators are : / , . - <SP> <CR> <ETX>
;
VALUE	LD	BC,0		;starting number
;
VALLP	LD	A,(HL)		;fetch a char
;
;	check for terminator
;
	CALL	CKTERM		;terminator?
	JR	Z,VALDN 	;go if yes
	INC	HL		;else bump pointer
;
;	check for separator
;
	CALL	CKSEP		;separator?
	JR	Z,VALDN 	;go if yes
;
;	remove ascii, check for valid digit
;
	SUB	30H		;remove the ascii
	RET	C		;invalid digit
	CP	10		;must be 0-9
	CCF			;reverse carry flag
	RET	C		;invalid digit
;
;	multiply subtotal times base (10)
;
	PUSH	HL		;save pointer
	LD	H,B		;get subtotal
	LD	L,C		;HL = subtotal
	ADD	HL,HL		;*2
	ADD	HL,HL		;*4
	ADD	HL,BC		;*5
	ADD	HL,HL		;*10
;
;	add new digit
;
	ADD	A,L		;add new digit
	LD	L,A		;update
	JR	NC,VAL1 	;go if no carry
	INC	H		;bump MSB
;
;	restore subtotal to BC
;
VAL1	LD	B,H		;put back into BC
	LD	C,L
	POP	HL		;restore pointer
;
	JR	VALLP		;go next character
;
;	value finished, load registers/flags
;
VALDN	LD	A,B		;get MSB value
	OR	A		;set Z flag on this
	LD	A,C		;but return with LSB
	RET			;done
;
	PAGE
;
;	check for terminator
;
CKTERM	PUSH	HL		;save it
	LD	HL,TRMTBL	;terminator table
	JR	CKTBL		;check it out
;
;	check for separator
;
CKSEP	PUSH	HL		;save it
	LD	HL,SEPTBL	;separator table
;
;	check if char in table
;
CKTBL	PUSH	BC		;save subtotal
	LD	BC,6		;# entries per table
	CPIR			;see if in table
	POP	BC		;restore stack
	POP	HL
	RET			;done, Z = yes
;
;	terminator table
;
TRMTBL	DB	13		;C/R
	DB	';'             ;C/R
	DB	3		;break
	DB	0		;dummy
	DB	0		;dummy
	DB	0		;dummy
;
;	separator table
;
SEPTBL	DB	'/'
	DB	':'
	DB	'.'
	DB	' '
	DB	','
	DB	'-'
;
	PAGE
;
;	ASCII	- convert A to ascii
;
;	ENT	A = binary digit to convert
;
;	EXT	ACB = 3 byte ascii representation
;
ASCII	LD	B,'0'           ;start digit
;
;	convert MSB
;
ASC1	SUB	100		;find 100's place
	JR	C,ASC2		;have it
	INC	B		;bump ascii
	JR	ASC1		;go more
ASC2	PUSH	BC		;save MSB on stack
	ADD	A,100		;adjust it back
	LD	C,'0'           ;start digit
;
;	convert NSB
;
ASC3	SUB	10		;find 10's place
	JR	C,ASC4		;have it
	INC	C		;bump NSB
	JR	ASC3		;go more
;
;	10's and 1's place computed
;
ASC4	ADD	A,3AH		;add 10 + ascii
	LD	B,A		;save LSB here
	POP	AF		;get MSB from stack
	RET			;done, ACB = ascii
;
;	PUTASC - insert ascii into string
;
;	ENT	BC = ascii digits to insert
;		HL => string to load
;
;	EXT	HL => next area (start HL+2)
;		BC unchanged
;
PUTASC	LD	(HL),C		;put NSB
	INC	HL		;bump pointer
	LD	(HL),B		;put LSB
	INC	HL		;bump pointer
	RET			;done, HL => new loc.
;
;	$POSTBL - position table at HL
;
;	ENT	HL => table of 3 byte entries
;		A = entry number to fetch
;
;	EXT	HL => 3 byte entry specified
;		BC = 3 (for LDIR)
;
POSTBL	LD	B,A		;save entry #
	ADD	A,A		;*2
	ADD	A,B		;*3 (3 byte entries)
	LD	BC,3		;setup for LDIR
	ADD	A,L		;add displace to LSB
	LD	L,A		;update it
	RET	NC		;did not cross page
	INC	H		;bump page
	RET			;done
;
;	table of days of week (3 bytes each)
;
DAYTBL	DEFM	'MonTueWedThuFriSatSun'
;
;	table of months (3 bytes each)
;
MONTBL	DEFM	'JanFebMarAprMayJunJulAugSepOctNovDec'
;
	PAGE
;
;	$COMPDAYS - compute day of year and day of week
;
;	ENT	DATE$ contains valid data
;
;	EXT	(DOW) = day # of the week (byte)
;		(DOY) = day # of the year (word)
;
COMPDAYS CALL	REGSAV		;save registers
	LD	A,(DATE$+2)	;get year
	PUSH	AF		;save it
	AND	3		;check for leap year
	LD	HL,MONTHS+1	;point to feb
	LD	A,28		;28 days non-leap
	JR	NZ,CMPD1	;go if not
	INC	A		;else feb = 29
CMPD1	LD	(HL),A		;put in table
	DEC	HL		;HL => max days/month
;
	POP	AF		;get year back
	AND	7		;make sure in range
	PUSH	HL		;save table
	LD	HL,STDAY	;start day table
	ADD	A,L		;add current year
	LD	L,A		;update
	JR	NC,CMPD2	;did not cross page
	INC	H		;else bump it
CMPD2	LD	A,(HL)		;get start day this yr
	POP	HL		;get table back
	EX	AF,AF'          ;save it here
	LD	A,(DATE$+1)	;get month
	LD	D,A		;save here
	LD	E,1		;start relative month
	LD	BC,0		;start DOY
;
CMPLP	LD	A,E		;see if rel mon = curr
	CP	D		;yes?
	JR	Z,CMPLP1	;go if at month
	LD	A,(HL)		;get max this month
	CALL	ADDMTH		;add A # of days
	INC	E		;bump rel month
	INC	HL		;bump table
	JR	CMPLP		;check next month
;
CMPLP1	LD	A,(DATE$)	;get current day
	CALL	ADDMTH		;add # days
	LD	(DOY),BC	;save day in year
	EX	AF,AF'          ;get day in week
	LD	(DOW),A 	;save it
	RET			;done, return
;
;	advance date pointers
;
ADDMTH	EX	AF,AF'          ;get relative DOW
	INC	A		;bump it
	CP	7		;end of week?
	JR	C,ADDMTH0	;go if not
	XOR	A		;else reset it
ADDMTH0 EX	AF,AF'          ;put it back
	INC	BC		;bump DOY
	DEC	A		;less # days to count
	JR	NZ,ADDMTH	;go more if any
	RET			;else done
;
	PAGE
;
;	$ACTL	- SVC 100 - SIO A control
;	$BCTL	- SVC 101 - SIO B control
;
;	ENT	B = command
;
;	EXT	Z = OK
;		NZ = A = error code
;
;	commands:
;	0	- get status
;	1	- get buff char count
;	2	- turn on request to send
;	3	- turn off request to send
;	4	- send break sequence
;	5	- turn off break sequence
;	6	- reset receive buffer nil
;	7	- reset SIO error condition
;
ACTL	LD	A,SADCB 	;CLA DCB #
	JR	ABCTL		;go common
;
BCTL	LD	A,SBDCB 	;CLB DCB #
;
ABCTL	PUSH	IX		;save registers
	PUSH	HL
	PUSH	BC
;
	CALL	LOCDCB		;find DCB
	LD	HL,SIOCTL	;lookup table
	LD	A,LOOKUP	;SVC #
	RST	SVC		;find the command
;
	LD	A,ERR01 	;bad function call
	RET	NZ		;back to caller
;
	LD	A,(IX+22)	;get CTC R5 control
	LD	C,(IX+17)	;get SIO base port
	JP	(HL)		;go vector!
;
SIOC0	CALL	RSTATUS 	;get status
SIOCBAK POP	BC		;get B
	LD	B,A		;put result here
	PUSH	BC		;put it back
	XOR	A		;set Z
;
SIOCRET POP	BC		;unstack
	POP	HL
	POP	IX
	RET			;done
;
SIOC1	LD	A,0		;LSB counter
	JR	SIOCBAK 	;return
;
SIOC2	SET	1,A		;request to send
;
SIOOK	EX	AF,AF'          ;save command
	LD	A,5		;select R5
	OUT	(C),A
	EX	AF,AF'          ;get command
	OUT	(C),A		;send it
	XOR	A		;set Z
	JR	SIOCRET 	;done
;
SIOC3	RES	1,A		;turn off request to send
	JR	SIOOK
;
SIOC4	SET	4,A		;set BREAK
	JR	SIOOK
;
SIOC5	RES	4,A		;reset BREAK
	JR	SIOOK
;
SIOC6	DI			;disable for reset
	CALL	NILBUFF 	;clear out buffer
	EI			;done
	JR	SIOCRET 	;back
;
SIOC7	DI			;disable
	LD	A,30H		;reset, enable char int
	OUT	(C),A
	LD	A,10H		;reset ext/status int
	OUT	(C),A
	LD	A,28H		;reset tx int pend.
	OUT	(C),A
	LD	A,0C0H		;reset tx underrun
	OUT	(C),A
	EI			;done
	XOR	A		;set Z
	JR	SIOCRET 	;back
;
;	lookup table
;
SIOCTL	DB	0		;get status
	DW	SIOC0
	DB	1		;get char counter
	DW	SIOC1
	DB	2		;request to send ON
	DW	SIOC2
	DB	3		;request to send OFF
	DW	SIOC3
	DB	4		;send BREAK
	DW	SIOC4
	DB	5		;reset BREAK
	DW	SIOC5
	DB	6		;reset buffer nil
	DW	SIOC6
	DB	7		;reset SIO error
	DW	SIOC7
;
	DB	-1		;terminator
;
;	fetch status of SIO port
;
RSTATUS LD	C,(IX+17)	;get SIO port
	LD	A,10H		;clear latch
	OUT	(C),A
	IN	A,(C)		;read R0
	AND	0ADH		;brk,cts,dcd,tx,rx
	XOR	28H		;reverse cts,dcd
	LD	B,A		;save it
	LD	A,1		;select R1
	OUT	(C),A
	IN	A,(C)		;read R1
	AND	50H		;framing,parity
	OR	B		;mask with last
	LD	B,A		;pass here
	RET
;
	PAGE
;
;	$INITIO - SVC 0 - initialize I/O
;
;	ENT	none
;
;	EXT	Z = OK
;		NZ = A = error code
;
INITIO	CALL	REGSAV		;save registers
	DI			;kill int
;
;	initialize video
;
	LD	BC,-1		;80 column, normal
	CALL	VDINIT		;initialize video
	RET	NZ		;if error
;
;	initialize keyboard
;
	CALL	KBINIT		;do it
	RET	NZ		;if error
;
;	initialize printer
;
	LD	BC,423CH	;B=page len, C=ptrd lns
	LD	D,84H		;D=chars/line
	CALL	PRINIT		;init it
;
	LD	A,PAGE0<-8	;int vector MSB
	LD	I,A		;set vector
	EI			;int ok
	IN	A,(0FEH)	;clear latch
	RET
;
JPINIT	XOR	A
	RST	8
	RST	0
;
;
ZZZZZ	EQU	$
;
	END	VECTORS
