DOSSYS ; dossys9/asm
 ;
 	TITLE	'<DOS+II - SYS09/SYS - Series A.00>'
 ;
 ;	THIS SYSTEM IS HIGH RESIDING (2200-27FF)
 ;	IT CONTAINS THE FOLLOWING ROUTINES:
 ;
 ;	1 -	$CAT	- FILE CATALOG
 ;	2 -	$DIR	- FILE DIRECTORY
 ;
 ;	Created: 17-JUN-82	SJP/MSS
 ;	Revised: 17-JUN-82	SJP/MSS
 ;
 	PAGE
 ;	LOCAL SUPPORT EQUIVALENCES
 ;
 BADF	EQU	1		;BAD FUNCTION CALL
 ERR3	EQU	3		;PARAMETER ERROR
 ERR8	EQU	8		;DRIVE NOT READY
 NCOL	EQU	4		;CAT COLS
 NROW	EQU	20		;CAT ROWS
 CR	EQU	13		;ASCII C/R
 ETX	EQU	3		;ASCII ETX
 BREAK	EQU	3		;BREAK KEY
 DDCB?	EQU	1		;DISPLAY DEVICE#
 LDA	EQU	3EH		;LD A,N
 LDB	EQU	01H		;LD BC,NNNN
 ;
 BUFER1	EQU	2700H		;GENERAL BUFFER
 BUFER2	EQU	2800H		;DISK I/O BUFFER
 BUFER3	EQU	2900H		;SORT I/O BUFFER
 ;
 ;	EXTERNAL SVC EQUIVALENCES
 ;
 KBD	EQU	4		;KEYBOARD CHARACTER
 BINDEC	EQU	21		;BINARY/DECIMAL CONV
 HLDKEY	EQU	29		;PROCESS HOLD KEY
 ERROR	EQU	39		;POST ERROR MESSAGE
 OPEN	EQU	40		;OPEN DEVICE/FILE
 CLOSE	EQU	42		;CLOSE DEVICE/FILE
 WILD	EQU	51		;WILD CARD PARSER
 SORT	EQU	56		;RAM SORT
 KBDW	EQU	73		;KBD INPUT W/WAIT
 PUT	EQU	78		;OUPPUT BYTE DEVICE
 EVAL	EQU	83		;EVALUATE CMD LINE
 ;
 ;	EXTERNAL I/O EQUIVALENCES
 ;
 DCHECK	EQU	0		;CHECK DRIVE READY
 SREAD	EQU	6		;READ SYSTEM SECTOR
 ;
 ;	EXTERNAL SUPPORT EQUIVALENCES
 ;
 SVC	EQU	08H		;EXECUTE SVC
 DIO	EQU	10H		;DISK I/O MANAGER
 ;
 DEVTBL	EQU	0100H		;DEVICE TABLE
 TMULT	EQU	0318H		;8 X 24 MULTIPLY
 TDIVD	EQU	031BH		;8 X 24 DIVIDE
 DIRRED	EQU	0324H		;DIRECTORY READ
 LOCDCB	EQU	032AH		;LOCATE DCB
 LOCDCT	EQU	032DH		;LOCATE DCT
 ;
 	PAGE
 ;	TABLE OF VECTORED ROUTINES
 ;
 	ORG	2200H		;HIGH OVERLAY
 ;
 START	DEFW	RETURN		;0  - NOTHING DONE
 	DEFW	CAT		;1  - FILE CATALOG
 	DEFW	DIR		;2  - FILE DIRECTORY
 	DEFW	UNDEF		;3  - UNDEFINED
 	DEFW	UNDEF		;4  - UNDEFINED
 	DEFW	UNDEF		;5  - UNDEFINED
 	DEFW	UNDEF		;6  - UNDEFINED
 	DEFW	UNDEF		;7  - UNDEFINED
 	DEFW	UNDEF		;8  - UNDEFINED
 	DEFW	UNDEF		;9  - UNDEFINED
 	DEFW	UNDEF		;10 - UNDEFINED
 	DEFW	UNDEF		;11 - UNDEFINED
 	DEFW	UNDEF		;12 - UNDEFINED
 	DEFW	UNDEF		;13 - UNDEFINED
 	DEFW	UNDEF		;14 - UNDEFINED
 	DEFW	UNDEF		;15 - UNDEFINED
 ;
 ;	UNDEFINED ROUTINE HANDLER
 ;
 UNDEF	LD	A,BADF		;BAD FUNCTION CALL
 	OR	A		;NZ STATUS
 RETURN	RET			;EXIT
 ;
 	PAGE
 ;	FILE CATALOG/DIRECTORY ROUTINES
 ;
 ;	ENT	HL =>	COMMAND LINE
 ;
 ;	INITIALIZE PARAMETERS
 ;
 DIR	DEFB	LDA		;DUMMY LD
 CAT	XOR	A		;0/NZ
 	LD	(D?C),A 	;TO FLAG
 	XOR	A		;0
 	LD	(I?C),A 	;INV
 	LD	(S?C),A 	;SYS
 	LD	(K?C),A 	;KILL
 	LD	(A?C),A 	;SORT
 ;
 ;	EVALUATE OPTIONAL FIELDS
 ;
 	LD	IX,PBLOCK	;IX => PARAM BLOCK
 	LD	A,EVAL		;EVAL CMD LINE
 	RST	SVC		;DO IT
 	JP	NZ,POSTE	;ERROR!
 ;
 ;	CHECK FOR PROPER FIELD TYPES
 ;
 	BIT	2,(IX+0)	;ANY SOURCE?
 	LD	C,0		;DEFAULT
 	JR	Z,CAT1		;IF GLOBAL
 	LD	A,(DCB?1)	;GET DEVICE
 	BIT	6,A		;FILESPEC?
 	JP	NZ,ERR1 	;ILLEGAL DRIVESPEC
 	AND	0FH		;IGNORE FLAGS
 	SUB	8		;CORRECT
 	JP	C,ERR1		;IF ERROR
 	LD	C,A		;C = DEVICE
 ;
 CAT1	BIT	1,(IX+0)	;DEST FILLED?
 	LD	A,DDCB? 	;DISPLAY DEVICE
 	JR	NZ,CAT2 	;IF YES - SKIP
 	LD	(DCB?2),A	;DEFAULT DEVICE
 ;
 	PAGE
 ;	INITIALIZE FIELDS
 ;
 ;	ENT	DCB?1 => OPTIONAL SOURCE
 ;		DCB?2 => OPTIONAL DEST
 ;		DCB?3 => OPTIONAL MASK
 ;
 CAT2	CALL	DOPEN		;OPEN DEST DEVICE
 	JP	NZ,POSTE	;ERROR!
 	CALL	WILD1		;CRACK WILD SPEC
 ;
 ;	INITIALIZE VARIABLES
 ;
 CAT3	LD	A,NCOL		;DEFAULT COL
 	LD	(COL),A 	;COL COUNT
 	LD	A,NROW		;DEFAULT ROW
 	LD	(ROW),A 	;ROW COUNT
 	LD	A,(A?C) 	;SORT FLAG
 	LD	(PAS),A 	;TO PASS
 ;
 ;	READ GAT FOR NAME/DATE
 ;
 	LD	A,DCHECK	;CHECK DRIVE
 	RST	DIO		;DO IT
 	JR	Z,CAT14 	;ALL OK
 	BIT	2,(IX+0)	;GLOBAL?
 	JP	Z,CAT16 	;IF YES
 	JP	ERR2		;ERROR!
 ;
 CAT14	LD	HL,BUFER1	;I/O BUFFER
 	LD	E,0		;GAT SECTOR
 	LD	A,SREAD 	;SYSTEM READ
 	RST	DIO		;DO IT
 	JP	NZ,POSTE	;I/O ERROR!
 	CALL	LOCDCT		;LOCATE DCT
 ;
 ;	FETCH DEVICE NAME/DATE
 ;
 	PUSH	BC		;SAVE REG
 	LD	H,DEVTBL<-8	;START DEVICE TABLE
 	LD	A,C		;DRIVE
 	ADD	A,8		;CORRECT
 	ADD	A,A		;* 2
 	ADD	A,A		;* 4
 	ADD	A,2		;TO NAME
 	LD	L,A		;HL => DEVICE NAME
 	LD	DE,CTITLE+13	;TITLE OFFSET
 	LD	BC,2		;COUNT
 	LDIR
 	LD	HL,BUFER1+0D0H	;HL => DISK TITLE
 	LD	DE,CTITLE+16	;TITLE OFFSET
 	LD	C,8		;BC = COUNT
 	LDIR
 	LD	DE,CTITLE+28	;TITLE OFFSE
 	LD	C,8		;BC = COUNT
 	LDIR
 	POP	BC
 ;
 ;	OUTPUT CATALOG TITLE
 ;
 	LD	HL,CTITLE	;CATALOG TITLE
 	CALL	LINE$		;TO DEVICE
 	LD	HL,CSUBTL	;SUB TITLE
 D?C	EQU	$+1
 	LD	DE,0		;GET PARAM
 	INC	E
 	DEC	E		;D SWITCH?
 	CALL	NZ,LINE$	;OUTPUT SUBTTL
 	CALL	OUTCR		;SKIP LINE
 ;
 	PAGE
 ;	INITIALIZE DIRECTORY READ
 ;
 CAT23	LD	E,2		;START DIRECTORY
 CAT24	LD	HL,BUFER1	;GENERAL BUFFER
 	LD	A,(A?C) 	;SORT PARAM
 	OR	A		;SORT?
 	JR	Z,CAT22 	;IF NOT
 	LD	H,BUFER3<-8	;HL => SORT BUFFER
 ;
 CAT21	LD	A,(PAS) 	;PASS COUNT
 	OR	A		;SORT PASS?
 	JR	Z,CAT5		;IF NOT
 ;
 CAT22	LD	A,SREAD 	;SYSTEM READ
 	RST	DIO		;DO IT
 	JP	NZ,POSTE	;ERROR!
 	LD	A,(PAS) 	;PASS COUNT
 	OR	A		;SORT PASS?
 	JP	NZ,CAT17	;IF YES
 ;
 ;	FIND NEXT ACTIVE FILE
 ;
 CAT5	PUSH	HL		;SAVE REG
 	PUSH	DE
 	PUSH	BC
 ;
 	BIT	7,(HL)		;FXDE?
 	JP	NZ,CAT13	;NEXT FILE
 ;
 ;	CHECK FOR DELETED OPTION
 ;
 	BIT	4,(HL)		;DELETED?
 	JR	NZ,CAT20	;IF NOT
 K?C	EQU	$+1
 	LD	DE,0		;GET PARAM
 	INC	E
 	DEC	E		;K SWITCH?
 	JP	Z,CAT13 	;NEXT FILE
 	JR	CAT7		;SKIP
 ;
 ;	CHECK FOR SYSTEM OPTION
 ;
 CAT20	BIT	6,(HL)		;SYSTEM?
 	JR	Z,CAT6		;IF NOT
 S?C	EQU	$+1
 	LD	DE,0		;GET PARAM
 	INC	E
 	DEC	E		;S SWITCH?
 	JP	Z,CAT13 	;NEXT FILE
 	JR	CAT7		;SKIP
 ;
 ;	CHECK FOR INVIS OPTION
 ;
 CAT6	BIT	3,(HL)		;INVISIBLE?
 	JR	Z,CAT7		;SKIP
 I?C	EQU	$+1
 	LD	DE,0		;GET PARAM
 	INC	E
 	DEC	E		;I SWITCH?
 	JP	Z,CAT13 	;NEXT FILE
 ;
 ;	COMPARE FILE WITH MASK
 ;
 CAT7	LD	A,5		;OFFSET
 	ADD	A,L
 	LD	L,A		;HL => NAME
 	LD	A,(HL)		;GET 1ST CHAR
 	OR	A		;NULL?
 	JP	Z,CAT13 	;NEXT FILE
 	CALL	WILD2		;CHECK!
 	JP	NZ,CAT13	;NEXT FILE
 ;
 ;	OUTPUT FILE'S NAME
 ;
 	LD	BC,8<8+16	;NAME + TAB
 ;
 CAT8	LD	A,(HL)		;GET CHAR
 	CP	' '             ;SHORT?
 	JR	Z,CAT9		;IF YES
 	CALL	PUT$		;TO DEVICE
 	INC	L		;NEXT
 	DEC	C		;TAB
 	DJNZ	CAT8		;TIL DONE
 ;
 ;	OUTPUT FILE'S EXTENSION
 ;
 CAT9	LD	A,B		;OFFSET
 	ADD	A,L
 	LD	L,A		;HL => EXT
 	LD	A,(HL)		;GET CHAR
 	CP	' '             ;ANY EXT?
 	JR	Z,CAT4		;IF NOT
 	LD	A,'/'
 	CALL	PUT$		;EXT SPEC
 	DEC	C		;TAB
 ;
 CAT4	LD	B,3		;EXT LENGTH
 CAT10	LD	A,(HL)		;GET CHAR
 	CALL	PUT$		;OUTPUT
 	INC	L		;NEXT
 	DEC	C		;TAB
 	DJNZ	CAT10		;TIL DONE
 	LD	A,C		;TAB COUNT
 	CALL	TAB		;TAB!
 ;
 	PAGE
 ;	CHECK FOR DIRECTORY INFO
 ;
 	LD	A,(D?C) 	;D SWITCH
 	OR	A		;TRUE?
 	JP	Z,CAT11 	;IF NOT
 ;
 ;	OUTPUT FILE'S PROTECTION
 ;
 	LD	DE,FLAGN	;'PKSDIM'
 	LD	A,(HL)		;LSB UPDATE
 	INC	L		;NEXT
 	CP	96H		;NULL?
 	JR	NZ,DIR7 	;IF NOT
 	LD	A,(HL)		;MSB UPD/ACC
 	INC	L
 	CP	22H		;NULL?
 	JR	NZ,DIR7 	;IF NOT
 	LD	A,(HL)		;LSB ACCESS
 	CP	96H		;NULL?
 	JR	Z,DIR8		;IF YES
 ;
 DIR7	SCF			;FLAG TRUE
 DIR8	PUSH	AF		;SAVE FLAG
 	LD	A,L
 	AND	0E0H
 	LD	L,A		;HL => START ENTRY
 	LD	A,(HL)		;GET ATTRIB
 	AND	7		;ACCESS ONLY
 	ADD	A,'0'           ;ASCII
 	CALL	PUT$		;TO DEVICE
 	POP	AF		;GET FLAG
 	CALL	SWTCH		;OUTPUT
 ;
 ;	OUTPUT FILE'S ATTRIBUTES
 ;
 	LD	A,(HL)		;GET ATTRIB
 	INC	L		;HL => FLAGS
 	RLCA
 	RLCA			;! TO CRY
 	CALL	SWTCH		;OUTPUT
 	RLCA			;S TO CRY
 	CALL	SWTCH		;OUTPUT
 	RLCA			;K TO CRY
 	CCF			;INVERT
 	CALL	SWTCH		;OUTPUT
 	RLCA			;I TO CRY
 	CALL	SWTCH		;OUTPUT
 	LD	A,(HL)		;GET FLAGS
 	INC	L
 	INC	L		;HL => EOF
 	LD	B,(HL)		;B = EOF BYTE
 	INC	L		;HL => LRL
 	RLCA
 	RLCA			;+ TO CRY
 	CALL	SWTCH		;OUTPUT
 ;
 ;	OUTPUT FILE'S LRL
 ;
 	LD	A,(HL)		;GET LRL
 	LD	E,A		;E = LRL
 	SUB	1		;256?
 	LD	A,0
 	ADC	A,0
 	LD	D,A		;DE = LRL
 	CALL	DEC		;OUPUT
 	LD	A,2		;COUNT
 	CALL	TAB		;TAB!
 ;
 ;	OUTPUT PHYSICAL RECORDS
 ;
 	LD	C,(HL)		;C = LRL
 	LD	A,10H
 	ADD	A,L
 	LD	L,A		;HL => PHY RECS
 	LD	E,(HL)
 	INC	L
 	LD	D,(HL)		;DE = PHY RECS
 	INC	L		;HL => SD LIST
 	CALL	DEC		;OUTPUT
 	LD	A,2		;COUNT
 	CALL	TAB		;TAB!
 ;
 ;	OUTPUT LOGICAL RECORDS
 ;
 	PUSH	HL		;SAVE SD LIST
 	LD	A,C		;GET LRL
 	OR	A		;BLK'D?
 	JR	Z,DIR2		;IF NOT
 	LD	A,B		;EOF
 	OR	A		;ANY EOF?
 	JR	Z,$+3		;IF NOT
 	DEC	DE		;CORRECT
 	LD	L,B
 	LD	H,E
 	LD	B,D		;BHL = BYTES
 	LD	A,C		;GET LRL
 	CALL	TDIVD		;BYTES/LRL
 	EX	DE,HL		;DE = RESULT
 	OR	A		;PARTIAL REC?
 	JR	Z,$+3		;IF NOT
 	INC	DE		;CORRECT
 ;
 DIR2	CALL	DEC		;OUTPUT
 	LD	A,1		;COUNT
 	CALL	TAB		;TAB!
 	POP	HL		;RESTORE SD LIST
 ;
 	PAGE
 ;	DETERMINE ALLOCATED SPACE
 ;
 	POP	BC
 	PUSH	BC		;C = DRIVE
 ;
 	PUSH	HL		;SAVE ENTRY
 	XOR	A
 	LD	B,A		;INIT SEG COUNT
 	LD	E,A
 	LD	D,A		;INIT GRAN COUNT
 ;
 DIR3	LD	A,(HL)		;GET LSB SD
 	INC	L
 	CP	0FEH		;TERM SD?
 	JR	NC,DIR4 	;POSSIBLE
 	LD	A,(HL)		;GET GRANS
 	INC	L		;HL => NEXT SD
 	AND	1FH		;GRAN COUNT
 	INC	A		;CORRECT
 	ADD	A,E
 	LD	E,A
 	JR	NZ,$+3
 	INC	D		;DE = RESULT
 	INC	B		;B = SD COUNT
 	JR	DIR3		;TIL END SD
 ;
 DIR4	JR	NZ,DIR6 	;END LIST
 	PUSH	BC		;SAVE SEG COUNT
 	LD	B,(HL)		;B = LFN
 	CALL	DIRRED		;READ ENTRY
 	POP	BC		;GET SEG COUNT
 	JR	Z,DIR5		;READ OK
 	POP	BC		;FLUSH STACK
 	POP	BC
 	POP	BC
 	POP	BC
 	RET
 ;
 DIR5	LD	A,16H
 	ADD	A,L
 	LD	L,A		;HL => SD LIST
 	JR	DIR3		;TIL DONE
 ;
 ;	OUTPUT FILE'S SEG COUNT
 ;
 DIR6	PUSH	DE		;SAVE GRANS
 	LD	E,B
 	LD	D,0		;DE = SEG
 	CALL	DEC		;OUTPUT
 	LD	A,1		;TAB COUNT
 	CALL	TAB		;TAB!
 	POP	HL		;HL = GRANS
 	LD	B,0		;BHL = GRANS
 ;
 ;	OUTPUT FILE'S ALLOC SPACE
 ;
 	LD	A,(IY+20)	;SEC/GRAN
 	CALL	TMULT		;SEC * GRANS
 	LD	B,H
 	LD	H,L
 	LD	L,A		;BHL = SECTORS
 	CALL	SIZEK		;OUTPUT SIZE IN K
 	LD	A,2		;TAB COUNT
 	CALL	TAB		;TAB!
 	POP	HL		;HL => SD LIST
 ;
 ;	OUTPUT LAST MODIFIED DATE
 ;
 	LD	A,L
 	AND	0E0H
 	LD	L,A		;HL => START ENTRY
 	INC	L		;HL => MONTH
 	LD	A,(HL)		;GET FLAGS/MONTH
 	INC	L		;HL => YEAR/DAY
 	AND	0FH		;MONTH ONLY
 	LD	B,A		;B = MONTH
 	LD	C,(HL)		;C = YEAR/DAY
 	CALL	DATEO		;OUTPUT DATE
 	CALL	OUTCR		;OUTPUT C/R
 	JR	CAT25		;SKIP
 ;
 	PAGE
 ;	CHECK FOR LAST COLUMN
 ;
 CAT11	LD	HL,COL		;COL COUNT
 	DEC	(HL)		;NEXT
 	JR	NZ,CAT13	;NO EOL
 	LD	(HL),NCOL	;NEXT LINE
 	CALL	OUTCR		;OUTPUT C/R
 ;
 ;	CHECK FOR LAST ROW
 ;
 CAT25	LD	HL,ROW		;ROW
 	DEC	(HL)		;NEXT
 	JR	NZ,CAT12	;NOT FULL
 	LD	(HL),NROW	;NEW SCREEN
 	LD	A,(DCB?2)	;DCB INFO
 	AND	0FH		;GET DEVICE
 	DEC	A		;DISPLAY?
 	JR	NZ,CAT12	;IF NOT
 	LD	A,KBDW		;WAIT FOR KEY
 	RST	SVC		;DO IT!
 ;
 ;	CHECK FOR HOLD KEY
 ;
 CAT12	LD	B,2		;CHECK HOLD
 	LD	A,HLDKEY	;HOLD ROUTINE
 	RST	SVC		;DO IT!
 ;
 ;	SET-UP FOR NEXT FILE
 ;
 CAT13	POP	BC		;RESTORE REG
 	POP	DE
 	POP	HL
 ;
 	LD	A,20H
 	ADD	A,L
 	LD	L,A		;HL => NEXT FILE
 	JP	NC,CAT5 	;NO READ
 ;
 CAT17	LD	A,(IY+18)	;SEC/DIR
 	DEC	A		;LAST SECTOR
 	INC	H		;NEXT BUFFER
 	INC	E		;NEXT
 	CP	E
 	JR	C,CAT18
 	LD	A,(A?C) 	;SORT FLAG
 	OR	A		;SORT?
 	JP	NZ,CAT21	;IF YES
 	JP	CAT24		;SAME BUFFER
 ;
 CAT18	LD	A,(PAS) 	;GET PASS
 	XOR	-1		;FLIP
 	JR	NZ,CAT19	;NO SORT
 	LD	(PAS),A 	;SET PASS
 ;
 	PUSH	BC		;SAVE REG
 	PUSH	IX
 	LD	IX,BUFER3	;START TABLE
 	LD	DE,0E0H 	;LAST ENTRY OFFSET
 	DEC	H		;HL => LAST PAGE
 	ADD	HL,DE		;HL => LAST ENTRY
 	EX	DE,HL		;DE => LAST ENTRY
 	LD	BC,5<8+32	;OFFSET+LENGTH
 	LD	HL,0<8+11	;ASCENDING+LENGTH
 	LD	A,SORT		;SORT!
 	RST	SVC		;DO IT!
 	POP	IX		;RESTORE
 	POP	BC		;RESTORE
 	JP	CAT23		;NEXT PASS
 ;
 CAT19	LD	A,(COL) 	;GET COL
 	CP	NCOL		;PARTIAL?
 	CALL	NZ,OUTCR	;CLEAN OUTPUT
 	CALL	OUTCR		;OUTPUT C/R
 ;
 ;	NEXT DRIVE IF AVAILABLE
 ;
 CAT16	BIT	2,(IX+0)	;GLOBAL?
 	JR	NZ,CAT15	;IF NOT
 	LD	A,7		;LAST DRIVE
 	INC	C		;NEXT
 	CP	C		;END CHAIN?
 	JP	NC,CAT3 	;IF NOT
 ;
 CAT15	CALL	DCLOSE		;CLOSE DEVICE
 	RET
 ;
 	PAGE
 ;	MESSAGES AND TEXT STRINGS
 ;
 CTITLE	DEFM	'Files drive: DS diskname -- mm/dd/yy'
 	DEFB	13
 	DEFB	3
 CSUBTL	DEFM	'Filespec         Attrib  LRL  #Secs  #Recs  #Seg    Space   Updated'
 	DEFB	13
 	DEFB	3
 ;
 FLAGN	DEFM	'PS!KI+'
 ;
 PAS	DEFB	0		;PASS FLAG
 COL	DEFB	0		;COL COUNT
 ROW	DEFB	0		;ROW COUNT
 A?C	DEFW	0		;SORT SWITCH
 ;
 ;	$EVAL PARAMETER BLOCK
 ;
 PBLOCK	DEFB	0		;WORK FLAGS
 	DEFW	DCB?1		;SOURCE DCB
 	DEFW	DCB?2		;DEST DCB
 	DEFW	DCB?3		;MASK DCB
 	DEFW	PLIST		;PARAMETER LIST
 ;
 ;	$PARAM PARAMETER BLOCK
 ;
 PLIST	DEFB	20H+3-1 	;SWITCH
 	DEFW	I?C
 	DEFM	'INV'
 	DEFB	20H+3-1 	;SWITCH
 	DEFW	S?C
 	DEFM	'SYS'
 	DEFB	20H+4-1 	;SWITCH
 	DEFW	K?C
 	DEFM	'KILL'
 	DEFB	20H+4-1 	;SWITCH
 	DEFW	A?C
 	DEFM	'SORT'
 	DEFB	0		;END LIST
 ;
 ;	RESERVED DATA AREAS
 ;
 DCB?1	DEFS	41		;SOURCE DCB
 DCB?2	DEFS	41		;DEST DCB
 DCB?3	DEFS	41		;MASK DCB
 ;
 	PAGE
 ;	S/R TO OUTPUT SWITCH
 ;
 ;	ENT	DE =>	FLAG NAME
 ;		C SET IF TRUE
 ;
 SWTCH	PUSH	AF		;SAVE
 	LD	A,(DE)		;GET FLAG
 	INC	DE		;NEXT
 	JR	C,$+4		;OK
 	LD	A,'.'           ;OFF
 	CALL	PUT$		;OUTPUT
 	POP	AF		;RESTORE
 	RET
 ;
 ;	OUTPUT BYTE TO DEST DEVICE
 ;
 OUTCR	LD	A,CR		;C/R
 ;
 PUT$	PUSH	DE		;SAVE
 	PUSH	BC
 	LD	DE,DCB?2+1	;DEVICE DCB
 	LD	B,A		;B = CHAR
 	LD	A,PUT		;PUT CHAR
 	RST	SVC		;DO IT
 	POP	BC
 	POP	DE		;RESTORE
 	RET
 ;
 ;	OUTPUT LINE TO DEST DEVICE
 ;
 ;	ENT	HL =>	ASCII TEXT
 ;
 LINE$	LD	A,(HL)		;GET CHAR
 	INC	HL
 	CP	ETX		;END TEXT?
 	RET	Z		;IF YES
 	CALL	PUT$		;OUTPUT
 	JR	LINE$		;IF NOT
 ;
 ;	OUTPUT SPECIFIED TAB
 ;
 ;	ENT	 A =	TAB COUNT
 ;
 TAB	PUSH	AF		;SAVE COUNT
 	LD	A,' '           ;BLANK
 	CALL	PUT$		;OUTPUT
 	POP	AF		;GET COUNT
 	DEC	A		;NEXT
 	JR	NZ,TAB		;TIL DONE
 	RET
 ;
 ;	OPEN DEST DEVICE FOR I/O
 ;
 DOPEN	LD	A,(DCB?2)	;GET DEVICE
 	AND	0FH		;IGNORE FLAGS
 	CP	8		;FILESPEC?
 	JR	C,OPEN1 	;IF NOT
 	LD	DE,DCB?2+1	;DEST DCB
 	LD	HL,OLIST	;OPEN PARAM LIST
 	LD	A,OPEN		;OPEN
 	RST	SVC		;DO IT!
 	RET
 ;
 OPEN1	PUSH	IX		;SAVE
 	CALL	LOCDCB		;LOCATE DCB
 	LD	(DCB?2+4),IX	;SET ROUTE
 	LD	A,10H		;ROUTED DEVICE
 	LD	(DCB?2+1),A	;SET ROUTE
 	POP	IX		;RESTORE
 	XOR	A
 	RET
 ;
 ;	CLOSE DEST DEVICE I/O
 ;
 DCLOSE	LD	DE,DCB?2+1	;DEST DCB
 	LD	A,(DE)		;GET TYPE
 	AND	80H		;DEVICE?
 	RET	Z		;ALL OK
 	LD	A,CLOSE 	;CLOSE DEVICE
 	RST	SVC		;DO IT
 	RET
 ;
 ;	OPEN PARAMETER LIST
 ;
 OLIST	DEFW	BUFER2		;BUFADR
 	DEFW	0		;RECADR
 	DEFW	0		;EOFADR
 	DEFM	'W'             ;READ/WRITE
 	DEFB	0		;LRL = 256
 	DEFB	'F'             ;FIXED LENGTH
 	DEFB	2		;CREATE
 ;
 ;	S/R TO OUTPUT SIZE IN K
 ;
 ;	ENT	BHL =	PAGE COUNT
 ;
 SIZEK	LD	A,4		;PAGES/K
 	CALL	TDIVD		;GET K
 	PUSH	AF		;SAVE REMAINDER
 	EX	DE,HL		;DE = RESULT
 	CALL	DEC		;OUTPUT
 	LD	A,'.'           ;DECIMAL
 	CALL	PUT$		;TO DEVICE
 	POP	AF		;GET REMAIN
 	LD	E,A		;E = REMAIN
 	ADD	A,A		;* 2
 	ADD	A,A		;* 4
 	ADD	A,E		;* 5
 	SRL	A		;/ 2
 	ADD	A,'0'           ;ASCII
 	CALL	PUT$		;TO DEVICE
 	LD	A,'k'           ;IN K
 	JP	PUT$		;TO DEVICE
 ;
 ;	S/R TO UNPACK DATE & OUTPUT
 ;
 ;	ENT	 B =	MONTH
 ;		 C =	YEAR/DAY
 ;
 DATEO	LD	A,B		;GET MONTH
 	CALL	BDEC		;OUTPUT
 	LD	A,'/'           ;DELIM
 	CALL	PUT$		;OUTPUT
 	LD	A,C		;GET MONTH/YEAR
 	AND	1FH		;MONTH ONLY
 	CALL	BDEC		;OUTPUT
 	LD	A,'/'           ;DELIM
 	CALL	PUT$		;OUTPUT
 	LD	A,C		;GET MONTH/YEAR
 	RLCA
 	RLCA
 	RLCA			;TO BITS 0-2
 	AND	7		;YEAR ONLY
 	ADD	A,80		;START 1980
 	CALL	BDEC		;OUTPUT
 	RET
 ;
 ;	CONVERT BINARY TO ASCII & OUTPUT
 ;
 ;	ENT	DE =	BINARY
 ;
 BDEC	PUSH	BC		;SAVE
 	LD	E,A
 	LD	D,0		;DE = BINARY
 	LD	BC,3<8+0	;DIGIT REQUEST
 	JR	DEC0		;SKIP
 ;
 DEC	PUSH	BC		;SAVE
 	LD	BC,4<8+1	;DIGIT REQUEST
 ;
 DEC0	PUSH	HL		;SAVE REG
 	PUSH	BC
 	LD	HL,TBUFF	;TEMP DATA
 	LD	B,0		;BIN TO ASCII
 	LD	A,BINDEC	;BINARY/DEC
 	RST	SVC		;DO IT!
 	POP	BC		;GET DIGIT REQUEST
 	BIT	0,C		;PRINT LEAD?
 	PUSH	AF		;SAVE FLAG
 	LD	C,5		;PRECISION
 ;
 DEC1	LD	A,(HL)		;GET DIGIT
 	CP	'0'             ;LEAD 0?
 	JR	NZ,DEC2 	;IF NOT
 	POP	AF		;GET FLAG
 	PUSH	AF		;SAVE FLAG
 	LD	A,' '           ;BLANK
 	CALL	NZ,PUT$ 	;OUTPUT
 	INC	HL		;NEXT
 	DEC	C		;NEXT
 	DJNZ	DEC1		;TIL DONE
 ;
 DEC2	POP	AF		;GET STATUS
 DEC3	LD	A,(HL)		;GET DIGIT
 	CALL	PUT$		;OUTPUT
 	INC	HL		;NEXT
 	DEC	C		;NEXT
 	JR	NZ,DEC3 	;TIL DONE
 	POP	HL		;RESTORE REG
 	POP	BC
 	RET
 ;
 TBUFF	DEFS	5		;TEMP BUFFER
 ;
 ;	S/R TO CRACK OPEN WILD MASK
 ;
 WILD1	BIT	0,(IX+0)	;ANY WILD?
 	RET	Z		;IF NOT
 	LD	B,0		;SET WILD
 	LD	HL,DCB?3+1	;HL => MASK
 	LD	A,WILD		;WILD ROUTINE
 	RST	SVC		;DO IT!
 	RET
 ;
 ;	S/R TO CHECK WILD MASK
 ;
 WILD2	BIT	0,(IX+0)	;ANY WILD?
 	RET	Z		;IF NOT
 	LD	B,3		;CHECK CRACKED
 	LD	A,WILD		;WILD ROUTINE
 	RST	SVC		;DO IT!
 	RET
 ;
 ;	POST ERROR MESSAGE & RETURN
 ;
 ERR1	LD	A,ERR3		;'ILLEGAL DRIVESPEC'
 	DEFB	LDB
 ERR2	LD	A,ERR8		;'DRIVE NOT READY'
 ;
 POSTE	AND	7FH		;NO ABORT
 	LD	B,A		;B = CODE
 	LD	A,ERROR 	;POST ERROR
 	RST	SVC		;DO IT!
 	XOR	A		;NO ERROR
 	RET
 ;
 	END	START
