; map/asm - kjw/bqsd - 08/82
;
	TITLE	'<DosPLUS II.A - Map Utility>'
;
SVC	EQU	08H		;execute SVC
DISKIO	EQU	10H		;direct disk I/O
OVLY	EQU	18H		;load/run overlay
;
DCHECK	EQU	0		;check drive for ready
SREAD	EQU	6		;system sector read
;
VALUE	EQU	89		;fetch value from string
SETBRK	EQU	03		;break processor
HLDKEY	EQU	29		;hold key processor
BINDEC	EQU	21		;binary => decimal
WILD	EQU	51		;wild parser
KBCHAR	EQU	04		;char from keyboard
OPEN	EQU	40		;open a file
BINHEX	EQU	24		;binary => hex ascii
VDCHAR	EQU	08		;char to video
FSPEC	EQU	84		;fetch filespec
ERROR	EQU	39		;display error message
VIDKEY	EQU	12		;video/keyboard
;
ERR32	EQU	32		;invalid drivespec
ERR47	EQU	47		;required param not found
ERR58	EQU	58		;device not avail
ERR86	EQU	86		;program terminated
;
DIRRED	EQU	0324H		;direct LFN read
LOCDCT	EQU	032DH		;load IY with DCT
IOBUFF	EQU	1A00H		;last page of overlay
;
	PAGE
;
	ORG	2200H		;high overlay area
;
;	program entry point
;
ENTRY	LD	(STACK),SP	;save stack pointer
;
	PUSH	HL		;save command pointer
	LD	HL,HELLO	;sign on message
	CALL	DISPLAY		;display it
;
	LD	HL,0		;break processor vector
	LD	A,SETBRK	;SVC #
	RST	SVC		;turn it off
;
	LD	B,0		;turn off hold
	LD	A,HLDKEY	;SVC #
	RST	SVC		;turn it off
	POP	HL		;restore pointer
;
;	interpret user parameters
;
INTPAR	CALL	POSHL		;any input?
	JR	Z,ASKPAR	;nope, prompt for 'em
;
	LD	DE,DCB		;dcb area
	LD	A,FSPEC		;evaluate input
	RST	SVC		;fetch it
	JR	NZ,POSTE	;error!
;
;	check type of map
;
	LD	A,(DE)		;get evaluated flag
	BIT	6,A		;filespec here?
	JP	NZ,MAPFILE	;map a file
;
;	check that drive was specified
;
	BIT	7,A		;device here?
	LD	B,ERR47		;required param
	JR	Z,POSTEB	;error, go!
	AND	0FH		;get device number
	CP	8		;must 8-15
	LD	A,ERR32		;invalid drive
	JR	C,POSTEB	;error!
;
	CALL	POSHL		;any more input?
	JP	Z,MAPDISK	;nope, map the disk
;
;	check for track, sector to map
;
	LD	A,VALUE		;fetch value
	RST	SVC
	JR	NZ,POSTE
	LD	A,C		;get value
	LD	(TRACK),A	;save track
;
	CALL	POSHL		;more input?
	LD	A,ERR47		;required not found
	JR	Z,POSTE
	LD	A,VALUE		;fetch value
	RST	SVC
	JR	NZ,POSTE
	LD	A,C		;get value
	LD	(SECTOR),A	;save sector
	JP	MAPSEC		;map the sector
;
;	error on param, display and prompt again
;
TERMIN	LD	A,ERR86		;terminated
;
POSTE	AND	7FH		;assure a return
	LD	B,A		;pass to B
POSTEB	LD	A,ERROR		;SVC #
	RST	SVC		;display error
;
EXIT	LD	SP,0		;reset stack
STACK	EQU	$-2
	XOR	A		;return ZERO
	RET
;
;	prompt for input params
;
ASKPAR	LD	HL,PMTMSG	;use * for prompt
	LD	DE,STRING	;input buffer
	LD	BC,1<8+79	;dsp len + key len
	LD	A,VIDKEY	;display & input keys
	RST	SVC		;do it
	JR	NZ,POSTE	;error, go!
	JR	C,TERMIN	;abort the program
	EX	DE,HL		;swap to HL=>input
	JR	INTPAR		;interpret params
;
	PAGE
;
;	$POSHL - position HL to first significant char
;
;	ENT	HL => input string
;
;	EXT	Z = terminator reached
;		NZ = HL => first character
;
	INC	HL		;bump pointer
POSHL	LD	A,(HL)		;fetch a char
	CP	' '		;separator?
	JR	Z,POSHL-1	;yes, go
	CP	','		;sep?
	JR	Z,POSHL-1	;yes, go
;
CKTERM	CP	13		;CR?
	RET	Z
	CP	';'		;embedded CR?
	RET	Z
	CP	3		;ETX?
	RET			;Z = yes
;
	PAGE
;
;	$DISPLAY - display string to video
;
;	ENT	HL => string to dislay (ETX terminates)
;
;	EXT	Z = OK
;		NZ = A = error code
;
DISPLAY	LD	A,(HL)		;fetch a character
	CP	3		;term?
	RET	Z		;yes, return Z
	CALL	VID		;output to video
	INC	HL		;bump pointer
	JR	DISPLAY		;continue
;
;	$VID - output char to display
;
;	ENT	A = char to display
;
;	EXT	Z = OK
;		NZ = A = error code
;
VID	PUSH	BC		;save it
	LD	B,A		;pass char
	LD	A,VDCHAR	;SVC #
	RST	SVC		;output byte
	JP	NZ,POSTE	;abort in error!
;
;	$PAUSE - check for hold action
;
PAUSE	LD	A,KBCHAR	;keyboard char
	RST	SVC		;fetch from keyboard
	LD	A,B		;fetch char
	JR	NZ,PAURET	;no key, return
;
	CP	3		;abort?
	JP	Z,TERMIN	;exit program
	OR	A		;hold?
	JR	NZ,PAURET	;nope, return
;
PAUWT	LD	A,KBCHAR	;SVC #
	RST	SVC		;wait for a key
	JR	NZ,PAUWT	;wait for a key
	LD	A,B		;get the key
	CP	3
	JP	Z,TERMIN
	OR	A		;hold again?
	JR	NZ,PAUWT	;wait if not
PAURET	XOR	A		;clear error
PAUBAK	POP	BC		;restore stack
	RET			;back to caller
;
;##
	PAGE
;
;	$MAPDISK - map out a GAT sector
;
;	ENT	(DCB) = device number
;
;	EXT	GAT map displayed to video device
;
MAPDISK	LD	A,(DCB)		;get device flag
	AND	0FH		;get device number
	SUB	8		;adjust for 0-7
	LD	C,A		;give here for LOCDCT
	CALL	LOCDCT		;load IY with drive DCT
;
;	check for drive ready
;
	LD	A,DCHECK	;check drive
	RST	DISKIO		;ready?
	LD	A,ERR58		;not available
	JP	NZ,POSTE
;
;	read in the GAT table
;
	LD	E,0		;sector 0
	LD	B,E		;exended sector
	LD	HL,IOBUFF	;I/O buffer
	LD	A,SREAD		;system read
	RST	DISKIO		;read it
	JP	NZ,POSTE
;
	LD	B,(IY+15)	;get cylinder count
	LD	IX,IOBUFF	;start of GAT
	LD	D,0		;start track
;
DSK0	CALL	OUTTRK		;output track number
	LD	A,(IX+60H)	;get lockout byte
	EX	AF,AF'		;save it here
	LD	A,(IX)		;fetch GAT byte
	LD	C,(IY+20)	;grans/cylinder
	LD	E,0		;start sector
;
DSK1	CALL	DSKGRAN		;show a granule
	PUSH	AF		;save flags
	LD	A,(IY+19)	;sectors/gran
	ADD	A,E		;next sector
	LD	E,A		;put it back
	POP	AF		;restore flag
;
	DEC	C		;more grans this byte?
	JR	NZ,DSK1		;yes, go!
;
	INC	D		;bump relative track
	INC	IX		;bump GAT byte
	DJNZ	DSK0		;finish 'em all up
	JP	EXIT		;completed, return
;
;	$DSKGRAN - display granule allocation
;
;	ENT	D  = current track
;		E  = start sector this gran
;		A  = allocation table byte
;		A' = lockout table byte
;		IX => current track byte
;
;	EXT	sector range and alloc/lock stat displayd
;		Z = OK
;		NZ = error
;
DSKGRAN	EX	AF,AF'		;get lockout byte
	LD	(LCKBYTE),A	;save it
	EX	AF,AF'		;get alloc byte back
	PUSH	AF		;save alloc byte
	CALL	OUTRNG		;output gran sect range
	POP	AF		;get byte back
	RRA			;check allocated, used
	PUSH	AF		;save it
	LD	A,'.'		;available
	JR	NC,DG1		;yes, go
	LD	A,D		;get track
	CP	(IY+18)		;on directory?
	LD	A,'D'		;dir track
	JR	Z,DG1
	LD	A,'x'
DG1	CALL	VID		;output to video
	BIT	5,(IY+5)	;rigid?
	JR	NZ,DG2		;yes, show not locked
;
	LD	A,0		;get lockout byte
LCKBYTE	EQU	$-1
	RRA			;check it out
	LD	(LCKBYTE),A	;re-save it
	LD	A,'L'		;locked out
	JR	C,DG3		;yes, go
DG2	LD	A,' '		;available for use
DG3	CALL	VID		;output to video
	LD	A,' '		;gap between
	CALL	VID		;to video
	LD	A,(LCKBYTE)	;get lockout byte
	EX	AF,AF'		;return in alt
	POP	AF		;restore stack
	CP	A		;set ZERO
	RET			;back to caller
;
;	$OUTTRK - output current track number
;
;	ENT	D = track number
;
;	EXT	Z = OK
;		NZ = A = error code
;
OUTTRK	LD	A,13		;send C/R first
	CALL	VID		;to video
	LD	A,D		;get track number
	CALL	OUTNUM		;output numeric
	LD	A,' '		;send gap
	CALL	VID
	LD	A,'-'		;dash
	CALL	VID
	LD	A,' '		;another blank
	JP	VID		;done, go!
;
OUTRNG	LD	A,E		;fetch start sector
	CALL	OUTNUM		;output numeric field
	LD	A,'-'		;separator
	CALL	VID		;to video
	LD	A,(IY+19)	;sects/gran
	ADD	A,E		;add to start
	DEC	A		;less one for length
	CALL	OUTNUM		;output numeric
	LD	A,'='		;separator
	JP	VID		;output it!
;
;	$OUTNUM	- output numerical field to video
;
;	ENT	A = binary digit
;
;	EXT	3 digit binary number displayed
;		Z = OK
;		NZ = A = error code
;
OUTNUM	PUSH	BC		;save 'em
	CALL	ASCII		;convert to ACB ascii
	CP	'0'		;msb = '0'?
	JR	NZ,OUTN1	;nope, go!
	LD	A,' '		;else make it invisible
OUTN1	CALL	VID		;send to display
	LD	A,C		;fetch NSB
	CALL	VID		;display it
	LD	A,B		;fetch LSB
	CALL	VID		;display it
	POP	BC		;unstack
	RET			;return status
;
;	$ASCII	- convert binary to decimal ascii
;
;	ENT	A = binary to convert
;
;	EXT	ACB = decimal ascii
;
ASCII	LD	B,'0'		;start MSB digit
ASC1	SUB	100		;100's place
	JR	C,ASC2		;have MSB
	INC	B		;bump ascii
	JR	ASC1		;continue
;
ASC2	PUSH	BC		;MSB to stack
	ADD	A,100		;adjust it back
	LD	C,'0'		;start NSB
ASC3	SUB	10		;10's place
	JR	C,ASC4		;done, go!
	INC	C		;bump ascii
	JR	ASC3		;do more
;
ASC4	ADD	A,'0'+10	;last sub + ascii
	LD	B,A		;B = LSB
	POP	AF		;get MSB
	RET			;done, ACB = ascii
;
;	$HEXCV - convert binary to hex ascii
;
;	ENT	A = binary digit
;
;	EXT	CB = hex ascii
;
HEXCV	LD	B,A		;save it
	RRCA			;align to low bit
	RRCA
	RRCA
	RRCA
	CALL	HEXMAK		;make it hex ascii
	LD	C,A		;save msb
	LD	A,B		;get original back
	CALL	HEXMAK		;make it hex
	LD	B,A		;pass it here
	RET
HEXMAK	AND	0FH		;clear high bits
	ADD	A,'0'		;add ascii
	CP	'9'+1		;0-9?
	RET	C		;yes, return
	ADD	A,7		;make it A-F
	RET			;done
;
;	$FIGEXT - interpret extent element
;
;	ENT	HL => 2 byte extent pair
;
;	EXT	D  = starting track
;		E  = starting sector
;		B  = number of contiguous grans
;
FIGEXT	LD	D,(HL)		;fetch track
	INC	HL		;bump pointer
	LD	A,(HL)		;fetch extent byte
	RRCA			;align sect offset bits
	RRCA
	RRCA
	RRCA
	RRCA
	AND	7		;low 3 good only
	LD	B,A		;save here for looper
	LD	A,0		;start sector
	JR	Z,FIGLP4	;go if done
FIGLP3	ADD	A,(IY+19)	;sectors/gran
	DJNZ	FIGLP3		;go for gran offset
FIGLP4	LD	E,A		;pass to E
	LD	A,(HL)		;fetch again
	AND	1FH		;gran count
	INC	A		;adjust to actual
	LD	B,A		;return in B
	INC	HL		;HL => next pair
	RET			;done!
;
	PAGE
;
;	$MAPFILE - map out sectors of a file
;
MAPFILE	INC	DE		;bump to actual DCB
	LD	H,D		;pass to HL for display
	LD	L,E
	CALL	DISPLAY		;show filename
	LD	HL,OLIST	;list for OPEN
	LD	A,OPEN		;SVC #
	RST	SVC		;find the file
	JP	NZ,POSTE
;
	PUSH	DE		;pass DCB to IX
	POP	IX		;IX => dcb
	LD	C,(IX+16)	;get drive number
	CALL	LOCDCT		;load DCT into IY
;
	LD	A,(IX+17)	;get DEC
	CALL	HEXCV		;to hex ascii
	LD	(FMSGA),BC	;to the string
;
	LD	A,(IX+11)	;get EOF byte
	CALL	HEXCV		;to hex ascii
	LD	(FMSGB),BC	;to the string
;
	LD	A,(IX+15)	;get LRL
	CALL	ASCII		;to decimal ascii
	LD	(FMSGC),A	;to the string
	LD	(FMSGC+1),BC	;to the string
;
	LD	D,(IX+13)	;get NSB EOF sector
	LD	E,(IX+14)	;get LSB EOF sector
	LD	HL,FMSGD	;string
	LD	B,0		;command
	LD	A,BINDEC	;SVC #
	RST	SVC		;load the string
;
	LD	HL,FMSG		;string
	CALL	DISPLAY		;display it
;
;	begin mapping the allocated sectors
;
FIGLP	LD	B,(IX+17)	;get logical file number
FIGLP1	LD	C,(IX+16)	;get drive number
	CALL	DIRRED		;direct read
	JP	NZ,POSTE
	LD	DE,22		;offset to extents
	ADD	HL,DE		;HL => first extent
	LD	C,5		;max extents / entry
;
FIGLP2	LD	A,(HL)		;get extent byte
	CP	0FFH		;terminator?
	JP	Z,EXIT		;done!
	CP	0FEH		;extension?
	INC	HL		;bump extent specifier
	LD	B,(HL)		;fetch DEC if extension
	JR	Z,FIGLP1	;map the extension
	DEC	HL		;put it back
	CALL	FIGEXT		;compute extent
	CALL	MAPEXT		;map this extent
	DEC	C		;less extent counter
	JR	NZ,FIGLP2	;go more
	JP	EXIT		;corrupt!, terminate here
;
;	$MAPEXT - map out a files extent
;
;	EXT	D  = starting track of extent
;		E  = starting sector of extent
;		B  = number of contiguous granules
;
;	EXT	Z  = OK
;		NZ = A = error code
;
MAPEXT	LD	A,D		;get track
	CALL	OUTNUM		;to the video
	LD	A,','		;separator
	CALL	VID		;to the video
	CALL	OUTRNG		;output sector range
	LD	A,8		;backspace = off display
	CALL	VID		;to the display
	PUSH	BC		;save
	LD	B,5		;5 spaces
SNDSPC	LD	A,' '		;space
	CALL	VID		;to video
	DJNZ	SNDSPC
	LD	B,(IY+19)	;sectors/gran
ADVSC	CALL	NEXSEC
	DJNZ	ADVSC		;DE => next gran
	POP	BC		;get # grans left
	DJNZ	MAPEXT		;finish it off
	RET			;done
;
	PAGE
;
;	$MAPSEC - map out file assigned to sector
;
MAPSEC	LD	A,(DCB)		;get DCB specifier
	AND	0FH		;get drive number
	SUB	8		;adjust for 0-7
	LD	C,A		;give it here
	LD	(DRIVE),A	;save drive
	CALL	LOCDCT		;load IY with DCT
	LD	A,DCHECK	;drive ready?
	RST	DISKIO		;yes?
	LD	A,ERR58		;device not avail
	JP	NZ,POSTE
;
;	setup loop for locating the file
;
	LD	HL,IOBUFF	;I/O buffer
	LD	E,2		;start of file data
	LD	B,(IY+17)	;directory length
	DEC	B		;less GAT
	JR	DIRD		;read a sector
;
SECLP	BIT	4,(HL)		;active entry?
	CALL	NZ,THISONE	;this the one?
	JR	NZ,FNDFILE	;have the file, go!
;
;	advance to next entry
;
	LD	A,L		;fetch LSB buffer
	ADD	A,20H		;offset to next file
	LD	L,A		;put it back
	JR	NZ,SECLP	;finish this page
	INC	E		;bump the sector
DIRD	DEC	B		;any more left?
	JR	Z,SECNONE	;not assigned!
;
	PUSH	BC		;save it
	LD	C,0		;get drive
DRIVE	EQU	$-1
	LD	B,0		;sector extension
	LD	A,SREAD		;sector read
	RST	DISKIO		;read it in!
	POP	BC		;restore
	JP	NZ,POSTE
	JR	SECLP		;check the sector
;
SECNONE	LD	HL,NOTMSG	;not assigned
	CALL	DISPLAY		;display it
	JP	EXIT		;finished
;
FNDFILE	LD	A,L		;get LSB
	ADD	A,5		;point to name
	LD	L,A		;HL => name
	LD	DE,DCB		;put it here
	LD	B,2		;command
	LD	A,WILD		;SVC #
	RST	SVC		;combine name
	JP	NZ,POSTE
	EX	DE,HL		;HL => packed name
	CALL	DISPLAY		;display filename
	LD	DE,(RELSEC)	;get relative sector
	LD	HL,FILMSGA	;text area
	LD	B,0		;command
	LD	A,BINDEC	;SVC #
	RST	SVC		;load ascii into string
	LD	HL,FILMSG	;text
	CALL	DISPLAY		;display it
	JP	EXIT		;finished!
;
THISONE	PUSH	HL		;save needed
	PUSH	DE
	PUSH	BC
	LD	BC,0		;reset rel sector
	LD	(RELSEC),BC	;save it
;
THISRES	LD	DE,22		;offset to extents
	ADD	HL,DE		;HL => records
;
THIS1	LD	A,(HL)		;get extent byte
	CP	0FFH		;terminator?
	JR	Z,THISNOT	;not this one
	CP	0FEH		;extended entry?
	JR	Z,THISNXT	;next entry!
	CALL	FIGEXT		;setup data block
;
;	check for desired sector
;
THIS2	LD	C,(IY+19)	;sectors / gran
;
THIS3	CALL	TSECT		;this sector?
	JR	Z,THISYES	;yes!, return it
	CALL	NEXSEC		;bump sector
	DEC	C		;less this sector
	JR	NZ,THIS3	;finish the gran
	DJNZ	THIS2		;try next gran
;
THISNOT	XOR	A		;set Z (not it)
	JR	THISRET		;return
THISYES	OR	-1		;set NZ (yes!)
THISRET	POP	BC		;unstack & return
	POP	DE
	POP	HL
	RET
;
THISNXT	INC	HL		;bump pointer
	LD	A,(DRIVE)	;get drive
	LD	C,A		;need it here
	LD	B,(HL)		;get next LFN
	CALL	DIRRED		;read it!
	JP	NZ,POSTE	;error, abort!
	JR	THISRES		;resume
;
;	$NEXSEC - advance DE to next sector
;
NEXSEC	PUSH	BC		;save it
	LD	BC,0		;get relative sector
RELSEC	EQU	$-2
	INC	BC		;bump it
	LD	(RELSEC),BC	;re-save it
	POP	BC		;restore
	INC	E		;bump sector
	LD	A,E		;get it
	CP	(IY+21)		;end of track?
	RET	C		;nope, return
	LD	E,0		;reset to start
	INC	D		;bump track
	RET			;done!
;
;	$TSECT - compare DE to desired track sector
;
TSECT	PUSH	HL		;save it
	LD	HL,0		;get desired sector
SECTOR	EQU	$-2
TRACK	EQU	$-1
	OR	A		;clear carry flag
	SBC	HL,DE		;this the right one?
	POP	HL		;restore
	RET			;done
;
	PAGE
;
;	text area
;**
HELLO	DEFB	13		;linefeed
	DEFM	'MAP - '
	DEFM	'DosPLUS II Map Utility - '
	DEFM	'Series A.00'
	DEFB	13
	DEFM	'(c)(p) Copyright 1982 '
	DEFM	'by MicroPower, Inc.'
	DEFB	13
	DEFB	13
	DEFB	13
	DEFB	3
;
PMTMSG	DEFM	'*'
;
FMSG	DEFM	' - DEC='
FMSGA	DEFM	'xxH LRL='
FMSGC	DEFM	'xxx EOFB='
FMSGB	DEFM	'xxH EOFS='
FMSGD	DEFM	'xxxxx'
	DEFB	13
	DEFB	3
;
FILMSG	DEFM	' @ Relative Sector '
FILMSGA	DEFM	'xxxxx'
	DEFB	13
	DEFB	3
;
NOTMSG	DEFM	'Sector NOT ASSIGNED'
	DEFB	13
	DEFB	3
;
;	data area
;
OLIST	DEFW	IOBUFF		;BUFADR
	DEFW	IOBUFF		;RECADR
	DEFW	0000H		;EODAD
	DEFB	'R'		;read/write
	DEFB	0		;record length
	DEFB	'F'		;fixed/variable
	DEFB	0		;open, leave LRL
	DEFB	0		;terminator
;
DCB	DEFS	41		;41 byte DCB
STRING	DEFS	80		;keyboard input area
;
ZZZZZ	EQU	$
;
	END	ENTRY
