; offset/asm - kjw/bqsd - 08/82
;
	TITLE	'<Load File Offset Utility - Series A.0>'
;
SETBRK	EQU	03		;set break processor
EVAL	EQU	83		;evaluate user input
VIDKEY	EQU	12		;video / keyboard
ERROR	EQU	39		;display error message
OPEN	EQU	40		;open file
CLOSE	EQU	42		;close file
BINHEX	EQU	24		;binary > hex
BINDEC	EQU	21		;binary > decimal
GET	EQU	77		;read character
PUT	EQU	78		;write character
VDCHAR	EQU	08		;char to video
KBCHAR	EQU	04		;char from keyboard
FEXT	EQU	88		;add default extension
HLDKEY	EQU	29		;hold key processor
REWD	EQU	81		;rewind file
;
SVC	EQU	08H		;SVC executor
;
ERR19	EQU	19		;invalid filespec
ERR34	EQU	34		;load file format error
ERR82	EQU	82		;insufficient memory
ERR86	EQU	86		;program terminated
;
LD	EQU	0EDH		;1st byte LDIR/LDDR
IR	EQU	0B0H		;2nd byte LDIR
DR	EQU	0B8H		;2nd byte LDDR
LDHL	EQU	21H		;LD HL opcode
LDDE	EQU	11H		;LD DE opcode
LDBC	EQU	01H		;LD BC opcode
JUMP	EQU	0C3H		;JP opcode
PUBC	EQU	0C5H		;push BC
PUDE	EQU	0D5H		;push DE
PUHL	EQU	0E5H		;push HL
POBC	EQU	0C1H		;pop BC
PODE	EQU	0D1H		;pop DE
POHL	EQU	0E1H		;pop HL
ETX	EQU	03		;end of text marker
CR	EQU	13		;carriage return
;
LOWMEM	EQU	0171H		;bottom free memory
TOPMEM	EQU	0173H		;top free memory
IOBUFF	EQU	1A00H		;I/O buffer
STRING	EQU	IOBUFF		;dual use
;
	ORG	2200H		;high overlay area
;
ENTRY	LD	(STACK),SP	;save stack pointer
	PUSH	HL		;save input pointer
;
	LD	HL,0		;turn off break processor
	LD	A,SETBRK	;SVC #
	RST	SVC		;turn it off, get old
	LD	(OLDBRK),HL	;save old vector
;
	LD	B,0		;command
	LD	A,HLDKEY	;SVC #
	RST	SVC		;turn on hold processor
;
	LD	HL,HELLO	;sign on message
	CALL	LINE$		;display it
	POP	HL		;restore input pointer
;
;	evaluate user input
;
SCAN	LD	IX,EBLOCK	;eval block
	LD	A,EVAL		;SVC #
	RST	SVC		;fetch user input
	JR	NZ,POSTE	;error, abort!
;
;	see if anything
;
	LD	A,(IX)		;get flags
	AND	6		;source/dest
	JR	NZ,PERFORM	;yes, go!
;
;	prompt for user input
;
ASK	LD	HL,PROMPT	;prompt string
	LD	BC,1<8+79	;dsp len + key len
	LD	DE,STRING	;input buffer
	LD	A,VIDKEY	;SVC #
	RST	SVC		;fetch from user
	JR	NZ,POSTE	;error, go!
	EX	DE,HL		;HL => input
	JR	NC,SCAN		;check out user input!
;
;	error/exit vectors
;
TERMIN	LD	A,ERR86		;terminated
;
POSTE	AND	7FH		;assure a return
	LD	B,A		;pass error code
POSTEB	LD	A,ERROR		;SVC #
	RST	SVC		;display error message
;
EXIT	LD	HL,0		;get old break processor
OLDBRK	EQU	$-2
	LD	A,SETBRK	;SVC #
	RST	SVC		;re-install old vector
	LD	SP,0		;reset stack
STACK	EQU	$-2
	XOR	A		;return ZERO
	RET			;on to the next routine!
;
	PAGE
;
PERFORM	LD	DE,SDCB		;source DCB
	LD	A,(DE)		;get flags byte
	BIT	6,A		;filespec here?
	LD	A,ERR19		;invalid filespec
	JR	Z,POSTE		;no file, error!
;
	INC	DE		;to normal DCB
	LD	HL,CMDEXT	;default extension
	LD	A,FEXT		;SVC #
	RST	SVC		;add it!
;
	LD	HL,SLIST	;source param list
	LD	A,OPEN		;SVC #
	RST	SVC		;find the file
	JR	NZ,POSTE	;error, abort!
;
;	compute buffer start
;
	LD	DE,(LOWMEM)	;get low memory pointer
	LD	HL,ZZZZZ	;program end
	SBC	HL,DE		;compare pointers
	JR	C,HAVBUF	;DE = buffer
	LD	DE,ZZZZZ	;lowmem < pgmend
HAVBUF	LD	(BUFFER),DE	;save buffer start
	EX	DE,HL		;HL => buffer
;
;	load file into memory
;
LOADIT	CALL	GET$		;get a byte
	LD	C,A		;save the byte
	DEC	A		;01 load marker?
	JR	Z,LOAD1		;yes, go!
	DEC	A		;02 entry marker?
	JR	Z,LOAD2		;yes, go!
	DEC	A		;03 terminator?
	JR	Z,LOAD3		;yes, go!
	CP	1DH		;valid marker?
	LD	A,ERR34		;load file format error
	JR	NC,POSTE	;error, abort!
;
;	parse out the remark
;
	CALL	GET$		;get length byte
	LD	B,A		;pass length
	CALL	ROOM		;enough room to load it?
;
	LD	(HL),C		;insert marker
	INC	HL		;bump pointer
	LD	(HL),B		;insert length
	INC	HL		;bump pointer
;
LOADREM	CALL	GET$		;get a byte
	LD	(HL),A		;put in buffer
	INC	HL		;bump pointer
	DJNZ	LOADREM		;finish it off
	JR	LOADIT		;get next block
;
;	load block
;
LOAD1	CALL	GET$		;get length byte
	LD	B,A		;pass length
	CALL	ROOM		;enough room to load it?
	CALL	GETADDR		;get address from file
;
	LD	(HL),C		;insert header byte
	INC	HL
	LD	(HL),B		;insert length byte
	INC	HL
	LD	(HL),E		;insert LSB block address
	INC	HL
	LD	(HL),D		;insert MSB block address
	INC	HL
	CALL	CKLOW		;check for low block
	DEC	B		;less address
	DEC	B
	CALL	SHOWIT		;display block range
;
LOAD1L	CALL	GET$		;fetch a byte
	LD	(HL),A		;insert in buffer
	INC	HL		;bump pointer
	INC	DE		;bump address
	DJNZ	LOAD1L		;finish the block
	CALL	CKHIGH		;check for high block
	JR	LOADIT		;get next block
;
;	entry point marker
;
LOAD2	LD	A,C		;get marker
	LD	(TYPE),A	;save it
	CALL	GET$		;get length byte
	CALL	GETADDR		;get address
	LD	(BEGIN),DE	;save transfer address
	LD	(OLDENT),DE	;save here too
	JR	LOADED		;file loaded!
;
GETADDR	CALL	GET$		;get a byte
	LD	E,A		;LSB address
	CALL	GET$		;get another
	LD	D,A		;MSB address
	RET			;done, DE = address
;
;	file terminator marker
;
LOAD3	LD	A,C		;fetch type
	LD	(TYPE),A	;save it
;
;	file is completely loaded, display results
;
LOADED	LD	(BUFFEND),HL	;save end of buffer
	LD	DE,(LOW)	;get low address
	LD	HL,LOWM		;text to load it
	LD	B,0		;command
	LD	A,BINHEX	;SVC #
	RST	SVC		;convert to HEX
;
	LD	DE,(HIGH)	;get high address
	DEC	DE		;adjust to actual
	LD	HL,HIGHM	;text where it goes
	LD	B,A		;command
	LD	A,BINHEX	;SVC #
	RST	SVC		;convert to HEX
;
	LD	DE,(BYTES)	;size of file
	LD	HL,BYTESM	;text where it goes
	LD	B,A		;command
	LD	A,BINDEC	;SVC #
	RST	SVC		;convert to DECIMAL
;
	LD	HL,WHERE	;start of message
	CALL	LINE$		;display it
	LD	A,(TYPE)	;get type of file
	CP	3		;load only?
	JR	Z,SKIPXF	;yes, continue
;
	LD	DE,(BEGIN)	;transfer address
	LD	HL,XFERM	;text where it goes
	LD	B,0		;command
	LD	A,BINHEX	;SVC #
	RST	SVC		;load the string
	LD	HL,XFER		;beginning of text
	CALL	LINE$		;display it
;
SKIPXF	LD	A,(REAPAR)	;get READ param
	INC	A		;read only ?
	JP	Z,EXIT		;yes, done!
;
;	see if 2 filespec's given
;
	LD	A,(EBLOCK)	;get flags from $EVAL
	AND	6		;source/dest fields
	CP	6		;both here?
	JR	NZ,SINGLE	;nope, single file
	LD	HL,DDCB		;get dest DCB
	LD	A,(HL)		;get flags
	BIT	6,A		;filespec here?
	LD	A,ERR19		;invalid filespec
	JP	Z,POSTE		;go error if not
	LD	DE,SDCB		;source DCB
	LD	BC,41		;length of DCB
	LDIR			;move it over
	LD	DE,SDCB+1	;point to source
	LD	HL,CMDEXT	;/CMD extension
	LD	A,FEXT		;SVC #
	RST	SVC		;add default
	LD	HL,DLIST	;param list for $OPEN
	LD	A,OPEN		;SVC #
	RST	SVC		;open the file
	JP	NZ,POSTE	;error!
	JR	SAVEIT		;write back the file
;
;	same file, just rewind to beginning
;
SINGLE	LD	DE,SDCB+1	;point to DCB
	LD	A,REWD		;SVC #
	RST	SVC		;reset to beginning
	JP	NZ,POSTE	;error, abort!
;
;	file is open and ready to go
;
SAVEIT	LD	HL,(OFFPAR)	;get offset param
	LD	A,H		;anything input?
	OR	L		;any bits on?
	JR	Z,SAVER		;no input, go!
;
	LD	DE,(LOW)	;get low address
	SBC	HL,DE		;HL = offset param
;
SAVER	LD	(OFFSET),HL	;save offset
	LD	HL,(BUFFER)	;get data start
;
;	write data to file
;
SAVE	CALL	DONE		;end of buffer?
	JR	Z,PUTENT	;yes, insert terminator
	LD	A,(HL)		;get a byte
	INC	HL		;bump pointer
	CP	1		;load block?
	JR	Z,SAVE1		;yes, go!
;
;	write remark block
;
	CALL	PUT$		;write the marker
	LD	A,(HL)		;get length byte
	INC	HL		;bump pointer
	LD	B,A		;save count
	CALL	PUT$		;write the length
SAVREM	LD	A,(HL)		;get a byte
	INC	HL		;bump pointer
	CALL	PUT$		;write it out
	DJNZ	SAVREM		;finish the remark
	JR	SAVE		;go next block
;
;	write load block
;
SAVE1	CALL	PUT$		;insert marker in file
	LD	A,(HL)		;get length
	INC	HL		;bump pointer
	LD	B,A		;save it
	CALL	PUT$		;write it out
;
;	get block load address
;
	LD	E,(HL)		;get LSB
	INC	HL
	LD	D,(HL)		;get MSB
	INC	HL
	PUSH	HL		;save buffer pointer
	LD	HL,0		;get offset
OFFSET	EQU	$-2
	ADD	HL,DE		;HL = new address
	LD	A,L		;get LSB
	CALL	PUT$		;to the file
	LD	A,H		;get MSB
	CALL	PUT$		;to the file
	POP	HL		;get buff pointer back
	DEC	B		;less address length
	DEC	B
;
SAVLOAD	LD	A,(HL)		;get a byte
	INC	HL		;bump pointer
	CALL	PUT$		;to the file
	DJNZ	SAVLOAD		;finish the block
	JR	SAVE		;go next block
;
;	file done, insert appendage, entry point
;
PUTENT	LD	A,(APPPAR)	;get appendage param
	INC	A		;true?
	JP	NZ,PUTBEG	;nope, insert xfer mark
;
	LD	A,(TYPE)	;get file type
	CP	3		;load only?
	JP	Z,PUTLAST	;cannot have appendage!
;
	LD	A,01H		;insert load marker
	CALL	PUT$		;to the file
	LD	A,22		;appendage length
	CALL	PUT$		;to the file
	LD	DE,(HIGH)	;get highest address
	LD	(BEGIN),DE	;new entry point
	LD	HL,(OFFSET)	;get offset
	ADD	HL,DE		;HL = new entry
	LD	A,L		;get LSB
	CALL	PUT$		;where appendage loads
	LD	A,H		;get MSB
	CALL	PUT$		;to the file
	LD	A,PUHL		;PUSH HL
	CALL	PUT$
	LD	A,PUDE		;PUSH DE
	CALL	PUT$
	LD	A,PUBC		;PUSH BC
	CALL	PUT$
;
;	compute direction of move
;
	LD	HL,(HIGH)	;get high address
	LD	DE,(LOW)	;get low address
	OR	A		;clear carry
	SBC	HL,DE		;HL = data length
	LD	B,H		;pass to BC
	LD	C,L		;BC = file length
	LD	HL,(LOW)	;get low address
	PUSH	HL		;save it
	LD	DE,(OFFSET)	;get offset
	ADD	HL,DE		;HL = where it goes
	POP	DE		;DE = old load
	LD	A,IR		;2nd byte LDIR opcode
	OR	A		;clear carry
	PUSH	HL		;save start
	SBC	HL,DE		;compare old-new
	POP	HL		;restore it
	JR	NC,ADDAPP	;use LDIR to move it
;
	ADD	HL,BC		;add length to start
	DEC	HL		;HL => last byte
	EX	DE,HL		;HL = new address
	ADD	HL,BC		;add length to start
	DEC	HL		;HL => last byte
	EX	DE,HL		;HL=old end, DE=new end
	LD	A,DR		;last part LDDR opcode
;
ADDAPP	PUSH	AF		;save opcode
	LD	A,LDHL		;LD HL opcode
	CALL	PUT$		;to the file
	LD	A,L		;get LSB
	CALL	PUT$
	LD	A,H		;get MSB
	CALL	PUT$
;
	LD	A,LDDE		;LD DE opcode
	CALL	PUT$
	LD	A,E
	CALL	PUT$
	LD	A,D		;get MSB
	CALL	PUT$
;
	LD	A,LDBC		;LD BC opcode
	CALL	PUT$
	LD	A,C		;get LSB
	CALL	PUT$
	LD	A,B		;get MSB
	CALL	PUT$
;
	LD	A,LD		;1st byte LDIR/LDDR
	CALL	PUT$
	POP	AF		;get second half
	CALL	PUT$
;
	LD	A,POBC		;POP BC
	CALL	PUT$
	LD	A,PODE		;POP DE
	CALL	PUT$
	LD	A,POHL		;POP Hl
	CALL	PUT$
	LD	A,JUMP		;JP opcode
	CALL	PUT$
	LD	HL,(OLDENT)	;get old entry point
	LD	A,L		;get LSB
	CALL	PUT$
	LD	A,H		;get MSB
	CALL	PUT$
;
;	insert terminator/entry point
;
PUTBEG	LD	A,(TYPE)	;get file type
	CALL	PUT$		;write 02H
	LD	A,02H		;another one
	CALL	PUT$		;for entry length
	LD	DE,(BEGIN)	;get entry point
	LD	HL,(OFFSET)	;get offset
	ADD	HL,DE		;HL = new entry
	LD	A,L		;get LSB
	CALL	PUT$
	LD	A,H		;get MSB
PUTLAST	CALL	PUT$		;to the file
;
;	close it up
;
	LD	DE,SDCB+1	;point to DCB
	LD	A,CLOSE		;SVC #
	RST	SVC		;close it!
	JP	NZ,POSTE	;error!
	JP	EXIT		;done! exit program
;
	PAGE
;
GET$	PUSH	BC		;save
	PUSH	DE
	LD	DE,SDCB+1	;source DCB
	LD	A,GET		;SVC #
	RST	SVC		;fetch a byte
	JP	NZ,POSTE	;error, abort!
	LD	A,B		;get the byte
	POP	DE		;unstack
	POP	BC
	RET			;A = new byte
;
PUT$	PUSH	BC		;save
	PUSH	DE
	LD	DE,SDCB+1	;dest DCB
	LD	B,A		;pass the byte to write
	LD	A,PUT		;SVC #
	RST	SVC		;write to the file
	JP	NZ,POSTE	;error, abort!
	POP	DE		;unstack
	POP	BC
	RET			;byte in the file!
;
LINE$	LD	A,(HL)		;fetch a byte
	CP	ETX		;end of string?
	RET	Z		;yes, done!
	CALL	DSP$		;display it
	INC	HL		;bump pointer
	JR	LINE$		;get next one
;
DSP$	PUSH	BC		;save it
	LD	B,A		;pass the byte
	LD	A,VDCHAR	;SVC #
	RST	SVC		;display a byte
	JP	NZ,POSTE	;error!
	POP	BC		;restore it
	RET			;done!
;
;	check for end of buffer
;
DONE	PUSH	HL		;save it
	PUSH	DE		;this too
	LD	DE,(BUFFEND)	;end of buffer
	OR	A		;clear carry
	SBC	HL,DE		;compare
	POP	DE		;unstack
	POP	HL
	RET			;Z = done!
;
;	check for room to add new block
;
ROOM	PUSH	BC		;save it
	PUSH	HL		;save current address
	EXX			;alt set
	POP	DE		;address
	POP	BC		;length to check
	LD	C,B		;pass length
	LD	B,0		;BC = length
	LD	A,C		;check for 0=256
	OR	A		;zero?
	JR	NZ,ROOM1	;nope, go!
	INC	B		;bump by 256 bytes
ROOM1	INC	BC		;+1 for marker byte
	LD	HL,(TOPMEM)	;get top free memory
	OR	A		;clear carry
	SBC	HL,DE		;compare topmem to buff
	JR	C,IMEM		;out of memory
	SBC	HL,BC		;compare to new length
	EXX			;restore pointers
	RET	NC		;OK, return
;
IMEM	LD	A,ERR82		;insufficient memory
	JP	POSTE		;abort!
;
;	compare DE to lowest address so far
;
CKLOW	PUSH	HL		;save buff pointer
	LD	HL,(LOW)	;get lowest address
	OR	A		;clear carry
	SBC	HL,DE		;compare
	JR	C,LOWRET	;nope, continue
	LD	(LOW),DE	;save new lowest
LOWRET	POP	HL		;restore pointer
	RET
;
;	compare DE to highest address so far
;
CKHIGH	PUSH	HL		;save buff pointer
	LD	HL,(HIGH)	;get highest address
	OR	A		;clear carry
	SBC	HL,DE		;compare
	JR	NC,HIGHRET	;nope, continue
	LD	(HIGH),DE	;save new highest
HIGHRET	POP	HL		;restore pointer
	RET			;done
;
SHOWIT	PUSH	HL		;save buff pointer
	PUSH	DE		;save address
	PUSH	BC		;save count
	LD	C,B		;pass count
	LD	B,0		;BC = count
	LD	A,C		;check for 0
	OR	A
	JR	NZ,SHOW1	;nope, continue
	INC	B		;bump MSB
SHOW1	LD	HL,(BYTES)	;get byte count
	ADD	HL,BC		;add new block
	LD	(BYTES),HL	;save new count
	LD	A,(SHOPAR)	;get display param
	INC	A		;display block?
	JR	NZ,SHOW2	;nope, unstack & return
;
	PUSH	DE		;save block start
	EX	DE,HL		;HL = current block
	DEC	BC		;adjust for add
	ADD	HL,BC		;HL = end of block
	EX	DE,HL		;DE = block end
	LD	HL,TO		;message
	LD	B,0		;command
	LD	A,BINHEX	;SVC #
	RST	SVC		;convert it
	POP	DE		;get block start
	LD	HL,FROM		;text message
	LD	B,A		;command
	LD	A,BINHEX	;SVC #
	RST	SVC		;convert it
	LD	HL,BLOCKM	;message
	CALL	LINE$		;display it
;
	LD	A,KBCHAR	;check keyboard
	RST	SVC		;fetch a key
	JR	NZ,SHOW2	;nothing, continue
	LD	A,B		;get the key
	CP	3		;break?
	JP	Z,TERMIN	;abort program
	OR	A		;hold?
	JR	NZ,SHOW2	;nope, continue
;
PAUSE	LD	A,KBCHAR	;check keyboard
	RST	SVC		;fetch a key
	JR	NZ,PAUSE	;wait for one
	LD	A,B		;get the key
	CP	3		;break?
	JP	Z,TERMIN	;abort on break
	OR	A		;hold?
	JR	NZ,PAUSE	;wait for it!
;
SHOW2	POP	BC		;unstack 'em
	POP	DE
	POP	HL
	RET			;done!
;
	PAGE
;
;	text area
;
HELLO	DEFB	CR
	DEFM	'OFFSET - '
	DEFM	'DosPLUS II Load File Offset Utility - '
	DEFM	'Series A.00'
	DEFB	CR
	DEFM	'(c)(p) Copyright 1982 '
	DEFM	'by MicroPower, Inc.'
	DEFB	CR
	DEFB	CR
	DEFB	ETX
;
PROMPT	DEFM	'*'		;input prompt
;
WHERE	DEFB	CR
	DEFM	'Program loads from '
LOWM	DEFM	'xxxxH to '
HIGHM	DEFM	'xxxxH - '
BYTESM	DEFM	'xxxxx Load Bytes'
	DEFB	CR
	DEFB	ETX
;
XFER	DEFM	'Transfer Address is '
XFERM	DEFM	'xxxxH'
	DEFB	CR
	DEFB	ETX
;
BLOCKM	DEFM	'Block loads from '
FROM	DEFM	'xxxxH to '
TO 	DEFM	'xxxxH'
	DEFB	CR
	DEFB	ETX
;
CMDEXT	DEFM	'CMD'		;default extension
;
	PAGE
;
;	data area
;
EBLOCK	DEFB	0		;flag byte
	DEFW	SDCB		;source DCB
	DEFW	DDCB		;dest DCB
	DEFW	IOBUFF		;dummy for mask
	DEFW	PARAMS		;param list
;
PARAMS	DEFB	20H+3
	DEFW	SHOPAR
	DEFM	'SHOW'
;
	DEFB	20H+3
	DEFW	REAPAR
	DEFM	'READ'
;
	DEFB	40H+5
	DEFW	OFFPAR
	DEFM	'OFFSET'
;
	DEFB	20H+5
	DEFW	APPPAR
	DEFM	'APPEND'
;
	DEFB	0
;
SHOPAR	DEFW	0000H		;display blocks
REAPAR	DEFW	0000H		;read only
OFFPAR	DEFW	0000H		;offset address
APPPAR	DEFW	0000H		;add appendage
;
HIGH	DEFW	0001H		;highest address
LOW	DEFW	0FFFFH		;lowest address
BEGIN	DEFW	0000H		;entry point
BUFFER	DEFW	0000H		;buffer start
BUFFEND	DEFW	0000H		;buffer end
BYTES	DEFW	0000H		;byte count
TYPE	DEFB	0		;file type
OLDENT	DEFW	0		;old entry point
;
SDCB	DEFS	41		;source DCB
DDCB	DEFS	41		;dest DCB
;
SLIST	DEFW	IOBUFF
	DEFW	0
	DEFW	0
	DEFB	'W'
	DEFB	0
	DEFB	'F'
	DEFB	3
	DEFB	0
;
DLIST	DEFW	IOBUFF
	DEFW	0
	DEFW	0
	DEFB	'W'
	DEFB	0
	DEFB	'F'
	DEFB	2
	DEFB	0
;
ZZZZZ	EQU	$
;
	END	ENTRY
