; patch/asm - kjw/bqsd - 08/82
;
	TITLE	'<DosPLUS II.A - Patch Utility>'
;
	ORG	2200H		;high overlay area
;
VALUE	EQU	89		;fetch value from string
SETBRK	EQU	3		;break processor
VDLINE	EQU	9		;string to video
EVAL	EQU	83		;evaluate input line
ERROR	EQU	39		;display error message
VIDKEY	EQU	12		;video & keyboard
CLOSE	EQU	42		;close file
OPEN	EQU	40		;open file
BAKSP	EQU	80		;backspace file
GET	EQU	77		;fetch from file
PUT	EQU	78		;put to file
DIRRD	EQU	35		;direct file read
DIRWR	EQU	44		;direct file write
FEXT	EQU	88		;add default extension
;
SVC	EQU	08H		;SVC executor
OVLAY	EQU	18H		;overlay executor
CR	EQU	13		;carriage return
ETX	EQU	03		;end of text
;
ERR09	EQU	09		;invalid data!
ERR34	EQU	34		;load file format error
ERR47	EQU	47		;required param not found
ERR82	EQU	82		;insufficient memory
;
SBUFF	EQU	1A00H		;buffer for source
UCASE	EQU	031EH		;char to upper case
REGSAVE	EQU	0303H		;save registers
HIMEM	EQU	0173H		;top free memory
LOMEM	EQU	0171H		;bottom free memory
;
	PAGE
;
ENTRY	LD	(STACK),SP	;save stack pointer
	PUSH	HL		;save input pointer
;
	LD	HL,0		;disable break vector
	LD	A,SETBRK	;SVC #
	RST	SVC		;turn off/fetch old
	LD	(OLDBRK),HL	;save old vector
;
	LD	HL,HELLO	;sign on message
	LD	BC,HELLOL<8+0	;length + term
	LD	A,VDLINE	;SVC #
	RST	SVC		;display it
	JR	NZ,POSTE	;error already!
;
	POP	HL		;fetch input pointer
	XOR	A		;clear type flag
	LD	(FLAG),A
	LD	(PFLAG),A	;file positioned flag
;
;	evaluate user input
;
	LD	IX,EBLOCK	;eval block
	LD	A,EVAL		;SVC #
	RST	SVC		;evaluate user input
	JR	NZ,POSTE	;error, abort!
;
;	check for 512 bytes available memory
;
	LD	HL,(HIMEM)	;get high memory
	LD	DE,(LOMEM)	;get low memory
	OR	A		;clear carry
	SBC	HL,DE		;HL = free memory
	LD	DE,200H		;need 512 bytes
	LD	A,ERR82		;insufficient memory
	JP	C,POSTE		;go error!
	LD	HL,(HIMEM)	;get high memory back
	DEC	H		;less one page
	LD	(DBUFF),HL	;save dest buff
	DEC	H		;less one page
	LD	(SAVBUFF),HL	;save work buffer
;
;	check for source and dest input
;
	LD	A,(IX)		;get flags
	AND	6		;source/dest only
	LD	B,ERR47		;required param
	JR	Z,POSTEB	;error!
	CP	4		;source ONLY?
	JP	Z,USEKEY	;yes, fetch from keyboard
	JP	USEFILE		;no, fetch from file
;
;	error/exit vectors
;
POSTE	AND	7FH		;assure a return
	LD	B,A		;pass error code
POSTEB	LD	A,ERROR		;SVC #
	RST	SVC		;display error
;
EXIT	LD	HL,0		;get old break vector
OLDBRK	EQU	$-2
	LD	A,SETBRK	;SVC #
	RST	SVC		;put back old processor
;
	LD	SP,0		;reset stack
STACK	EQU	$-2
	XOR	A		;return ZERO
	RET			;done!
;
	PAGE
;
;	patch file using keyboard
;
USEKEY	CALL	SOPEN		;open the source file
	XOR	A		;set flag for KEY patch
	LD	(FFLAG),A	;save it
	JR	GETLINE		;go!
;
;	error on input command, display and continue
;
GETERR	AND	7FH		;assure a return
	LD	B,A		;pass the error code
	LD	A,ERROR		;SVC #
	RST	SVC		;display error
	JR	NZ,POSTE	;error! abort!
;
;	fetch a line from keyboard
;
GETLINE	LD	HL,PROMPT	;prompt string
	LD	DE,STRING	;input buffer for keys
	LD	BC,1<8+79	;display len + key len
	LD	A,VIDKEY	;SVC #
	RST	SVC		;fetch from keyboard
	JR	NZ,POSTE	;error!
	JR	C,CLOSER	;break, done, close it up
	EX	DE,HL		;HL => input
;
	CALL	INSTALL		;install current line
	JR	NZ,GETERR	;invalid data!
	JR	GETLINE		;get another
;
;	close up the file and insert entry point
;
CLOSER	LD	A,0		;get flag
FLAG	EQU	$-1
	OR	A		;anything?
	JR	Z,NOTHING	;nothing done!
;
	INC	A		;FF = direct patch?
	JR	Z,CLOSEIT	;yes, just close it
;
	LD	A,0		;entry marker
ENTMARK	EQU	$-1
	CP	3		;load only marker?
	JR	Z,CLOSE1	;yes, put single byte
	CALL	PUT$		;put in the file
	LD	A,02H		;entry address length
	CALL	PUT$		;put in the file
;
	LD	HL,0		;get program entry point
BEGIN	EQU	$-2
	LD	A,L		;get LSB
	CALL	PUT$		;put to the file
	LD	A,H		;get MSB
CLOSE1	CALL	PUT$		;write it out
;
;	close the file
;
	LD	DE,SDCB+1	;point to the DCB
	LD	A,CLOSE		;SVC #
	RST	SVC		;close it!
	JR	NZ,POSTE	;error, go!
;
CLOSEIT	LD	HL,BYE		;completed message
	LD	BC,BYEL<8+0	;length + term
SHOW	LD	A,VDLINE	;SVC #
	RST	SVC		;display message
	JR	NZ,POSTE	;error, go!
	JR	EXIT		;else done, return
;
;	nothing to patch with!
;
NOTHING	LD	HL,NIL		;nil, message
	LD	BC,NILL<8+0	;length + term
	JR	SHOW		;continue
;
;	open up the source file
;
SOPEN	LD	DE,SDCB+1	;source DCB
	LD	HL,CMDEXT	;/CMD extension
	LD	A,FEXT		;SVC #
	RST	SVC		;add default
	LD	HL,SLIST	;param list for source
	LD	A,OPEN		;SVC #
	RST	SVC		;open the file
	JP	NZ,POSTE	;error, abort
	RET
;
;	parse through the file and find the EOF marker
;
POSFILE	CALL	SGET$		;fetch a byte
	CP	01		;01 load marker?
	JP	Z,LOADBLK	;yes, go!
	CP	02		;02 entry marker?
	JP	Z,ENTBLK	;yes, go!
	CP	03		;03 entry marker?
	JP	Z,ENTRET	;return entry marker
	CP	07		;07 patch marker?
	JR	Z,PATBLK	;yes, go!
	CP	20H		;valid marker? (<20h)
	LD	A,ERR34		;load file format error
	JP	NC,POSTE	;invalid marker!
;
;	remark, fetch it
;
REMOVR	CALL	SGET$		;get a byte
	LD	B,A		;B = length of remark
;
REMBLK	CALL	SGET$		;get a byte
	DJNZ	REMBLK		;finish the remark
	JR	POSFILE		;get next block
;
PATBLK	LD	A,(PULPAR)	;is this a pull?
	INC	A		;yes?
	JR	NZ,REMOVR	;consider it a remark!
	LD	A,0		;get file flags
FFLAG	EQU	$-1
	INC	A		;data from file?
	JR	NZ,REMOVR	;nope, can't yank it!
;
;	save positioned DCB and partial buffer
;
	CALL	BACK$		;backspace the file
	LD	HL,SDCB+1	;open source DCB
	LD	DE,DDCB+1	;dest DCB
	LD	BC,40		;length
	LDIR			;save open file position
	LD	DE,(DBUFF)	;get I/O buffer to use
	LD	(DDCB+1+3),DE	;save user buffer
	LD	HL,SBUFF	;partial buffer
	LD	BC,100H		;length
	LDIR			;save buffer
;
	CALL	SGET$		;get marker again
	CALL	SGET$		;get length byte
	LD	B,A		;pass length (08)
	LD	HL,NAMSAVE	;saved patch name
;
CKPUL	CALL	SGET$		;get a byte
	CP	(HL)		;matching name?
	JR	NZ,REMBLK+3	;nope, remove as remark
	INC	HL		;bump pointer
	DJNZ	CKPUL		;finish it
;
;	block found! continue parsing till next found
;
PULLBLK	CALL	SGET$		;get a byte
	CP	02		;entry header?
	JR	Z,PUTPUL	;yes, finish writing
	CP	03		;load header?
	JR	Z,PUTPUL	;yes, finish writing
	CP	07		;another block?
	JR	Z,PUTPUL	;yes, finish it
	CP	20H		;valid marker?
FMTERR	LD	A,ERR34		;load file format error
	JP	NC,POSTE	;go error if out of range
	CALL	SGET$		;get length of block
	LD	B,A		;pass length
PUBLK	CALL	SGET$		;get a byte
	DJNZ	PUBLK		;continue
	JR	PULLBLK		;get next block
;
;	load block has been pulled, write remainder data
;
PUTPUL	PUSH	AF		;save marker
	LD	HL,SDCB+1	;source DCB
	LD	DE,DDCB+1	;dest DCB
	LD	B,40		;length of DCB
	CALL	SWAP		;swap 'em
	POP	AF		;get load marker
	JR	PULFIN+3	;continue
;
SWAP	LD	A,(HL)		;get a byte
	EX	AF,AF'		;save it
	LD	A,(DE)		;get another
	LD	(HL),A		;swap one
	EX	AF,AF'		;get HL
	LD	(DE),A		;swap other
	INC	HL		;bump pointers
	INC	DE
	DJNZ	SWAP		;finish 'em off
	RET			;done
;
PULFIN	CALL	DGET$		;get a dest byte
	CP	20H		;valid marker?
	JP	NC,FMTERR	;format error!
	CP	02		;terminator?
	JR	Z,PUL2		;yes, finish
	CP	03		;terminator?
	JR	Z,PUL3		;yes, finish
	CALL	PUT$		;write to source file
	CALL	DGET$		;get length byte
	LD	B,A		;save it
	CALL	PUT$		;write it out
PUTBK	CALL	DGET$		;get source byte
	CALL	PUT$		;write source byte
	DJNZ	PUTBK		;finish the block
	JR	PULFIN		;continue
;
PUL2	CALL	PUT$		;write to file
	CALL	DGET$		;get length (02)
	CALL	PUT$
	CALL	DGET$		;get lsb address
	CALL	PUT$
	CALL	DGET$		;get msb address
PUL3	CALL	PUT$		;write it out
;
;	close up the file
;
	LD	DE,SDCB+1	;open DCB
	LD	A,CLOSE		;SVC #
	RST	SVC		;close the file
	JP	NZ,POSTE	;go error!
;
	LD	HL,PULMSG	;patch pulled message
	LD	BC,PULMSGL<8+0	;length + term
	JP	SHOW		;display and terminate
;
;	load block, fetch it
;
LOADBLK	CALL	SGET$		;get length byte
	LD	B,A		;pass length
	CALL	GETADDR		;fetch address
	DEC	B		;less address length
	DEC	B
	JP	REMBLK		;parse through the block
;
;	entry block, fetch old entry address
;
ENTBLK	CALL	SGET$		;get length byte
	CALL	GETADDR		;fetch address
	LD	A,02H		;save term marker
	LD	(ENTMARK),A	;save marker
	LD	(BEGIN),HL	;save entry point
;
;	position the file back 4 bytes
;
	CALL	BACK$		;move it back a byte
	CALL	BACK$
	CALL	BACK$
	JR	SETPOS		;set file posit'd
;
;	end of file with return
;
ENTRET	LD	A,03H		;term marker
	LD	(ENTMARK),A	;save for file close
;
;	set file as positioned
;
SETPOS	LD	A,-1		;positioned flag
	LD	(PFLAG),A	;save & backspace
;
;	check for PULL param
;
	LD	A,(PULPAR)	;pull?
	INC	A		;yes?
	JR	Z,NOTPUL	;pull patch not found
;
	CALL	BACK$		;backspace the file
	LD	A,(FFLAG)	;get file flag
	INC	A		;patch from file?
	RET	NZ		;nope, return
;
;	insert 07 patch header
;
	LD	A,07		;header
	CALL	PUT$		;to the file
	LD	A,08		;length
	CALL	PUT$		;to the file
	LD	HL,NAMSAVE	;saved name
	LD	B,8		;8 chars to go
;
PUTNAM	LD	A,(HL)		;get a char
	CALL	PUT$		;to the file
	INC	HL		;bump pointer
	DJNZ	PUTNAM		;contine
	RET			;done, file positioned
;
NOTPUL	LD	HL,NPULMSG	;pull patch not found
	LD	BC,NPULMSGL<8+0	;length + term
	JP	SHOW		;abort, pull not found!
;
;	position the file back 1 byte
;
BACK$	LD	DE,SDCB+1	;open DCB
	LD	A,BAKSP		;SVC #
	RST	SVC		;back it up
	RET	Z		;OK, return
	JP	POSTE		;else post error & abort!
;
;	fetch address from file
;
GETADDR	CALL	SGET$		;get source byte
	LD	L,A		;LSB address
	CALL	SGET$		;get source byte
	LD	H,A		;MSB address
	RET			;done! HL = address
;
;	fetch a byte from the source file
;
SGET$	LD	DE,SDCB+1	;source DCB
	CALL	GET$		;get the byte
	RET	Z		;no error, return
	JP	POSTE		;else abort here!
;
;	fetch a byte from the patch file
;
DGET$	LD	DE,DDCB+1	;dest DCB
;
GET$	PUSH	BC		;save it
	LD	A,GET		;SVC #
	RST	SVC		;fetch the byte
	JR	NZ,ERRGET	;error!
	LD	A,B		;get the byte
ERRGET	POP	BC		;restore it
	RET			;return with A char
;
;	insert a byte into the source file
;
PUT$	PUSH	BC		;save it
	LD	DE,SDCB+1	;open DCB
	LD	B,A		;pass the byte
	LD	A,PUT		;SVC #
	RST	SVC		;write to the file
	POP	BC		;restore stack
	RET	Z		;OK, return
	JP	POSTE		;else abort in error
;
	PAGE
;
;	patch using a file
;
USEFILE	CALL	SOPEN		;open source file
	LD	A,-1		;set flag for FILE patch
	LD	(FFLAG),A	;save it
;
;	open up the patch input file
;
	LD	DE,DDCB+1	;point to the DCB
	LD	HL,PATEXT	;/PAT extension
	LD	A,FEXT		;SVC #
	RST	SVC		;add default
;
;	save patch file name in case of PULL
;
	PUSH	DE		;save DCB pointer
	LD	HL,NAMSAVE	;saver area for name
	LD	B,8		;8 chars max
;
NAMLP	LD	A,(DE)		;get a name byte
	CP	ETX		;terminator?
	JR	Z,NAMFIL	;yes, pad the rest
	CP	'/'		;extension?
	JR	Z,NAMFIL	;yes, pad it
	CP	'.'		;password?
	JR	Z,NAMFIL
	CP	':'		;drive
	JR	Z,NAMFIL
	CP	'('		;disk ID?
	JR	Z,NAMFIL
	LD	(HL),A		;save the char
	INC	DE		;bump pointers
	INC	HL
	DJNZ	NAMLP		;finish it off
	JR	NAMHAV		;done, go!
;
NAMFIL	LD	(HL),' '	;pad with spaces
	INC	HL		;bump pointer
	DJNZ	NAMFIL		;finish it
;
NAMHAV	POP	DE		;DE => DCB
;
;	check for PULL param
;
	LD	A,(PULPAR)	;get param
	INC	A		;pull it?
	JP	Z,POSFILE	;yes, go!
;
	LD	HL,DLIST	;param list for open
	LD	A,OPEN		;SVC #
	RST	SVC		;open the file
	JP	NZ,POSTE	;error, go!
;
LINEGET	CALL	FETCH		;fetch a line from file
	JP	NZ,POSTE	;error, go!
	JP	C,CLOSER	;done, close the file
;
	CALL	INSTALL		;install the patch
	JP	NZ,POSTE	;invalid file data!
	JR	LINEGET		;continue
;
;	fetch a line from a file
;
FETCH	LD	HL,STRING	;use string buffer
;
FETCHLP	CALL	DGET$		;get a byte
	JR	NZ,FETERR	;error! check it out
	LD	(HL),A		;else put char in buffer
	INC	HL		;bump pointer
	CP	CR		;end of line?
	JR	NZ,FETCHLP	;nope, go
;
	LD	HL,STRING	;point to input start
	RET			;done, HL => line
;
FETERR	CP	28		;end of file?
	SCF			;carry = yes
	RET	Z		;return OK
	CP	29		;record out of range?
	SCF			;carry = yes
	RET	Z		;yes, return OK
	OR	A		;else return NZ with err
	RET			;back to caller
;
	PAGE
;
;	install patch!
;
INSTALL	CALL	POSHL		;position to first char
	RET	Z		;nil, return
	CP	'.'		;remark?
	RET	Z		;yes, ignore it
;
;	check for type of patch
;
	CALL	UCASE		;make it upper case
	CP	'R'		;direct patch?
	JP	Z,DIRECT	;yes, go!
	CP	'A'		;address patch?
	JP	NZ,INVALID	;error!
;
;	type of patch is address dependent
;
	LD	A,(FLAG)	;get flag byte
	OR	A		;anything now?
	JR	Z,AOK		;nothing, OK
	INC	A		;direct patch already?
	JP	Z,INVALID	;can't mix 'em
;
AOK	CALL	EQUAL		;find = sign
	RET	NZ		;error, abort
;
	LD	A,VALUE		;fetch address value
	RST	SVC
	RET	NZ		;error, return
;
	LD	(ADDRESS),BC	;save load address
;
;	flag for address type patch
;
	LD	A,-2		;FE = address patch
	LD	(FLAG),A	;save it
;
;	find data pointer
;
GETDATA	CALL	POSHL		;any more input?
	JP	Z,INVALID	;error if not
	CALL	UCASE		;make it upper case
	CP	'D'		;data marker?
	JP	NZ,INVALID	;nope, error
;
	CALL	EQUAL		;find = sign
	RET	NZ		;error!
;
;	HL => patch to be made
;
	LD	DE,WORK		;work space
	LD	B,0		;length counter
	LD	A,(HL)		;get next char
	CP	'"'		;string?
	JR	Z,PATSTR	;yes, get it
	CP	27H		;string?
	JR	Z,PATSTR	;yes, get it
;
;	patch is numeric
;
NUMBER	CALL	POSHL		;position to next field
	JR	Z,PUTPAT	;done, insert patch
;
	CALL	HEXVAL		;get 2 digit HEX value
	RET	NZ		;invalid, go!
	EX	DE,HL		;HL => work buffer
;
	LD	(HL),C		;insert value
	INC	HL		;bump pointer
	EX	DE,HL		;put 'em back
	INC	B		;+ character
	JP	Z,INVALID	;too long!
	JR	NUMBER		;get next entry
;
;	patch is a string
;
PATSTR	LD	(STREND),A	;save string end
	INC	HL		;bump input pointer
;
STRLP	LD	A,(HL)		;get a character
	INC	HL		;bump pointer
	CP	0		;end list marker?
STREND	EQU	$-1
	JR	Z,PUTPAT	;yes, intall patch
	CP	CR		;end of line?
	JR	Z,PUTPAT	;yes, install patch
	LD	(DE),A		;else insert char to buff
	INC	DE		;bump pointer
	INC	B		;bump length counter
	JP	Z,INVALID	;too long!
	JR	STRLP		;go more!
;
;	install patch
;	patch data starts at $WORK
;	patch length in BC
;
PUTPAT	LD	A,B		;anything?
	OR	A		;zero?
	JP	Z,INVALID	;nil?
;
;	check for type of patch
;
	LD	A,(FLAG)	;get flag byte
	INC	A		;FF direct patch?
	JR	Z,PUTDIR	;yes, install it!
;
;	check that the file is positioned
;
	LD	A,0		;get position flag
PFLAG	EQU	$-1
	OR	A		;nil?
	PUSH	BC		;save it
	CALL	Z,POSFILE	;position the file
	POP	BC
;
;	insert load marker
;
	LD	A,01H		;load marker
	CALL	PUT$		;to the file
	LD	A,B		;get length
	ADD	A,2		;length + address
	CALL	PUT$		;to the file
	LD	HL,0		;get data address
ADDRESS	EQU	$-2
	LD	A,L		;get LSB
	CALL	PUT$		;to the file
	LD	A,H		;get MSB
	CALL	PUT$		;to the file
;
	LD	HL,WORK		;start of data buffer
;
PUTLP	LD	A,(HL)		;get a character
	CALL	PUT$		;write to the file
	INC	HL		;bump pointer
	DJNZ	PUTLP		;yes, finish it off
;
	XOR	A		;set Z flag for OK
	RET			;return ZERO
;
;	insert direct patch
;
PUTDIR	PUSH	BC		;save byte length
;
;	set sector operations
;
	LD	A,(SDCB+1+1)	;get flags 1
	AND	7FH		;turn off blocking
	LD	(SDCB+1+1),A	;put it back
	LD	BC,0		;get record to posit to
RECORD	EQU	$-2
	LD	DE,SDCB+1	;open DCB
	LD	A,DIRRD		;SVC #
	RST	SVC		;read in the sector
	JP	NZ,POSTE	;error!
;
;	sector is in buffer at $SBUFF
;
	LD	DE,WORK		;patch data here
	LD	HL,SBUFF	;point to data
	LD	L,0		;get byte offset
BYTE	EQU	$-1		;HL => patch start
	POP	BC		;fetch length
;
;	HL => data to patch
;	B  = data length
;	DE = source data
;
DIRELP	LD	A,(DE)		;get a byte
	INC	DE		;bump pointer
	LD	(HL),A		;put into buffer
	DEC	B		;less byte count
	JR	Z,DIRPUT	;done, write it back
	INC	L		;bump data pointer
	JR	NZ,DIRELP	;go more
;
;	end of sector reached, write partial/read next
;
	CALL	PARTIAL		;write partial, get next
	JR	DIRELP		;continue
;
DIRPUT	LD	BC,(RECORD)	;get record to write
	LD	DE,SDCB+1	;open DCB
	LD	A,DIRWR		;SVC #
	RST	SVC		;write it back!
	JP	NZ,POSTE	;error!
	RET			;done, patch installed!
;
;	write back current sector, read next one
;
PARTIAL	CALL	REGSAVE		;save registers!
;
	LD	BC,(RECORD)	;get record #
	LD	DE,SDCB+1	;open DCB
	LD	A,DIRWR		;SVC #
	RST	SVC		;write it back!
	JP	NZ,POSTE	;error!
;
;	read in next record
;
	INC	BC		;bump record
	LD	(RECORD),BC	;save for update
	LD	A,DIRRD		;SVC #
	RST	SVC		;read next record
	JP	NZ,POSTE	;error!
	RET			;else unstack & return
;
;	invalid data!
;
INVALID	LD	A,ERR09		;invalid data provided
	OR	A		;return NZ
	RET			;done!
;
	PAGE
;
;	direct patch
;
DIRECT	CALL	EQUAL		;find = sign
	RET	NZ		;error, return
;
;	fetch record # to patch
;
	LD	A,VALUE		;fetch record #
	RST	SVC
	RET	NZ		;error!
	LD	(RECORD),BC	;save record #
;
	CALL	POSHL		;position to next char
	JR	Z,INVALID	;nothing more, error!
;
;	fetch relative byte #
;
	CALL	UCASE		;make it upper case
	CP	'B'		;byte number?
	JR	NZ,INVALID	;invalid!
	CALL	EQUAL		;find = sign
	RET	NZ		;error!
;
	LD	A,VALUE		;get the value
	RST	SVC
	RET	NZ		;error!
	LD	A,B		;get MSB
	OR	A		;>255?
	JR	NZ,INVALID	;number is too big!
	LD	A,C		;get byte offset
	LD	(BYTE),A	;save it!
;
;	flag for direct patch
;
	LD	A,-1		;FF direct flag
	LD	(FLAG),A	;save it
	JP	GETDATA		;fetch data params!
;
;	position HL => next char after = sign
;
EQUAL	LD	A,(HL)		;get a character
	INC	HL		;bump pointer
	CP	CR		;terminator?
	JR	Z,INVALID	;missing data!
	CP	'='		;equal sign?
	RET	Z		;yes, done!
	JR	EQUAL		;else get next char
;
;	position HL to first significant character
;
	INC	HL		;bump pointer
POSHL	LD	A,(HL)		;get a character
	CP	' '		;separator?
	JR	Z,POSHL-1	;skip it
	CP	','		;separator?
	JR	Z,POSHL-1	;skip too
	CP	CR		;terminator?
	RET			;Z = end of string
;
;	get 2 digit HEX value from string
;
HEXVAL	LD	A,(HL)		;get a byte
	CALL	UCASE		;make it upper case
;
;	adjust to binary number
;
	CALL	ASC2BIN		;convert ascii to binary
	JR	C,INVALID	;invalid digit!
;
;	align to high nibble
;
	RLCA
	RLCA
	RLCA
	RLCA
	AND	0F0H		;A = MSB nibble
	LD	C,A		;save here
	INC	HL		;bump pointer
;
	LD	A,(HL)		;get LSB nibble
	CALL	ASC2BIN		;convert it
	JR	C,INVALID	;invalid digit!
	OR	C		;merge it
	LD	C,A		;C = value
	INC	HL		;bump pointer
	XOR	A		;return ZERO
	RET			;OK, return
;
;	convert ascii char in A to binary
;
ASC2BIN	CALL	UCASE		;make it upper case
	SUB	'0'		;remove ascii
	RET	C		;invalid digit!
	CP	10		;0-9?
	CCF			;reverse carry flag
	RET	NC		;OK, return
	SUB	7		;adjust for A-F
	RET	C		;error, return
	CP	16		;0-15?
	CCF			;reverse carry
	RET			;C = error
;
	PAGE
;
;	text for program
;
HELLO	DEFB	CR
	DEFM	'PATCH - '
	DEFM	'DosPLUS II Patch Utility - '
	DEFM	'Series A.00'
	DEFB	CR
	DEFM	'(c)(p) Copyright 1982 '
	DEFM	'by MicroPower, Inc.'
	DEFB	CR
	DEFB	CR
HELLOL	EQU	$-HELLO
;
BYE	DEFM	'Patch(s) Installed'
	DEFB	CR
BYEL	EQU	$-BYE
;
NIL	DEFM	'Nothing Done'
	DEFB	CR
NILL	EQU	$-NIL
;
PULMSG	DEFM	'Patch(s) Pulled'
	DEFB	CR
PULMSGL	EQU	$-PULMSG
;
NPULMSG	DEFM	'Patch NOT FOUND'
	DEFB	CR
NPULMSGL EQU	$-NPULMSG
;
PROMPT	DEFM	'*'		;prompt character
;
PATEXT	DEFM	'PAT'		;default extension
CMDEXT	DEFM	'CMD'		;default extension
;
;	data area
;
SDCB	DEFS	41		;source DCB
DDCB	DEFS	41		;dest DCB
NAMSAVE	DEFS	8		;storage for name
STRING	DEFS	79		;80 chars for input
	DEFB	CR		;forced terminator
;
EBLOCK	DEFB	0		;flags
	DEFW	SDCB		;source DCB
	DEFW	DDCB		;dest DCB
	DEFW	DDCB		;dest DCB (for USING)
	DEFW	PARAMS		;param block
;
PARAMS	DEFB	20H+3		;switch + length-1
	DEFW	PULPAR		;data vector
	DEFM	'PULL'		;remove patch
	DEFB	0		;terminator
;
PULPAR	DEFW	8080H		;param for PULL
SAVBUFF	DEFW	0		;save buffer pointer
;
;	open lists for files
;
SLIST	DEFW	SBUFF		;source buffer
	DEFW	0
	DEFW	0
	DEFB	'W'
	DEFB	1
	DEFB	'F'
	DEFB	3
	DEFB	0
;
DBUFF	EQU	$
DLIST	DEFW	0		;dest buffer
	DEFW	0
	DEFW	0
	DEFB	'R'
	DEFB	1
	DEFB	'F'
	DEFB	3
	DEFB	0
;
WORK	EQU	$		;work area
;
ZZZZZ	EQU	$		;program end
;
	END	ENTRY
