RTC    ; rtc/asm
 ;
 	TITLE	'<DOS+II - REAL TIME CLOCK SERVICE>'
 ;
 NMICODE PUSH	AF		;SAVE REGISTERS
 	PUSH	BC
 	PUSH	DE
 	PUSH	HL
 	PUSH	IX
 ;
 	IN	A,(RDRTC)	;CLEAR RTC
 	XOR	A		;INIT
 ;
 NMILOOP PUSH	AF		;SAVE COUNT
 	CALL	NMIEXEC 	;GO ROUTINE!
 	POP	AF		;GET COUNT
 	INC	A		;NEXT
 	AND	7		;DONE?
 	JR	NZ,NMILOOP	;TIL DONE
 ;
 	LD	HL,DATAK0	;DRIVE TIME OUT
 	OR	(HL)		;TIMED OUT?
 	JR	Z,NMIXIT	;IF YES
 	DEC	(HL)		;COUNT
 	JR	NZ,NMIXIT
 	LD	A,4FH		;SIDE 0 NO SEL
 	OUT	(DRSL),A	;DE-SEL DRIVE
 ;
 NMIXIT	POP	IX		;RESTORE PARTIAL
 	POP	HL
 	POP	DE
 	POP	BC
 	POP	AF
 	RETN			;GO DEBUG!
 ;
 ;	EXECUTE INTERRUPT TASK
 ;
 NMIEXEC LD	HL,RTCTBL	;HL => INT TABLE
 	ADD	A,A		;* 2
 	ADD	A,L		;+ TABLE
 	LD	L,A		;HL => TABLE ENTRY
 	LD	E,(HL)
 	INC	HL
 	LD	D,(HL)		;DE => INDIRECT ADD
 	PUSH	DE
 	POP	IX		;IX => INDIRECT ADD
 	EX	DE,HL		;HL => INDIRECT ADD
 	LD	E,(HL)
 	INC	HL
 	LD	D,(HL)		;DE => INT TASK
 	EX	DE,HL		;HL => INT TASK
 	JP	(HL)		;GO!
 ;
 ;	NILL INTERRUPT TASK
 ;
 NILL	DEFW	NULL		;ENTRY ADD
 NULL	RET
 ;
 	PAGE
 ;	$NMICTL - ADD/REMOVE INT TASKS
 ;
 ;	ENT	 B =	SET/RESET FLAAG
 ;		 C =	SLOT NUMBER 0-15
 ;		DE =>	USER ROUTINE
 ;
 NMICTL	CALL	REGSAV		;SAVE REGISTERS
 	INC	B
 	DEC	B		;REMOVE?
 	JR	NZ,SETNMI	;IF NOT
 ;
 RESNMI	LD	DE,NILL 	;DE => NILL ROUTINE
 ;
 SETNMI	LD	A,C		;GET SLOT
 	CP	8		;VALID?
 	JR	NC,NMIBAD	;invalid, go!
 	LD	HL,RTCTBL	;HL => INT TABLE
 	ADD	A,A
 	ADD	A,L
 	LD	L,A		;HL => TABLE ENTRY
 	CALL	NMIOFF		;DISABLE RTC
 	LD	(HL),E
 	INC	HL
 	LD	(HL),D		;SET VECTOR
 	CALL	NMION		;ENABLE RTC
 	XOR	A		;NO ERRORS
 	RET
 ;
 NMIBAD	LD	A,3		;PARAMETER ERROR
 	OR	A		;NZ STATUS
 	RET
 ;
 	PAGE
 ;	real time clock advance TASK
 ;
 TIMTASK DEFW	CLOCK0		;driver address
 ;
 CLOCK0	LD	HL,TIME$	;point to current time
 	LD	DE,TIMETBL	;max values
 	LD	B,4		;4 bytes for time
 CLOCK1	INC	(HL)		;bump the ticks
 	LD	A,(DE)		;get max
 	SUB	(HL)		;test it
 	RET	NZ		;not time to change yet
 	LD	(HL),A		;reset the value to 0
 	INC	L		;bump time pointer
 	INC	E		;bump max pointer
 	DJNZ	CLOCK1		;do all 4 time values
 ;
 ; optional date adjust for each 24 hour period
 ; leap years are automatically adjusted
 ;
 	PUSH	HL		;save day pointer
 	INC	L		;point to month
 	LD	A,(HL)		;get it
 	DEC	A		;make 1-12 into 0-11
 	ADD	A,E
 	LD	E,A		;(DE)=max days this month
 	INC	L		;point to year
 	LD	A,(HL)		;get the year
 	AND	3		;check for leap year
 	LD	A,28		;28 days hath february
 	JR	NZ,CLOCK3	;go, not leap year
 	INC	A		;else 29 days this year
 CLOCK3	LD	(MONTHS+1),A	;set FEB days
 	POP	HL		;now pointing to day
 	INC	(HL)		;bump the day
 	LD	A,(DE)		;get the max this month
 	SUB	(HL)		;compare to current #
 	RET	NC		;not end of month yet
 	LD	(HL),1		;set first day
 	INC	L		;point to month
 	INC	(HL)		;bump it
 	LD	A,(HL)		;get the month
 	CP	13		;at end of year yet?
 	RET	C		;nope, done
 	LD	(HL),1		;set month 1
 	INC	L		;point to year
 	INC	(HL)		;bump the year
 	RET			;completed
 ;
 	PAGE
 ;	real tile clock to display TASK
 ;
 CLKTASK DEFW	CLK0		;driver address
 	DEFB	30		;count
 ;
 CLK0	DEC	(IX+2)		;OK?
 	RET	NZ		;if not
 	LD	(IX+2),30	;1 per/sec
 	LD	HL,DATAK9	;video address
 	LD	DE,TIME$+3	;point to hour
 	LD	B,3		;3 loops
 	JR	CLK2		;go!
 ;
 CLK1	INC	HL		;next video slot
 	LD	(HL),':'        ;time separator
 	INC	HL		;next video slot
 ;
 CLK2	LD	A,(DE)		;get time byte
 	DEC	DE		;point to next
 	CALL	CLKASC		;quick ascii
 	LD	(HL),C		;put MSB
 	INC	HL		;bump pointer
 	LD	(HL),A		;put LSB ascii
 	DJNZ	CLK1		;more?
 	LD	DE,VIDEO+69+7	;video add
 	LD	BC,8		;count
 	JR	TOVIDEO 	;SKIP
 ;
 CLKASC	LD	C,'0'           ;start for MSB
 CLKA1	SUB	10		;less 10
 	JR	C,CLKA2 	;go if done
 	INC	C		;bump ascii digit
 	JR	CLKA1		;go again
 CLKA2	ADD	A,3AH		;make it ascii
 	RET			;done, CA = ascii
 ;
 	PAGE
 ;	real time alive to video TASK
 ;
 ALVTASK DEFW	ALV0		;driver address
 	DEFB	8		;count
 ;
 ALV0	DEC	(IX+2)		;OK?
 	RET	NZ		;if not
 	LD	(IX+2),8	;count
 	LD	A,(DDCB+9)	;video mask
 	OR	80H		;sel video
 	OUT	(LBANK),A	;do it!
 	LD	HL,VIDEO+78	;video location
 	LD	A,(HL)		;get char
 	CP	'#'             ;well?
 	LD	A,'*'           ;assume
 	JR	Z,ALV1		;skip
 	LD	A,'#'           ;other
 ALV1	LD	(HL),A		;to video
 	JR	NMION		;SKIP
 ;
 	PAGE
 ;	real time trace to display TASK
 ;
 TRCTASK DEFW	TRC0		;driver address
 ;
 TRC0	LD	HL,14		;SKIP REG
 	ADD	HL,SP		;HL => RET ADD
 	LD	C,(HL)
 	INC	HL
 	LD	B,(HL)		;DE => RET ADD
 	LD	HL,DATAK9	;temp storage
 	LD	A,B		;get MSB
 	CALL	B2H		;quick hex convert
 	LD	A,C		;get LSB
 	CALL	B2H		;put on video
 	DEC	HL		;bksp
 	LD	DE,VIDEO+64+3	;video address
 	LD	BC,4		;count
 ;
 TOVIDEO LD	A,(DDCB+9)	;video mask
 	OR	80H		;enable video
 	OUT	(LBANK),A	;do it!
 	LDDR			;to video
 ;
 ;	RESTORE RTC/VIDEO
 ;
 NMION	LD	A,(DDCB+9)	;video mask
 	OUT	(LBANK),A	;restore video
 	RET
 ;
 ;	DISABLE RTC (NMI)
 ;
 NMIOFF	LD	A,(DDCB+9)	;get video mask
 	AND	0DFH		;no RTC
 	OUT	(LBANK),A	;do it!
 	RET
 ;
 	PAGE
 ;	real time user timer TASK
 ;
 TMRTASK DEFW	TMR0		;driver address
 	DEFB	30		;count
 ;
 TMR0	DEC	(IX+2)		;OK?
 	RET	NZ		;if not
 	LD	(IX+2),30	;1 per/sec
 	LD	HL,(DATAK1)	;get seconds left
 	DEC	HL		;less one second
 	LD	(DATAK1),HL	;update the counter
 	LD	A,H		;see if time
 	OR	L
 	RET	NZ		;nope, return
 	LD	C,8		;C = slot
 	CALL	RESNMI		;turn it off
 	LD	HL,(DATAK2)	;get timer vector
 	JP	(HL)		;go!
 ;
 	PAGE
 ;	real time printer spooler TASK
 ;
 PRTTASK DW	PRTASK		;for vector
 ;
 PRTASK	LD	A,PDCB? 	;printer DCB number
 	CALL	LOCDCB		;find the DCB
 	BIT	5,(IX+5)	;OK?
 	RET	NZ		;do notin
 ;
 SPOOL	LD	B,20		;max chars/interrupt
 SPOOL1	LD	A,(IX+13)	;get counter
 	OR	(IX+14) 	;empty?
 	RET	Z		;return if empty
 	CALL	PRIN		;get status byte
 	JR	NZ,$+9		;not ready to go
 	CALL	BUFTAKE 	;take a char
 	LD	A,(HL)		;get the character
 	CALL	PROUT		;send the character
 	DJNZ	SPOOL1		;go for max chars
 	RET			;else return
 ;
 ; routine to send a char to printer
 ; ENTRY:  IX => DCB
 ;	  A  = character
 ; EXIT:   character sent directly to the device
 ;
 PROUT	BIT	7,(IX+5)	;serial?
 	JR	NZ,PROUTS	;go if yes
 	OUT	(0E1H),A	;send parallel
 	RET			;done
 ;
 PROUTS	OUT	(0F5H),A	;send character
 	RET
 ;
 ; routine to fetch printer status
 ; ENTRY:  IX => DCB
 ; EXIT:   Z  = printer can accept char
 ;	  NZ = not ready
 ;
 PRIN	BIT	7,(IX+5)	;serial?
 	JR	NZ,PRINS	;go if yes
 	IN	A,(0E0H)	;get parallel status
 	AND	0F0H		;top 4 bits only
 	RET			;Z = ready
 ;
 PRINS	LD	A,10H		;reset
 	OUT	(0F7H),A	;channel B command
 	IN	A,(0F7H)	;get status
 	CPL			;reverse bits
 	AND	4		;TX buffer empty?
 	RET			;Z = yes
 ;
 	PAGE
 ; common subroutine to position buffer pointers
 ;
 ;	BUFFTAKE - attempt to take char from buffer
 ;
 ;	ENT	IX => DCB
 ;
 ;	EXT	HL =>	take location
 ;		Z set chars available
 ;		offset incremented
 ;		counter decremented
 ;
 BUFTAKE LD	E,(IX+13)	;get counter
 	LD	D,(IX+14)
 	LD	A,D		;see if anything
 	OR	E
 	RET	Z		;Z = nil
 ;
 	DEC	DE		;else reduce it
 	LD	(IX+13),E	;update counter
 	LD	(IX+14),D
 	LD	L,(IX+9)	;get buffer pointer
 	LD	H,(IX+10)
 	LD	E,(IX+17)	;get taker offset
 	LD	D,(IX+18)
 	ADD	HL,DE		;HL => buff posit
 	CALL	INCBUFF 	;bump buffer pointer
 	LD	(IX+17),E	;update pointer
 	LD	(IX+18),D
 	OR	-1		;set NZ flag for OK
 	RET
 ;
 ;	BUFFADD - attempt to add char to buffer
 ;
 ;	ENT	IX => DCB
 ;
 ;	EXT	HL =>	add location
 ;		C set no room to add
 ;		offset incremented
 ;		counter incremented
 ;
 BUFFADD LD	L,(IX+11)	;get length
 	LD	H,(IX+12)
 	LD	E,(IX+13)	;get counter
 	LD	D,(IX+14)
 	INC	DE		;next
 	OR	A		;clr cry
 	SBC	HL,DE		;compare
 	RET	C		;no more room
 ;
 	LD	(IX+13),E	;update counter
 	LD	(IX+14),D
 	LD	L,(IX+9)	;get buffer start
 	LD	H,(IX+10)
 	LD	E,(IX+15)	;get adder offset
 	LD	D,(IX+16)
 	ADD	HL,DE		;HL => buff posit
 	CALL	INCBUFF 	;bump ring pointer
 	LD	(IX+15),E	;update pointer
 	LD	(IX+16),D
 	OR	A		;clear carry
 	RET			;done, HL => buffer
 ;
 ;	INCBUFF - increment ring buffer pointer
 ;
 ;	ENT	IX =>	DCB
 ;		DE =	ring location
 ;
 ;	EXT	DE =	new ring location
 ;
 INCBUFF INC	DE		;bump pointer
 	PUSH	HL		;save from test
 	LD	L,(IX+11)	;get max length
 	LD	H,(IX+12)
 	SCF			;for one adjust
 	SBC	HL,DE		;see if at end
 	POP	HL		;restore pointer
 	RET	NC		;not at end
 	LD	DE,0		;else start over
 	RET
 ;
